/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef GRAPHICS_H_
#define GRAPHICS_H_


#include <nn/gx.h>
#include <nw/ut.h>
#include <nw/gfx/gfx_IRenderTarget.h>

#include <nw/demo/demo_DisplayBufferSwapper.h>
#include <nw/demo/demo_GraphicsSystem.h>


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class Graphics
{
public:
    //
    Graphics( nw::os::IAllocator* allocator, const nw::demo::RenderSystem::Description& description );

    //
    ~Graphics();

    //
    //
    //
    //
    void SetRenderTarget( nw::gfx::IRenderTarget* renderTarget );


    //
    //
    //
    //
    nw::gfx::RenderContext*  GetRenderContext() { return this->m_RenderContext; }


    //
    //
    //
    //
    void ClearBuffer( nw::ut::FloatColor clearColor );


    //
    //
    //
    //
    void TransferBuffer(s32 screenKind);


    //
    void WaitCommandList();


    //
    //
    //
    //
    void WaitVSync(s32 screenKind);


    //
    //
    //
    //
    void SwapBuffer(s32 screenKind);

private:
    //
    static void DestroyInternal_();

    //
    NW_INLINE void CreateInternal_(const nw::demo::RenderSystem::Description& description);

    nw::os::IAllocator*                 m_Allocator;
    nw::gfx::RenderContext*             m_RenderContext;

    nw::demo::DisplayBufferSwapper*     m_UpperSwapper;
    nw::demo::DisplayBufferSwapper*     m_LowerSwapper;
    nw::demo::DisplayBufferSwapper*     m_ExtensionSwapper;

    GLuint                              m_CommandList;
    GLuint                              m_CachingCommandList;
};


#endif // GRAPHICS_H_
