/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_BUTTON_H_
#define SM_BUTTON_H_

#include "../include/SmPrimitive.h"
#include "../include/SmMessage.h"
#include "../include/SmRectCollision.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
//------------------------------------------------------------------------------
// Button class
class SmButton : public SmMessage
{
public:
    // Constructor
    SmButton( uint x, uint y, uint w, uint h );

    // Destructor
    ~SmButton();

    // Receive message
    virtual bool ReceveMessage( SmMessageType type, void* object, uint targetId );

    // Rendering
    virtual void Render();

    // Set button status.
    void SetSwitch( bool bswitch );

    // Swap button status.
    void SwapSwitch();

    // Get switch on/off status.
    bool IsSwitch()
    {
        return m_IsSwitch;
    }

    // Set the button color.
    void SetColor( nw::ut::Color8 onColor, nw::ut::Color8 offColor );

private:
    Sm2DPrimPC*         m_Button;
    Sm2DPrimPC*         m_PushButton;

    u16                 m_LastX;
    u16                 m_LastY;

    SmRectCollision     m_Collision;

    bool                m_IsPressed;
    bool                m_IsGrab;
    bool                m_IsSwitch;

    nw::ut::Color8      m_OnColor;
    nw::ut::Color8      m_OffColor;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif




#endif  // SM_BUTTON_H_
