/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_LYT_BUTTON_H_
#define SM_LYT_BUTTON_H_

//#include <nw/demo.h>

#include "../include/SmLayout.h"
#include "../include/SmMessage.h"
#include "../include/SmRectCollision.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
//------------------------------------------------------------------------------
// Button class
class SmLytButton : public SmMessage
{
public:
    // Constructor
    SmLytButton( SmLayout* smLayout, const wchar_t* string, const char* bounding );

    // Destructor
    ~SmLytButton();

    // Receive message
    virtual bool ReceveMessage( SmMessageType type, void* object, uint targetId );

    // Rendering
    virtual void Render();

    // Set button status.
    void SetSwitch( bool bswitch );

    // Swap button status.
    void SwapSwitch();

    // Get switch on/off status.
    bool IsSwitch()
    {
        return m_IsSwitch;
    }

    /*
        Positioning features
    */
    // position
    void SetPosition( f32 x, f32 y );

    // Toggle visibility.
    virtual void SetVisible( bool visible )
    {
        // Toggle layout visibility.
        m_Layout->SetVisible( visible );
    }


private:
    SmLayout*           m_Layout;

    u16                 m_LastX;
    u16                 m_LastY;

    SmRectCollision     m_Collision;

    wchar_t*            m_String;

    bool                m_IsPressed;
    bool                m_IsGrab;
    bool                m_IsSwitch;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif




#endif  // SM_LYT_BUTTON_H_
