/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef SM_RENDER_SYSTEM_H_
#define SM_RENDER_SYSTEM_H_

#include "../include/SmBase.h"
#include <nw/demo/demo_DisplayBufferSwapper.h>
#include <nw/gfx/gfx_IRenderTarget.h>
#include <nw/gfx/gfx_RenderQueue.h>
#include <nw/ut/ut_MoveArray.h>
#include <nw/ut/ut_ResUtil.h>
#include <nw/math.h>
#include <nw/dev/dev_CallbackChain.h>
#include <nn/gx.h>
#include <nn/ulcd/CTR/ulcd_StereoCamera.h>
#include <nw/demo/demo_GraphicsMemoryAllocator.h>
#include <nw/gfx.h>


#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class SmRenderSystem : public SmBase
{
public:
    enum CommandBufferSide
    {
        COMMAND_BUFFER_SIDE_0 = 0,
        COMMAND_BUFFER_SIDE_1 = 1,
        COMMAND_BUFFER_SIDE_MAX = 2
    };

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    struct Description
    {
        //
        struct ScreenSize
        {
            s32 width;  //
            s32 height; //
            //
            ScreenSize(s32 w, s32 h) : width(w), height(h) {}
        };

        //
        struct ScreenOffset
        {
            s32 x;  //
            s32 y;  //
            //
            ScreenOffset(s32 x, s32 y) : x(x), y(y) {}
        };
        //
        nw::demo::DisplayBufferSwapper::Description upperScreenDescription;

        //
        nw::demo::DisplayBufferSwapper::Description lowerScreenDescription;

        nw::gfx::RenderColorFormat renderColorFormat;   //
        size_t displayBufferCount;                      //
        size_t maxGpuProfilingEntryCount;               //

        //
        //
        Description()
        : renderColorFormat(nw::gfx::RENDER_COLOR_FORMAT_RGBA8),
          displayBufferCount(2),
          maxGpuProfilingEntryCount(8)
        {
            upperScreenDescription.screenKind = nw::demo::UPPER_SCREEN;
            upperScreenDescription.width  = NN_GX_DISPLAY0_HEIGHT;
            upperScreenDescription.height = NN_GX_DISPLAY0_WIDTH;

            lowerScreenDescription.screenKind = nw::demo::LOWER_SCREEN;
            lowerScreenDescription.width  = NN_GX_DISPLAY1_HEIGHT;
            lowerScreenDescription.height = NN_GX_DISPLAY1_WIDTH;
        }
    };


    //----------------------------------------
    // Constructor.
    SmRenderSystem( const Description& description );

    //----------------------------------------
    // Destructor.
    ~SmRenderSystem();

    //----------------------------------------
    // Sets the render target.
    void SetRenderTarget( nw::gfx::IRenderTarget* renderTarget );

    //----------------------------------------
    // Deletes the buffer.
    void ClearBuffer( nw::ut::FloatColor clearColor );

    //----------------------------------------
    // Transfers the buffer.
    void MakeTransferBufferCommand(s32 screenKind, bool withSwappingBuffer);

    //----------------------------------------
    // Initiates a Vsync wait.
    void WaitVSync(s32 screenKind);

    //----------------------------------------
    // Reserves a buffer swap.
    void SwapBuffer( s32 screenKind );

    //----------------------------------------
    // Gets RenderContext.
    nw::gfx::RenderContext* GetRenderContext() { return m_RenderContext; }

    // Parallax calculation
    void CalcStereoCamera(
        nw::gfx::Camera* leftCamera,
        nw::gfx::Camera* rightCamera,
        nw::gfx::Camera* baseCamera,
        f32 depthLevel = 1.0f,
        f32 depthRange = 1.0f);

    GLuint GetCacheCmdListUpper(CommandBufferSide side)  { return m_CacheCmdListUpper[side]; }
    GLuint GetCmdList(CommandBufferSide side) { return m_CmdList[side]; }

private:
    NW_INLINE void CreateInternal_();

    nw::gfx::RenderContext*             m_RenderContext;
    nw::demo::DisplayBufferSwapper*     m_UpperSwapper;
    nw::demo::DisplayBufferSwapper*     m_LowerSwapper;
    nw::demo::DisplayBufferSwapper*     m_ExtensionSwapper;

    GLuint                              m_CacheCmdListUpper[COMMAND_BUFFER_SIDE_MAX];
    GLuint                              m_CmdList[COMMAND_BUFFER_SIDE_MAX];

    nn::ulcd::CTR::StereoCamera*        m_StereoCamera;

    nw::gfx::IRenderTarget*             m_RenderTarget;
};


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void SmInitializeGraphicsSystem(nw::demo::DemoAllocator* deviceAllocator);

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
void SmFinalizeGraphicsSystem(nw::demo::DemoAllocator* deviceAllocator);
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif // SM_RENDER_SYSTEM_H_
