/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_SLIDERBAR_H_
#define SM_SLIDERBAR_H_

#include <nw/demo.h>

#include "../include/SmPrimitive.h"
#include "../include/SmMessage.h"
#include "../include/SmRectCollision.h"
#include "../include/SmCommandUtility.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
// Class for displaying slider bars.
class SmSliderBar : public SmMessage
{
public:
    // Type
    enum
    {
        SM_SLIDER_BAR_VERTICAL = 0,     // Vertical.
        SM_SLIDER_BAR_HORAIZON = 1      // Horizontal.
    };
    typedef u32 SmSliderBarType;

    // Constructor
    SmSliderBar( uint x,
                 uint y,
                 uint length,
                 SmSliderBarType type );

    // Destructor
    ~SmSliderBar();

    // Receive message
    virtual bool ReceveMessage( SmMessageType type, void* object, uint targetId );

    // Rendering
    virtual void Render();

    // 0.f ~ 1.f
    f32 GetSliderParam()
    {
//        return m_OffsetY;
        return static_cast<f32>(m_OffsetY)/m_Length;
    }

private:
    SmSliderBarType     m_SliderType;

    Sm2DPrimPC*         m_Bar;
    Sm2DPrimPC*         m_Slider;

    SmRectCollision     m_Collision;

    uint                m_Length;
    bool                m_IsGrab;
    s32                 m_BaseX;
    s32                 m_BaseY;
    s32                 m_GrabX;
    s32                 m_GrabY;
    s32                 m_OffsetX;
    s32                 m_OffsetY;
    s32                 m_PreOffsetX;
    s32                 m_PreOffsetY;

    SmCommandReuser     m_BarCmd;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif




#endif  // SM_SLIDERBAR_H_
