/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef SM_TOUCHPANEL_CAMERA_H_
#define SM_TOUCHPANEL_CAMERA_H_

#include <nw/os.h>
#include <nw/gfx.h>
#include "../include/SmMessage.h"
#include "../include/SmCamera.h"
#include "../include/SmButton.h"
#include "../include/SmRectCollision.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
//------------------------------------------------------------------------------
// Touch panel camera class.
class SmTouchPanelCamera : public SmMessage
{
    enum
    {
        SM_TOUCH_PANEL_CAMERA_SLIDER = 0
    };

public:
    // Constructor
    SmTouchPanelCamera( SmCamera* smcamera );

    // Destructor
    ~SmTouchPanelCamera(){}

    // Receive message
    virtual bool ReceveMessage( SmMessageType type, void* object, uint targetId );

    // Rendering
    virtual void Render();

    // Valid/invalid
    virtual void SetEnable( bool enable )
    {
        SmMessage::SetEnable( enable );

        // Inherit m_Camera status.
         if ( GetEnable() )
        {
            // Camera viewpoint
            m_SmCamera->GetPosition( &m_CameraPos );

            // Camera viewpoint
            m_SmCamera->GetTarget( &m_CameraTarget );

            // Camera orientation.
            m_CameraToward.x = m_CameraTarget.x - m_CameraPos.x;
            m_CameraToward.y = m_CameraTarget.y - m_CameraPos.y;
            m_CameraToward.z = m_CameraTarget.z - m_CameraPos.z;
            m_CameraToward.Normalize();

            // Initialize left-right pan values.
            m_DegX = m_DegY = 0.f;
        }
    }

private:
    void calcCamera();
    void updateCamera();

private:
    SmCamera*           m_SmCamera;

    nw::math::VEC3      m_CameraPos;
    nw::math::VEC3      m_CameraTarget;
    nw::math::VEC3      m_CameraToward; // Direction

    SmRectCollision     m_Collision;

    bool                m_IsGrab;

    s32                 m_TouchPanelPosLastX;
    s32                 m_TouchPanelPosLastY;
    s32                 m_TouchPanelPosX;
    s32                 m_TouchPanelPosY;
    f32                 m_DegX;
    f32                 m_DegY;
};
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif  // SM_TOUCHPANEL_CAMERA_H_
