/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#include <ctype.h>
#include <stdlib.h>
#include "../include/SmFile.h"
#include "../include/SmInitialFileParser.h"


namespace
{

/*
  
 */
SmInitialFileParser::SmInitialFileParser()
: m_Top( NULL ),
  m_Current( NULL )
{

}


/*
  
 */
SmInitialFileParser::~SmInitialFileParser()
{
    m_File.Release();
}


/*
  
 */
bool
SmInitialFileParser::Load( const wchar_t* configFileName )
{
    m_File.Read( configFileName, m_Allocator );

    // todo:cast
//    m_Top = static_cast<char *>(m_File.Buffer());
    m_Top = (char *)(m_File.Buffer());
    m_Current = m_Top;

    // Generates SmCmd command tree.
    _createCmdTree();
    return true;
}


/*
  
 */
void
SmInitialFileParser::_createCmdTree()
{
    const char charCmdBegin     = 0x3C; // <
    const char charCmdEnd       = 0x3E; // >
    const char charCmdSlash     = 0x2F; // /
    const char charCmdSpace     = 0x20; // " "
    const char charCmdComma     = 0x2C; // ,
    const char charCmdEqual     = 0x3D; // =

    char* curChar = m_Current;
    char strTemp[32];

    u32 cmdReadingState = 0;
    u32 readCharNum = 0;
    u32 readDataNum = 0;

    SmCmd* addCmd;

    while( 1 )
    {
        // Look for the < of a command.
        if ( *curChar == charCmdBegin )
        {
            cmdReadingState = 1;

            // Generate 1 command.
            addCmd = new SmCmd();
            m_CmdArray.push_back( addCmd );

            // Load element.
            ++curChar;
            while( 1 )
            {
                strTemp[readCharNum] = *curChar;
                ++curChar;
                ++readCharNum;

                if ( *curChar == charCmdSpace )
                {
                    strTemp[readCharNum] = '\0';
                    addCmd->m_Command = strTemp;
                    break;
                }
            }
        }

        // Interpret data part represented by xxxxx,
        if ( cmdReadingState )
        {
            u32 state = 0;
            readCharNum = 0;

            while( 1 )
            {
                switch ( state )
                {
                // Load name part.
                case 0:
                    {
                        while( *curChar == charCmdSpace ) ++curChar;

                        strTemp[readCharNum] = *curChar;
                        ++curChar;
                        ++readCharNum;

                        if ( *curChar == charCmdEqual )
                        {
                            state = 1;
                            ++curChar;

                            strTemp[readCharNum] = '\0';
                            addCmd->m_Data[readDataNum].name = strTemp;

                            readCharNum = 0;
                        }
                    }
                    break;

                // Load param part.
                case 1:
                    {
                        while( *curChar == charCmdSpace ) ++curChar;

                        strTemp[readCharNum] = *curChar;
                        ++curChar;
                        ++readCharNum;

                        // Detect the last character of a command here.
                        if ( *curChar == charCmdComma ||
                            *curChar == charCmdSlash ||
                            *curChar == charCmdEnd )
                        {
                            state = 1;
                            ++curChar;

                            strTemp[readCharNum] = '\0';
                            addCmd->m_Data[readDataNum].param.SetParam( strTemp );

                            ++readDataNum;
                            readCharNum = 0;
                            state = 0;

                             if ( *curChar == charCmdSlash || *curChar == charCmdEnd )
                            {
                                state = 3;
                            }
                        }
                    }
                    break;
                }

                 if ( state == 3 )
                {
                    // Initialization executed when loading a command ends.
                    readDataNum = 0;
                    readCharNum = 0;
                    cmdReadingState = 0;
                    break;
                }
            } // while(1)
        } //  if ( cmdReadingState )

        // Advance through white space.
        //while( *curChar == charCmdSpace )
        u32 readSize = curChar - m_Top;
        if ( readSize < m_File.Size() )
        {
            ++curChar;
        }
        else
        {
            break;
        }
    }
}


} // Namespace


