/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#include <nw/demo/demo_Utility.h>
#include "../include/demo.h"
#include "../include/SmDef.h"
#include "../include/SmInitialFileParser.h"

namespace
{

//----------------------------------------
// Constructor
Demo::Demo( GfxCtrl* gfxCtrl, LytCtrl* lytCtrl )
: SmMessage( NULL ),
  m_GfxCtrl( gfxCtrl ),
  m_LytCtrl( lytCtrl ),
  m_Pierrot( NULL ),
  m_Robot( NULL ),
  m_Stage( NULL ),
  m_Stick( NULL ),
  m_Audience( NULL ),
  m_Empyrean( NULL ),
  m_LytTabPanel( NULL ),
  m_Tab0VisibleBtn( NULL ),
  m_Tab1VisibleBtn( NULL ),
  m_Tab2VisibleBtn( NULL ),
  m_Tab0Tree( NULL ),
  m_Tab1Tree( NULL ),
  m_Tab2Tree( NULL ),
  m_LytPierrotVisibleBtn( NULL ),
  m_LytRobotVisibleBtn( NULL ),
  m_LytStageVisibleBtn( NULL ),
  m_LytStereoOnOffBtn( NULL ),
  m_EmpyreanVisibleBtn( NULL ),
  m_CameraCtrlBtn( NULL ),
  m_TouchPanelCamera( NULL ),
  m_LevelSlider( NULL ),
  m_RangeSlider( NULL ),
  m_StereoLevel( 0.f ),
  m_StereoRange( 0.f ),
  m_PierrotSlider( NULL ),
  m_RobotSlider( NULL ),
  m_StageMeshSlider( NULL ),
  m_AudienceSlider( NULL ),
  m_SmCamera( NULL ),
  m_LytUpdate( NULL )
{
    m_CameraBasePos.x = 0.f;
    m_CameraBasePos.y = 0.f;
    m_CameraBasePos.z = 0.f;
    m_CameraBaseTarget.x = 0.f;
    m_CameraBaseTarget.y = 0.f;
    m_CameraBaseTarget.z = 0.f;
}


//----------------------------------------
// Destructor
Demo::~Demo()
{
    // Destroy all non-GFX objects.
    SM_SAFE_DELETE( m_LytTabPanel );
    SM_SAFE_DELETE( m_Tab0VisibleBtn );
    SM_SAFE_DELETE( m_Tab1VisibleBtn );
    SM_SAFE_DELETE( m_Tab2VisibleBtn );
    SM_SAFE_DELETE( m_Tab0Tree );
    SM_SAFE_DELETE( m_Tab1Tree );
    SM_SAFE_DELETE( m_Tab2Tree );

    SM_SAFE_DELETE( m_LytPierrotVisibleBtn );
    SM_SAFE_DELETE( m_LytRobotVisibleBtn );
    SM_SAFE_DELETE( m_LytStageVisibleBtn );
    SM_SAFE_DELETE( m_LytParticleVisibleBtn );
    SM_SAFE_DELETE( m_LytAudienceVisibleBtn );
    SM_SAFE_DELETE( m_EmpyreanVisibleBtn );

    SM_SAFE_DELETE( m_LytStereoOnOffBtn );
    SM_SAFE_DELETE( m_CameraCtrlBtn );
    SM_SAFE_DELETE( m_TouchPanelCamera );

    SM_SAFE_DELETE( m_LevelSlider );
    SM_SAFE_DELETE( m_RangeSlider );

    SM_SAFE_DELETE( m_PierrotSlider );
    SM_SAFE_DELETE( m_RobotSlider );
    SM_SAFE_DELETE( m_StageMeshSlider );
    SM_SAFE_DELETE( m_AudienceSlider );
    SM_SAFE_DELETE( m_ParticleSlider );
}


//----------------------------------------
// Initialize scene.
void
Demo::InitializeScene()
{
    /*
        Nw4c demo and contents are coded here.
        Gfx objects being generated can be accessed from the <tt>SmModel</tt> class generated using the <tt>GfxCtrl</tt> class.
        
    */

    // Initialize Gfx objects, etc.

    // Celestial sphere.
    m_Empyrean = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyBg.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_Empyrean->SetLayerId( 0 );

    // Do not display the celestial sphere by default.
    m_Empyrean->SetVisible( true );

    // Clown model.
    m_Pierrot = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyPierrot.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_Pierrot->SetLayerId( 5 );

    // Back dancer clown model.
    m_AddPierrot = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyPierrotBackDancer.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_AddPierrot->SetVisible( false );    // Draw using a copied instance.
    m_AddPierrot->SetLayerId( 5 );

    // Robot model.
    m_Robot = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyRobot.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_Robot->SetLayerId( 5 );

    // Back dancer robot model.
    m_AddRobot = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyRobotBackDancer.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_AddRobot->SetVisible( false );    // Draw using a copied instance.
    m_AddRobot->SetLayerId( 5 );

    // Stage
    m_Stage = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyStageMatFix.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_A );
    m_Stage->SetLayerId( 1 );

    // Stage accessories.
    m_StageObject = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyStageObj.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_A );
    m_StageObject->SetLayerId( 5 );
    m_StageObject->SetVisibleZone( 0.0f );  // Not displayed at startup.

    // Audience model.
    m_Audience = m_GfxCtrl->CreateModel( NULL, NW_DEMO_FILE_PATH(L"ToyAudience.bcmdl"), GfxCtrl::GFX_CTRL_VRAM_A );
    m_Audience->SetVisible( false );    // Draw using a copied instance.
    m_Audience->SetLayerId( 5 );

    // Lights, etc.
    m_GfxCtrl->CreateSceneEnvironment( NW_DEMO_FILE_PATH(L"ToyMrnRbtLight.bcenv") );

    // Particle
    m_Particle = m_GfxCtrl->CreateParticle( NW_DEMO_FILE_PATH(L"ToyParticle.bcres"), GfxCtrl::GFX_CTRL_VRAM_B );
    m_Particle->SetLayerId( 6 );

    // Camera, etc.
    m_GfxCtrl->CreateCamera( NW_DEMO_FILE_PATH(L"ToyCamAni.bccam") );

    // Get camera.
    m_SmCamera = m_GfxCtrl->GetSmCamera();

    // Store initial view and viewpoint of camera.
    m_SmCamera->GetPosition( &m_CameraBasePos );
    m_SmCamera->GetTarget( &m_CameraBaseTarget );

    // Bind to message tree, because camera is controlled using the pad.
     this->AddChild( m_SmCamera );

    {
        // Configure clown animation.
        m_Pierrot->SetAnimEvaluatorNo( 0 );
        m_Pierrot->SetAnimationFrame( 0.f );

        // Change robot position.
         if ( m_Robot )
        {
            m_Robot->SetAnimEvaluatorNo( 0 );
            m_Robot->SetAnimationFrame( 0.f );
        }

        // Configure camera animation.
        m_SmCamera->SetAnimEvaluatorNo( 0 );

        // Configure clown animation.
        m_Audience->SetAnimEvaluatorNo( 0 );
        m_Audience->SetAnimationFrame( 0.f );

        // Configure clown animation.
        m_Stage->SetAnimEvaluatorNo( 0 );
        m_StageObject->SetAnimEvaluatorNo( 0 );
    }


    /*
        The UI is managed with a tree structure that uses the <tt>SmMessage</tt> class.
        Touch panel and page messages are sent from <tt>main.cpp</tt> and handled appropriately by the <tt>ReceiveMessage</tt> member function in each class according to the message.
        
    */

    // Initialize lower screen control panel UI.
    {
        // File structure for main window layout.
        m_tabResFile.clyt = "Main.bclyt";
        m_tabResFile.clan.push_back( "Main_Tab00.bclan" );
        m_tabResFile.clan.push_back( "Main_Tab01.bclan" );
        m_tabResFile.clan.push_back( "Main_Tab02.bclan" );

        // File structure for clown button layout.
        m_btnPierrorResFile.clyt = "Button.bclyt";
        m_btnPierrorResFile.clan.push_back( "Button_ButtonOn.bclan" );
        m_btnPierrorResFile.clan.push_back( "Button_ButtonOff.bclan" );
        SmLayoutResource* btnPierrotRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutPierrotButton.arc"), m_btnPierrorResFile );

        // File structure for robot button layout.
        m_btnRobotResFile.clyt = "Button.bclyt";
        m_btnRobotResFile.clan.push_back( "Button_ButtonOn.bclan" );
        m_btnRobotResFile.clan.push_back( "Button_ButtonOff.bclan" );
        SmLayoutResource* btnRobotRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutRobotButton.arc"), m_btnRobotResFile );

        // File structure for stage button layout.
        m_btnStageResFile.clyt = "Button.bclyt";
        m_btnStageResFile.clan.push_back( "Button_ButtonOn.bclan" );
        m_btnStageResFile.clan.push_back( "Button_ButtonOff.bclan" );
        SmLayoutResource* btnStageRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutStageButton.arc"), m_btnStageResFile );

        // File structure for audience button layout.
        m_btnAudienceResFile.clyt = "Button.bclyt";
        m_btnAudienceResFile.clan.push_back( "Button_ButtonOn.bclan" );
        m_btnAudienceResFile.clan.push_back( "Button_ButtonOff.bclan" );
        SmLayoutResource* btnAudienceRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutAudienceButton.arc"), m_btnAudienceResFile );

        // File structure for particle button layout.
        m_btnParticleResFile.clyt = "Button.bclyt";
        m_btnParticleResFile.clan.push_back( "Button_ButtonOn.bclan" );
        m_btnParticleResFile.clan.push_back( "Button_ButtonOff.bclan" );
        SmLayoutResource* btnParticleRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutParticleButton.arc"), m_btnParticleResFile );

        // File structure for slider bar layout.
        m_sliderResFile.clyt = "Slider.bclyt";
        SmLayoutResource* sliderRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutSlider.arc"), m_sliderResFile );

        // Create main window layout class.
        SmLayoutResource* tabPanelRes = m_LytCtrl->CreateSmLayoutResource( NW_DEMO_FILE_PATH(L"LayoutMain.arc"), m_tabResFile );
        m_LytTabPanel = m_LytCtrl->CreateSmLayout( tabPanelRes );
        m_LytTabPanel->SetVisiblePane( "CameraZoomIn", false );
        m_LytTabPanel->SetVisiblePane( "CameraZoomOut", false );

        // Create Tab 0 on/off buttons.
        nw::ut::Rect rect = tabPanelRes->GetRect( "TabBounding00" );
        m_Tab0VisibleBtn = new SmButton( rect.GetX(), rect.GetY(), rect.GetWidth(), rect.GetHeight() );
        this->AddChild( m_Tab0VisibleBtn );
        m_Tab0VisibleBtn->SetTarget( this, DEMO_TAB0_VISIBLE_BTN );
        m_Tab0VisibleBtn->SetVisible( false );

        // Create Tab 1 on/off buttons.
        rect = tabPanelRes->GetRect( "TabBounding01" );
        m_Tab1VisibleBtn = new SmButton( rect.GetX(), rect.GetY(), rect.GetWidth(), rect.GetHeight() );
        this->AddChild( m_Tab1VisibleBtn );
        m_Tab1VisibleBtn->SetTarget( this, DEMO_TAB1_VISIBLE_BTN );
        m_Tab1VisibleBtn->SetSwitch( false );
        m_Tab1VisibleBtn->SetVisible( false );

        // Create Tab 2 on/off buttons.
        rect = tabPanelRes->GetRect( "TabBounding02" );
        m_Tab2VisibleBtn = new SmButton( rect.GetX(), rect.GetY(), rect.GetWidth(), rect.GetHeight() );
        this->AddChild( m_Tab2VisibleBtn );
        m_Tab2VisibleBtn->SetTarget( this, DEMO_TAB2_VISIBLE_BTN );
        m_Tab2VisibleBtn->SetSwitch( false );
        m_Tab2VisibleBtn->SetVisible( false );


        // Create Tab 0 contents.
        m_Tab0Tree = new SmMessage( this );
        this->AddChild( m_Tab0Tree );
        {
            f32 baseX = 80.f;

            // Create clown on/off button.
            m_LytPierrotVisibleBtn = new SmLytButton( m_LytCtrl->CreateSmLayout( btnPierrotRes ), NULL, "ButtonBounding" );
            m_Tab0Tree->AddChild( m_LytPierrotVisibleBtn );
            m_LytPierrotVisibleBtn->SetPosition( baseX -120.0f, -50.f );
            m_LytPierrotVisibleBtn->SetTarget( this, DEMO_PIERROT_VISIBLE_BTN );

            // Create slider for adjusting the number of additional clowns to be displayed.
            m_PierrotSlider = new SmLytSliderBar( baseX -120, 40, m_LytCtrl->CreateSmLayout( sliderRes ) );
            m_Tab0Tree->AddChild( m_PierrotSlider );
            m_PierrotSlider->SetTarget( this, DEMO_ADD_PIERROT_NUM_CTRL_SLDR );
            m_PierrotSlider->SetParam( 1.f );


            // Create robot on/off button.
            m_LytRobotVisibleBtn = new SmLytButton( m_LytCtrl->CreateSmLayout( btnRobotRes ), NULL, "ButtonBounding" );
            m_Tab0Tree->AddChild( m_LytRobotVisibleBtn );
            m_LytRobotVisibleBtn->SetPosition( baseX -80.0f, -50.f );
            m_LytRobotVisibleBtn->SetTarget( this, DEMO_ROBOT_VISIBLE_BTN );

            // Create slider for adjusting the number of additional robots to be displayed.
            m_RobotSlider = new SmLytSliderBar( baseX -80, 40, m_LytCtrl->CreateSmLayout( sliderRes ) );
            m_Tab0Tree->AddChild( m_RobotSlider );
            m_RobotSlider->SetTarget( this, DEMO_ROBOT_NUM_CTRL_SLDR );
            m_RobotSlider->SetParam( 1.f );


            // Create stage on/off button.
            m_LytStageVisibleBtn = new SmLytButton( m_LytCtrl->CreateSmLayout( btnStageRes ), NULL, "ButtonBounding" );
            m_LytStageVisibleBtn->SetPosition( baseX -40.0f, -50.f );
            m_Tab0Tree->AddChild( m_LytStageVisibleBtn );
            m_LytStageVisibleBtn->SetTarget( this, DEMO_STAGE_VISIBLE_BTN );

            // Create mesh control slider for stage accessories.
            m_StageMeshSlider = new SmLytSliderBar( baseX -40, 40, m_LytCtrl->CreateSmLayout( sliderRes ) );
            m_Tab0Tree->AddChild( m_StageMeshSlider );
            m_StageMeshSlider->SetTarget( this, DEMO_STAGE_OBJECT_MESH_CTRL_SLDR );
            m_StageMeshSlider->SetParam( 1.f );


            // Create audience on/off button.
            m_LytAudienceVisibleBtn = new SmLytButton( m_LytCtrl->CreateSmLayout( btnAudienceRes ), NULL, "ButtonBounding" );
            m_LytAudienceVisibleBtn->SetPosition( baseX + 0.0f, -50.f );
            m_Tab0Tree->AddChild( m_LytAudienceVisibleBtn );
            m_LytAudienceVisibleBtn->SetTarget( this, DEMO_AUDIENCE_VISIBLE_BTN );

            // Create slider for increasing/decreasing spectators.
            m_AudienceSlider = new SmLytSliderBar( baseX + 0.0f, 40.0f, m_LytCtrl->CreateSmLayout( sliderRes ) );
            m_Tab0Tree->AddChild( m_AudienceSlider );
            m_AudienceSlider->SetTarget( this, DEMO_AUDIENCE_NUM_CTRL_SLDR );
            m_AudienceSlider->SetParam( 1.f );


            // Create particle on/off button.
            m_LytParticleVisibleBtn = new SmLytButton( m_LytCtrl->CreateSmLayout( btnParticleRes ), NULL, "ButtonBounding" );
            m_Tab0Tree->AddChild( m_LytParticleVisibleBtn );
            m_LytParticleVisibleBtn->SetPosition( baseX + 40.0f, -50.0f );
            m_LytParticleVisibleBtn->SetTarget( this, DEMO_PARTICLE_VISIBLE_BTN );

            // Create slider for controlling particles.
            m_ParticleSlider = new SmLytSliderBar( baseX + 40.0f, 40.0f, m_LytCtrl->CreateSmLayout( sliderRes ) );
            m_Tab0Tree->AddChild( m_ParticleSlider );
            m_ParticleSlider->SetTarget( this, DEMO_PARTICLE_NUM_CTRL_SLDR );
            m_ParticleSlider->SetParam( 0.f );
        }
        // Disable initial status.
        m_Tab0Tree->SetEnable( false );


        // Create Tab 1 contents.
        m_Tab1Tree = new SmMessage( this );
        this->AddChild( m_Tab1Tree );
        // Disable initial status.
        m_Tab1Tree->SetEnable( false );

        // Create Tab 2 contents.
        m_Tab2Tree = new SmMessage( this );
        this->AddChild( m_Tab2Tree );
        // Enable initial status.
        m_Tab2Tree->SetEnable( true );
        m_LytTabPanel->SetAnimationNo( 2 );
    }

    /*
        Load the init file (CmdTree.txt) that specifies the initial status of the demo and execute its contents.
        
    */
    {
        s32 pierrot_visible = 0;
        s32 robot_visible = 0;
        s32 audience_visible = 0;

        f32 stage_obj_rate = 0.0f;
        f32 particle_rate = 0.0f;

        // Initialization file
        const SmCmd* cmd;
        SmInitialFileParser* parser = new SmInitialFileParser();
        parser->Load( NW_DEMO_FILE_PATH(L"CmdTree.txt") );

        for ( u32 i = 0; i < parser->GetCmdNum(); i ++ )
        {
            cmd = parser->GetCmd( i );

            // Set initial status of stage accessories.
            if ( strcmp( cmd->GetCommand(), "STAGE_OBJECT" ) == 0 )
            {
                stage_obj_rate = cmd->GetF32( "rate" );
                m_StageMeshSlider->SetParam(1.0f - stage_obj_rate);
                m_StageObject->SetVisibleZone(stage_obj_rate);
            }

            // Set initial status of particles.
            if ( strcmp( cmd->GetCommand(), "PARTICLE" ) == 0 )
            {
                particle_rate = cmd->GetF32( "rate" );
                m_ParticleSlider->SetParam(1.0f - particle_rate);
                m_Particle->SetVisibleZone(particle_rate);
            }

            // Set initial status of the audience object.
            if ( strcmp( cmd->GetCommand(), "ADD_AUDIENCE" ) == 0 )
            {
                // Copy the audience.
                SmModel* audienceModel = m_Audience->Duplicate();
                m_GfxCtrl->AddModel( audienceModel );

                // Adjust the display position.
                f32 x = cmd->GetF32( "tx" );
                f32 y = cmd->GetF32( "ty" );
                f32 z = cmd->GetF32( "tz" );
                audienceModel->SetPosition( x, y, z );

                // Adjust the display angle.
                f32 rx = cmd->GetF32( "rx" );
                f32 ry = cmd->GetF32( "ry" );
                f32 rz = cmd->GetF32( "rz" );
                audienceModel->SetRotate( rx, ry, rz );

                // Initialize animations.
                audienceModel->SetAnimEvaluatorNo( 0 );

                // Control visibility.
                s32 visible = cmd->GetF32( "visible" );
                if ( visible != 0 )
                {
                    audienceModel->SetVisible( true );
                    audience_visible++;
                }
                else
                {
                    audienceModel->SetVisible( false );
                }

                m_AudienceArray.push_back( audienceModel );
            }

            // Add clown.
            if ( strcmp( cmd->GetCommand(), "ADD_PIERROT" ) == 0 )
            {
                // Copy the audience.
                SmModel* pierrotModel = m_AddPierrot->Duplicate();
                m_GfxCtrl->AddModel( pierrotModel );

                // Adjust the display position.
                f32 x = cmd->GetF32( "tx" );
                f32 y = cmd->GetF32( "ty" );
                f32 z = cmd->GetF32( "tz" );
                pierrotModel->SetPosition( x, y, z );

                // Adjust the display angle.
                f32 rx = cmd->GetF32( "rx" );
                f32 ry = cmd->GetF32( "ry" );
                f32 rz = cmd->GetF32( "rz" );
                pierrotModel->SetRotate( rx, ry, rz );

                // Initialize animations.
                pierrotModel->SetAnimEvaluatorNo( 0 );

                // Control visibility.
                s32 visible = cmd->GetF32( "visible" );
                if ( visible != 0 )
                {
                    pierrotModel->SetVisible( true );
                    pierrot_visible++;
                }
                else
                {
                    pierrotModel->SetVisible( false );
                }

                // Maintain instance.
                m_AddPierrotArray.push_back( pierrotModel );
            }

            // Add robot.
            if ( strcmp( cmd->GetCommand(), "ADD_ROBOT" ) == 0 )
            {
                // Copy the audience.
                SmModel* robotModel = m_AddRobot->Duplicate();
                m_GfxCtrl->AddModel( robotModel );

                // Adjust the display position.
                f32 x = cmd->GetF32( "tx" );
                f32 y = cmd->GetF32( "ty" );
                f32 z = cmd->GetF32( "tz" );
                robotModel->SetPosition( x, y, z );

                // Adjust the display angle.
                f32 rx = cmd->GetF32( "rx" );
                f32 ry = cmd->GetF32( "ry" );
                f32 rz = cmd->GetF32( "rz" );
                robotModel->SetRotate( rx, ry, rz );

                // Initialize animations.
                robotModel->SetAnimEvaluatorNo( 0 );

                // Control visibility.
                s32 visible = cmd->GetF32( "visible" );
                if ( visible != 0 )
                {
                    robotModel->SetVisible( true );
                    robot_visible++;
                }
                else
                {
                    robotModel->SetVisible( false );
                }

                // Maintain instance.
                m_AddRobotArray.push_back( robotModel );
            }

            // Initialize stereoscopic view parameters.
            if ( strcmp( cmd->GetCommand(), "STEREO_PARAM" ) == 0 )
            {
                m_StereoLevel = cmd->GetF32( "level" );
                m_StereoRange = cmd->GetF32( "range" );
//                m_LevelSlider->SetParam( m_StereoLevel );
//                m_RangeSlider->SetParam( m_StereoRange );
            }

            // Set the default camera position.
            if ( strcmp( cmd->GetCommand(), "DEFAULT_CAMERA" ) == 0 )
            {
                nw::math::VEC3 pos, target;
                pos.x = cmd->GetF32( "px" );
                pos.y = cmd->GetF32( "py" );
                pos.z = cmd->GetF32( "pz" );
                m_SmCamera->SetPadCameraPosition( &pos );

                target.x = cmd->GetF32( "tx" );
                target.y = cmd->GetF32( "ty" );
                target.z = cmd->GetF32( "tz" );
                m_SmCamera->SetPadCameraTarget( &target );
            }
        }

        // Adjust the slider bar position based on the number of additional objects displayed.
        f32 sliderParam = static_cast<f32>(pierrot_visible)/m_AddPierrotArray.size();
        m_PierrotSlider->SetParam(1.0f - sliderParam);

        sliderParam = static_cast<f32>(robot_visible)/m_AddRobotArray.size();
        m_RobotSlider->SetParam(1.0f - sliderParam);

        sliderParam = static_cast<f32>(audience_visible)/m_AudienceArray.size();
        m_AudienceSlider->SetParam(1.0f - sliderParam);
    }

    // Initialize the scene.
    m_GfxCtrl->InitializeScene();
}


//----------------------------------------
//
void
Demo::Exec( f32 frame )
{
    // Set clown animation.
    m_Pierrot->SetAnimationFrame( frame );

    // Set robot animation.
    m_Robot->SetAnimationFrame( frame );

    // Set camera animation.
    m_SmCamera->SetAnimationFrame( frame );

    // Set particle animation.
    m_Particle->SetAnimationFrame( frame );

    // Set stage animation.
    m_Stage->SetAnimationFrame( frame );

    // Audience model.
    for ( s32 i = 0; i < m_AudienceArray.size(); i++ )
    {
        m_AudienceArray[i]->SetAnimationFrame( frame );
    }

    // Additional clown.
    for ( s32 i = 0; i < m_AddPierrotArray.size(); i++ )
    {
        m_AddPierrotArray[i]->SetAnimationFrame( frame );
    }

    // Additional robot.
    for ( s32 i = 0; i < m_AddRobotArray.size(); i++ )
    {
        m_AddRobotArray[i]->SetAnimationFrame( frame );
    }

    // Light animation
    nw::gfx::AnimEvaluator* lightAnim = m_GfxCtrl->GetLightAnimEvaluator();
    if (lightAnim)
    {
        lightAnim->SetFrame(frame);
    }
}


//----------------------------------------
//
void
Demo::Reset()
{
    m_Pierrot->SetAnimationFrame( 0.0f );
    m_Robot->SetAnimationFrame( 0.0f );
    m_SmCamera->SetAnimationFrame( 0.0f );
    m_Particle->SetAnimationFrame( 0.0f );
    m_Particle->Reset();
    for ( s32 i = 0; i < m_AudienceArray.size(); i++ )
    {
        m_AudienceArray[i]->SetAnimationFrame( 0.0f );
    }
}



//----------------------------------------
//
bool
Demo::ReceveMessage( SmMessageType type, void* object, uint targetId )
{
    NW_UNUSED_VARIABLE(object);
    NW_UNUSED_VARIABLE(targetId);

    bool ret = false;
    bool updateTree = false;

    if ( type == SM_MESSAGE_UPDATE_PARAM )
    {
        switch( targetId )
        {
        // Switch tabs.
        case DEMO_TAB0_VISIBLE_BTN:
            m_LytTabPanel->SetAnimationNo( 0 );
            m_Tab0Tree->SetEnable( true );
            m_Tab1Tree->SetEnable( false );
            m_Tab2Tree->SetEnable( false );
            break;
        case DEMO_TAB1_VISIBLE_BTN:
            m_LytTabPanel->SetAnimationNo( 1 );
            m_Tab0Tree->SetEnable( false );
            m_Tab1Tree->SetEnable( true );
            m_Tab2Tree->SetEnable( false );
/*
            m_TouchPanelCamera->SetEnable( !m_CameraCtrlBtn->IsSwitch() );
*/
            break;
        case DEMO_TAB2_VISIBLE_BTN:
            m_LytTabPanel->SetAnimationNo( 2 );
            m_Tab0Tree->SetEnable( false );
            m_Tab1Tree->SetEnable( false );
            m_Tab2Tree->SetEnable( true );
            break;


        // Toggle the visibility of individual objects.
        case DEMO_PIERROT_VISIBLE_BTN:
            m_Pierrot->SetVisible( m_LytPierrotVisibleBtn->IsSwitch() );
            updateTree = true;
            ret = true;
            break;
        case DEMO_ROBOT_VISIBLE_BTN:
            m_Robot->SetVisible( m_LytRobotVisibleBtn->IsSwitch() );
            updateTree = true;
            ret = true;
            break;
        case DEMO_STAGE_VISIBLE_BTN:
            m_Stage->SetVisible( m_LytStageVisibleBtn->IsSwitch() );
            updateTree = true;
            ret = true;
            break;

        case DEMO_EMPYREAN_VISIBLE_BTN:
/*
            m_Empyrean->SetVisible( m_EmpyreanVisibleBtn->IsSwitch() );
            ret = true;
*/
            break;

        // Toggle the visibility of individual meshes.
        case DEMO_STAGE_OBJECT_MESH_CTRL_SLDR:
            m_StageObject->SetVisibleZone( 1.0f - m_StageMeshSlider->GetParam() );
            updateTree = true;
            ret = true;
            break;

        // Adjust the number of clowns being added using the slider.
        case DEMO_ADD_PIERROT_NUM_CTRL_SLDR:
            {
                f32 fparam = m_PierrotSlider->GetParam();
                s32 visibleNum = m_AddPierrotArray.size() * (1.0f - m_PierrotSlider->GetParam());

                for ( int i = 0; i < m_AddPierrotArray.size(); i++ )
                {
                     if ( i < visibleNum )
                    {
                        m_AddPierrotArray[i]->SetVisible( true );
                    }
                    else
                    {
                        m_AddPierrotArray[i]->SetVisible( false );
                    }
                }
                updateTree = true;
                ret = true;
            }
            break;

        // Adjust the number of robots being added using the slider.
        case DEMO_ROBOT_NUM_CTRL_SLDR:
            {
                f32 fparam = m_RobotSlider->GetParam();
                s32 visibleNum = m_AddRobotArray.size() * (1.0f - m_RobotSlider->GetParam());

                for ( int i = 0; i < m_AddRobotArray.size(); i++ )
                {
                     if ( i < visibleNum )
                    {
                        m_AddRobotArray[i]->SetVisible( true );
                    }
                    else
                    {
                        m_AddRobotArray[i]->SetVisible( false );
                    }
                }
                updateTree = true;
                ret = true;
            }
            break;

        // Adjust the number of spectators using the slider.
        case DEMO_AUDIENCE_NUM_CTRL_SLDR:
            {
                f32 fparam = m_AudienceSlider->GetParam();
                s32 visibleNum = m_AudienceArray.size() * (1.0f - m_AudienceSlider->GetParam());

                for ( int i = 0; i < m_AudienceArray.size(); i++ )
                {
                    if ( i < visibleNum )
                    {
                        m_AudienceArray[i]->SetVisible( true );
                    }
                    else
                    {
                        m_AudienceArray[i]->SetVisible( false );
                    }
                }
                updateTree = true;
                ret = true;
            }
            break;

        // Turn particles on/off.
        case DEMO_PARTICLE_VISIBLE_BTN:
            m_Particle->SetVisible( m_LytParticleVisibleBtn->IsSwitch() );
            updateTree = true;
            ret = true;
            break;

        case DEMO_PARTICLE_NUM_CTRL_SLDR:
             m_Particle->SetVisibleZone( 1.0f - m_ParticleSlider->GetParam() );
             updateTree = true;
             ret = true;
            break;

        default:
            break;
        }

        // Update the scene.
        m_GfxCtrl->TraverseScene();
    }

    // Update the scene.
    if ( type == SM_MESSAGE_SCENE_UPDATE || updateTree )
    {
        m_GfxCtrl->TraverseScene();
    }

    // Handle pad messages.
    if ( type == SM_MESSAGE_PAD_UPDATE )
    {
        if ( m_SmCamera->IsPadCamera() )
        {
            m_LytTabPanel->SetVisiblePane( "CameraZoomIn", true );
            m_LytTabPanel->SetVisiblePane( "CameraZoomOut", true );
        }
        else
        {
            m_LytTabPanel->SetVisiblePane( "CameraZoomIn", false );
            m_LytTabPanel->SetVisiblePane( "CameraZoomOut", false );
        }
    }

    return ret;
}

} // Namespace
