/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_DEBUGUTILITY_H_
#define NW_DEMO_DEBUGUTILITY_H_

//#include <nw/types.h>

#include <nw/dev/dev_Screenshot.h>
#include <nw/demo/demo_HioPacketChannel.h>
#include <nw/demo/demo_GraphicsDrawing.h>
#include <nw/demo/demo_GraphicsSystem.h>
#include <nw/demo/demo_Memory.h>
#include <nw/demo/demo_Utility.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw
{

namespace demo
{

//============================================================================
//
//

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class DebugUtility
{
private:
    static const s32 NW_PROFILE_OUTPUT_INTERVAL = 600;
    static const s32 NW_LOAD_METER_INTERVAL = 60;

    static const s32 POS_COMMAND_SIZE_X = 15;
    static const s32 POS_COMMAND_SIZE_Y = 115;
    static const s32 POS_LOAD_METER_X = 16;
    static const s32 POS_LOAD_METER_Y = 140;
    static const s32 HEIGHT_LOAD_METER = 18;
    static const s32 POS_BAR_CHART_X = 15;
    static const s32 POS_BAR_CHART_Y = 188;
    static const s32 WIDTH_LOAD_METER_BAR_CHART = 256;
    static const s32 HEIGHT_LOAD_METER_BAR_CHART = 20;

public:
    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    static void CalcLoadMeter(
        nw::demo::RenderSystem* renderSystem
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void DrawLoadMeterText(
        nw::demo::RenderSystem* renderSystem,
        nw::demo::GraphicsDrawing* graphicsDrawing
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void DrawLoadMeter(
        nw::demo::RenderSystem* renderSystem,
        nw::demo::GraphicsDrawing* graphicsDrawing
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    static void DumpLoadMeter(
        nw::demo::RenderSystem* renderSystem
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static bool IsCpuProfilingMode()
    {
        return s_IsForceCpuProfilingMode || nw::demo::PadFactory::GetPad()->IsButtonPress(nw::demo::Pad::BUTTON_DEBUG);
    }

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void InitializeAutoTest(
        os::IAllocator* allocator,
        os::IAllocator* deviceAllocator,
        nw::demo::RenderSystem** renderSystem,
        const u32* correctHashDisplay0 = NULL,
        const u32* correctHashDisplay1 = NULL,
        bool needCustomTest = false
    )
    {
        s_ProfileCallCount = 0;
        s_DebugFrameCount = 0;

        #if defined(NW_AUTO_TEST)
            s_AutoTester = AutoTester::Create(
                allocator,
                deviceAllocator
            );

            s_AutoTester->Initialize(
                5,
                needCustomTest,
                renderSystem,
                correctHashDisplay0,
                correctHashDisplay1);
        #else
            NW_UNUSED_VARIABLE(allocator);
            NW_UNUSED_VARIABLE(deviceAllocator);
            NW_UNUSED_VARIABLE(renderSystem);
            NW_UNUSED_VARIABLE(correctHashDisplay0);
            NW_UNUSED_VARIABLE(correctHashDisplay1);
            NW_UNUSED_VARIABLE(needCustomTest);
        #endif
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    static void FinalizeAutoTest()
    {
        #if defined(NW_AUTO_TEST)
        s_AutoTester->Finalize();
        nw::ut::SafeDestroy(s_AutoTester);
        #endif
    }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    static void AdvanceAutoTestFrame()
    {
        #if defined(NW_AUTO_TEST)
        NW_ASSERT(s_AutoTester);
        s_AutoTester->AdvanceFrame();
        #endif
        if (!s_SilentMode)
        {
            if (s_ProfileCallCount % NW_PROFILE_OUTPUT_INTERVAL == 0)
            {
                NW_DUMP_PROFILE();
            }
            else if (s_ProfileCallCount % NW_PROFILE_OUTPUT_INTERVAL == 1)
            {
                NW_CLEAR_PROFILE();
            }
            s_ProfileCallCount++;
        }
    }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    static void PostInitializeScenes()
    {
        // Send the state of the allocator set with SetAllocatorForTest to the PC.
        #if defined(NW_AUTO_TEST)
        NW_ASSERT(s_AutoTester);
        s_AutoTester->NotifyMemoryStatus(
            s_DeviceAllocator,
            s_ParticleAllocator,
            AutoTester::TIMING_POST_INITIALIZE_SCENES);
        #endif
    }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    static void PreTerminateScenes()
    {
        // Send the state of the allocator set with SetAllocatorForTest to the PC.
        #if defined(NW_AUTO_TEST)
        NW_ASSERT(s_AutoTester);
        s_AutoTester->NotifyMemoryStatus(
            s_DeviceAllocator,
            s_ParticleAllocator,
            AutoTester::TIMING_PRE_TERMINATE_SCENES);
        #endif
    }

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void
    SetAllocatorForTest(
        nw::demo::DemoAllocator* deviceAllocator,
        nw::demo::DemoAllocator* particleAllocator = NULL
    )
    {
        s_DeviceAllocator = deviceAllocator;
        s_ParticleAllocator = particleAllocator;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void
    ResetMemoryLeakTester()
    {
        s_DeviceMemoryFreeSize = s_DeviceAllocator->GetFreeSize();
        if (s_ParticleAllocator)
        {
            s_ParticleMemoryFreeSize = s_ParticleAllocator->GetFreeSize();
        }
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static bool
    TestMemoryLeak()
    {
        bool result = true;

        s32 mainLeakSize = s_DeviceMemoryFreeSize - s_DeviceAllocator->GetFreeSize();
        s32 particleLeakSize = 0;
        if (s_ParticleAllocator)
        {
            particleLeakSize = s_ParticleMemoryFreeSize - s_ParticleAllocator->GetFreeSize();
        }

        if (!s_SilentMode)
        {
            if (mainLeakSize)
            {
                NW_DEV_LOG(
                    "######### Device Memory Leak Detected !! (%d bytes)\n", mainLeakSize);
                s_DeviceAllocator->Dump();
                result = false;
            }
            if (particleLeakSize)
            {
                NW_DEV_LOG(
                    "######### Particle Memory Leak Detected !! (%d bytes)\n", particleLeakSize);
                s_ParticleAllocator->Dump();
                result = false;
            }
        }

    #if defined(NW_AUTO_TEST)
        if (s_AutoTester)
        {
            s_AutoTester->NotifyMemoryLeak(mainLeakSize, particleLeakSize);
        }
    #endif

        return result;
    }

    //

    //=====================
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    static void SetSilentMode(bool isEnabled) {s_SilentMode = isEnabled;}

    //

    //
    static void SendScreenshot(int param)
    {
        #if defined(NW_AUTO_TEST)
        s_AutoTester->SendScreenshot(param);
        #else
        NW_UNUSED_VARIABLE(param);
        #endif
    }

    //
    static void SendText(const char* text)
    {
        #if defined(NW_AUTO_TEST)
        s_AutoTester->SendText(text);
        #else
        NW_UNUSED_VARIABLE(text);
        #endif
    }

    //
    static void SendLoadMeter()
    {
        #if defined(NW_AUTO_TEST)
        s_AutoTester->SendLoadMeter();
        #endif
    }

private:
    static size_t s_DeviceMemoryFreeSize;
    static size_t s_ParticleMemoryFreeSize;
    static nw::demo::DemoAllocator* s_DeviceAllocator;
    static nw::demo::DemoAllocator* s_ParticleAllocator;
    static bool s_IsForceCpuProfilingMode; //
    static bool s_SilentMode; //
    static s32 s_ProfileCallCount;
    static s32 s_DebugFrameCount;

#if defined(NW_AUTO_TEST)
    class AutoTester
    {
    public:
        enum NOTIFY_TIMING
        {
            TIMING_POST_INITIALIZE_SCENES = 1,
            TIMING_PRE_TERMINATE_SCENES
        };

        //
        static AutoTester* Create(
            os::IAllocator* allocator,
            os::IAllocator* deviceAllocator
        );

        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        //
        int Initialize(
            s32 channelNumber,
            bool needCustom,
            nw::demo::RenderSystem** renderSystem,
            const u32* correctHashDisplay0,
            const u32* correctHashDisplay1
        );

        //
        void Finalize();

        //
        void Destroy();

        //
        void AdvanceFrame();

        //
        //
        //
        //
        //
        //
        void NotifyMemoryLeak(s32 deviceLeakSize, s32 particleLeakSize);

        //
        //
        //
        //
        //
        void NotifyMemoryStatus(
            nw::demo::DemoAllocator* deviceAllocator,
            nw::demo::DemoAllocator* particleAllocator,
            NOTIFY_TIMING timing
        );

        void SendScreenshot(int)
        {
            ScreenshotParam param;
            param.display = ScreenshotParam::DISPLAY_0;
            SendScreenshot(param);
        }

        void SendLoadMeter()
        {
            SendLoadMeterParam param;
            SendLoadMeter(param);
        }

        void SendText(const char* text);

    private:
        //
        enum EventType
        {
            EVENT_SCREENSHOT = 1,               //
            EVENT_SET_PAD_STATUS,               //
            EVENT_BEACON,                       //
            EVENT_SEND_LOAD_METER,              //
            EVENT_RESET_LOAD_METER,             //
            EVENT_TEST_DONE,                    //
            EVENT_SWITCH_CPU_PROFILING_MODE    //
        };

        struct ScreenshotParam
        {
            enum DisplayFlag
            {
                DISPLAY_0 = 1,
                DISPLAY_0_EXT,
                DISPLAY_1,
                DISPLAY_0_BOTH,
                DISPLAY_ALL
            };
            s32 display;
            u32 hash[5]; // If all are 0, the test is not performed.
        };

        struct SetPadStatusParam
        {
            u32 padStatus;
        };

        struct BeaconParam
        {
        };

        struct SendLoadMeterParam
        {
        };

        struct ResetLoadMeterParam
        {
        };

        struct SwitchCpuProfilingModeParam
        {
            s32 isModeEnabled;
        };

        struct TestEvent
        {
            s32 frameCount;
            EventType eventType;

            union
            {
                ScreenshotParam screenshot;
                SetPadStatusParam setPadStatus;
                BeaconParam beacon;
                SendLoadMeterParam sendLoadMeter;
                ResetLoadMeterParam resetLoadMeter;
                SwitchCpuProfilingModeParam switchCpuProfilingMode;
            } param;
        };

        // The receive packet's structure.

        //
        struct TestInfoHeader
        {
            s32 packetType;
            s32 eventCount;
            char programName[64];
            enum { PACKET_TYPE_ID = 0x213FEB92 };

            TestInfoHeader()
            : packetType(PACKET_TYPE_ID),
              eventCount(0)
            {}
        };

        //
        struct ScenarioHeader
        {
            s32 packetType;
            s32 eventCount;
            enum { PACKET_TYPE_ID = 0x6794C4C8 };

            ScenarioHeader()
            : packetType(PACKET_TYPE_ID),
              eventCount(0)
            {}

        };

        // The send packet's structure.

        //
        struct ScreenshotHeader
        {
            s32 packetType;
            s32 frameCount;
            s32 width;
            s32 height;
            s32 display;

            enum { PACKET_TYPE_ID = 0x23848aa6 };

            ScreenshotHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0),
              width(0),
              height(0)
            {}
        };

        //
        struct BeaconHeader
        {
            s32 packetType;
            s32 frameCount;

            enum { PACKET_TYPE_ID = 0x9538a79a };

            BeaconHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        //
        struct LoadMeterHeader
        {
            s32 packetType;
            s32 frameCount;

            enum { PACKET_TYPE_ID = 0xc57c557 };

            LoadMeterHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        //
        struct LoadMeterData
        {
            s32 callCount;
            s32 cpuTime;
            s32 gpuTime;
            s32 otherGpuTime;
            s32 cmdSize;
        };

        //
        struct MemoryLeakHeader
        {
            s32 packetType;
            s32 frameCount;

            enum { PACKET_TYPE_ID = 0x4F1B0AD2 };

            MemoryLeakHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        //
        struct MemoryLeakData
        {
            s32 deviceLeakSize;
            s32 particleLeakSize;
        };

        //
        struct DoneHeader
        {
            s32 packetType;
            s32 frameCount;

            enum { PACKET_TYPE_ID = 0x916bbe7 };

            DoneHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        //
        struct MemoryStatusHeader
        {
            s32 packetType;
            s32 frameCount;
            s32 timing;

            enum { PACKET_TYPE_ID = 0xE568134E };

            MemoryStatusHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        //
        struct MemoryStatusData
        {
            s32 deviceMemoryUsed;
            s32 particleMemoryUsed;
        };

        //
        struct TestHeader
        {
            s32 packetType;
            s32 frameCount;
            u32 reason;

            enum Reason
            {
                REASON_NONE
            };

            enum { PACKET_TYPE_ID = 0xA13BBC12 };

            TestHeader()
            : packetType(PACKET_TYPE_ID),
              frameCount(0)
            {}
        };

        AutoTester(
            os::IAllocator* allocator,
            os::IAllocator* deviceAllocator
        );
        ~AutoTester();

        void SendScreenshot(const ScreenshotParam& param);
        void SetPadStatus(const SetPadStatusParam& param);
        void SendBeacon(const BeaconParam& param);
        void SendLoadMeter(const SendLoadMeterParam& param);
        void NotifyDone();
        void SwitchCpuProfilingMode(const SwitchCpuProfilingModeParam& param);

        os::IAllocator* m_Allocator;
        os::IAllocator* m_DeviceAllocator;

        nw::dev::ScreenshotManager* m_ScreenshotManager;

        nw::demo::HioPacketChannel* m_HioPacketChannel;

        s32 m_FrameCount;

        nw::ut::MoveArray<TestEvent> m_Scenario;
        nw::ut::MoveArray<TestEvent>::iterator m_CurrentEvent;

        nw::demo::RenderSystem** m_RenderSystem;
    };

    static AutoTester* s_AutoTester;
#else
    class AutoTester
    {
    public:
        static AutoTester* Create(os::IAllocator*,os::IAllocator*){return NULL;}
        int Initialize(s32,bool,nw::demo::RenderSystem**,const u32*,const u32*){return -1;}
        void Finalize(){}
        void Destroy(){}
        void AdvanceFrame(){}
        void NotifyMemoryLeak(s32, s32){}
    };
#endif
};

//

namespace internal
{


//
class DemoTestLoop
{
private:
    static const int NW_PROFILE_MAX_REPORT = 60;
public:
    DemoTestLoop(
        nw::demo::DemoAllocator* deviceAllocator,
        nw::demo::DemoAllocator* particleAllocator,
        nw::demo::RenderSystem** renderSystem,
        const u32* hash1,
        const u32* hash2
    )
    : m_DeviceAllocator(deviceAllocator),
      m_ParticleAllocator(particleAllocator),
      m_ContinueFlag(true),
      m_RenderSystem(renderSystem)
    {
        nw::demo::DebugUtility::InitializeAutoTest(m_DeviceAllocator, m_DeviceAllocator, m_RenderSystem, hash1, hash2);
        NW_INITIALIZE_PROFILE(NW_PROFILE_MAX_REPORT, m_DeviceAllocator);
#if defined(NW_DEBUG_CHECK_MEMORY_LEAK)
        nw::demo::DebugUtility::SetAllocatorForTest(m_DeviceAllocator, m_ParticleAllocator);
        nw::demo::DebugUtility::ResetMemoryLeakTester();
#endif
    }
    ~DemoTestLoop()
    {
        NW_FINALIZE_PROFILE(m_DeviceAllocator);
        nw::demo::DebugUtility::FinalizeAutoTest();
    }
    void Continue() {
#if defined(NW_DEBUG_CHECK_MEMORY_LEAK)
        nw::demo::DebugUtility::TestMemoryLeak();
#else
        m_ContinueFlag = false;
#endif
        nw::demo::DebugUtility::FinalizeAutoTest();
        // HACK: Since an optional status cannot be passed from AutoTester on the PC-side, make the trigger an operation that cannot happen on the actual device.
        const bool isAutotesting =
            nw::demo::PadFactory::GetPad()->IsButtonPress(nw::demo::Pad::BUTTON_UP) &&
            nw::demo::PadFactory::GetPad()->IsButtonPress(nw::demo::Pad::BUTTON_RIGHT) &&
            nw::demo::PadFactory::GetPad()->IsButtonPress(nw::demo::Pad::BUTTON_DOWN) &&
            nw::demo::PadFactory::GetPad()->IsButtonPress(nw::demo::Pad::BUTTON_LEFT);
        nw::demo::DebugUtility::InitializeAutoTest(m_DeviceAllocator, m_DeviceAllocator, m_RenderSystem, NULL, NULL, isAutotesting);
    }
    bool IsContinuing() {
#if defined(NW_DEBUG_CHECK_MEMORY_LEAK)
        return ! ::nw::demo::internal::IsTerminatingImpl();
#else
        return m_ContinueFlag;
#endif
    }
private:
    nw::demo::DemoAllocator* m_DeviceAllocator;
    nw::demo::DemoAllocator* m_ParticleAllocator;
    bool m_ContinueFlag;
    nw::demo::RenderSystem** m_RenderSystem;
};

} // namespace internal

//============================================================================
//
//

#define NW_DEMO_TEST_LOOP(deviceAllocator, particleAllocator, renderSystem)                                             \
    for (                                                                                                               \
        nw::demo::internal::DemoTestLoop demoTestLoop(deviceAllocator, particleAllocator, renderSystem, NULL, NULL) ;   \
        demoTestLoop.IsContinuing() ;                                                                                   \
        demoTestLoop.Continue() )

//

} // namespace demo
} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif // NW_DEMO_DEBUGUTILITY_H_
