/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_GRAPHICS_MEMORY_H_
#define NW_DEMO_GRAPHICS_MEMORY_H_

#include <nw/types.h>
#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <nw/demo/demo_Memory.h>
#include <nn/fnd.h>


#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw
{
namespace demo
{

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class GraphicsMemoryAllocator
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void    Initialize(os::IAllocator* allocator);

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void    Finalize();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void*   Allocate(GLenum area, GLenum aim, GLuint id, GLsizei size);

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void    Deallocate(GLenum area, GLenum aim, GLuint id, void* addr);

    //----------------------------------------
    //
    //

    //
    GraphicsMemoryAllocator()
     : m_Initialized( false ),
       m_CurrentAddrVramA( NULL ),
       m_CurrentAddrVramB( NULL ),
       m_DeviceMemoryAllocator( NULL )
       {}
    //
    virtual ~GraphicsMemoryAllocator() { Finalize(); }
    //

private:
    bool m_Initialized;

    uptr m_CurrentAddrVramA;
    uptr m_CurrentAddrVramB;

    os::IAllocator* m_DeviceMemoryAllocator;

    uptr MathRoundup(uptr x, int base)
    {
        return ((x) + ((base)-1)) & ~((base)-1);
    }
};

} // namespace demo
} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif      // NW_DEMO_GRAPHICS_MEMORY_H_
