/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_HIOPACKETCHANNEL_H_
#define NW_DEMO_HIOPACKETCHANNEL_H_

#include <nw/assert.h>
#include <nw/os/os_Memory.h>
#include <nw/dev/dev_Utility.h>

#if defined(NN_SWITCH_ENABLE_HOST_IO)

#include <nn/hio/CTR/hio_Api.h>
#include <nn/hio/CTR/hio_SerialChannel.h>

namespace nw {
namespace demo {

//---------------------------------------------------------------------------
//
//
//
//---------------------------------------------------------------------------
class HioPacketChannel
{
private:
    static const s32 PACKET_HEADER_ID;
    static const size_t BUFFER_SIZE;

public:
    //
    //
    //
    struct Description
    {
        s32 channelNumber;      //

        //
        Description()
        : channelNumber(1)
        {}
    };

    //
    //
    //
    class Builder
    {
    public:
        //
        Builder& HioPacketChannelDescription(const Description& description)
        {
            m_Description = description;
            return *this;
        }

        //
        Builder& ChannelNumber(s32 channelNumber)
        {
            NW_MINMAX_ASSERT(channelNumber, 0, nn::hio::CTR::MAX_CHANNEL_NUM);

            m_Description.channelNumber = channelNumber;
            return *this;
        }

        //---------------------------------------------------------------------------
        //
        //
        //
        //
        //
        //
        //---------------------------------------------------------------------------
        HioPacketChannel* Create(
            os::IAllocator* allocator,
            os::IAllocator* deviceAllocator
        );

    private:
        Description m_Description;
    };

    //----------------------------------------
    //
    //

    //
    void Destroy();

    //}

    //----------------------------------------
    //
    //

    //
    //
    //
    static bool Initialize(os::IAllocator* deviceAllocator);

    //
    static void Finalize();

    //}

    //----------------------------------------
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool Listen(s32 timeOut);

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool Connect();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    bool Disconnect();

    //}



    //----------------------------------------
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    size_t SendPacket(
        const void* headerBuffer,
        size_t headerLength,
        const void* dataBuffer,
        size_t dataLength
    );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool ReceivePacket(
        size_t* headerWrittenSize,
        size_t* dataWrittenSize,
        void* headerBuffer,
        size_t headerLength,
        void* dataBuffer,
        size_t dataLength,
        int timeOut
    );

    //}

private:
    HioPacketChannel(
        os::IAllocator* allocator,
        os::IAllocator* deviceAllocator,
        const Description& description
    );

    ~HioPacketChannel();

    nw::os::IAllocator* m_Allocator;
    nw::os::IAllocator* m_DeviceAllocator;

    static nw::os::IAllocator* s_DeviceAllocatorForHioWorkMemory;
    static void* s_HioWorkMemory;
    void* m_SerialWorkMemory;

    nn::hio::CTR::SerialChannel m_Serial;
};


} // namespace nw::demo
} // namespace nw

#else


namespace nw
{

namespace os
{
class IAllocator;
}
namespace demo
{

//
class HioPacketChannel
{
private:
    static const s32 PACKET_HEADER_ID;
    static const size_t BUFFER_SIZE;
public:
    //
    struct Description
    {
        Description()
        {}
    };
    //
    class Builder
    {
    public:
        Builder& HioPacketChannelDescription(const Description&){return *this;}
        Builder& ChannelNumber(s32 ){return *this;}
        HioPacketChannel* Create(os::IAllocator* ,os::IAllocator* ) { return NULL; }
    };
    void Destroy() {}
    static bool Initialize(os::IAllocator*) { return false; }
    static void Finalize() {}
    bool Listen(s32 ) { return false; }
    bool Connect() { return false; }
    bool Disconnect() { return false; }
    size_t SendPacket(const void* ,size_t ,const void* ,size_t ) { return 0; }
    bool ReceivePacket(size_t* ,size_t* ,void* ,size_t ,void* ,size_t ,int ){ return false; }
private:
    HioPacketChannel(os::IAllocator* ,os::IAllocator* ,const Description& ){ }
    ~HioPacketChannel() { }
    static nw::os::IAllocator* s_DeviceAllocatorForHioWorkMemory;
    static void* s_HioWorkMemory;
};


} // namespace nw::demo
} // namespace nw

#endif

#endif // NW_DEMO_HIOPACKETCHANNEL_H_

