/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_PAD_H_
#define NW_DEMO_PAD_H_

#include <nn/types.h>
#include <nn/hid.h>

#include <nw/demo/demo_PadButton.h>
#include <nw/demo/demo_Memory.h>
#include <nw/math/math_Types.h>
#include <nw/ut/ut_Preprocessor.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw {
namespace demo {

class PadFactory;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class Pad
{
private:
    NW_DISALLOW_COPY_AND_ASSIGN(Pad);

public:
    friend class PadFactory;

    //
    enum Button
    {
        BUTTON_A               = nn::hid::CTR::BUTTON_A,            //
        BUTTON_B               = nn::hid::CTR::BUTTON_B,            //
        BUTTON_START           = nn::hid::CTR::BUTTON_START,        //
        BUTTON_RIGHT           = nn::hid::CTR::BUTTON_RIGHT,        //
        BUTTON_LEFT            = nn::hid::CTR::BUTTON_LEFT,         //
        BUTTON_UP              = nn::hid::CTR::BUTTON_UP,           //
        BUTTON_DOWN            = nn::hid::CTR::BUTTON_DOWN,         //
        BUTTON_R               = nn::hid::CTR::BUTTON_R,            //
        BUTTON_L               = nn::hid::CTR::BUTTON_L,            //
        BUTTON_X               = nn::hid::CTR::BUTTON_X,            //
        BUTTON_Y               = nn::hid::CTR::BUTTON_Y,            //
        BUTTON_DEBUG           = nn::hid::CTR::BUTTON_DEBUG,        //
        // BUTTON_HEADPHONE       = nn::hid::CTR::BUTTON_HEADPHONE,
        BUTTON_EMULATION_RIGHT = nn::hid::CTR::BUTTON_RIGHT,        //
        BUTTON_EMULATION_LEFT  = nn::hid::CTR::BUTTON_RIGHT,        //
        BUTTON_EMULATION_UP    = nn::hid::CTR::BUTTON_RIGHT,        //
        BUTTON_EMULATION_DOWN  = nn::hid::CTR::BUTTON_RIGHT,        //
        BUTTON_SELECT          = nn::hid::CTR::PSEUDO_BUTTON_SELECT //
    };

    //
    //
    void Initialize();

    //
    //
    void Finalize();

    //
    //
    void Update();

    //
    //
    void Reset();


    //
    //
    //
    //
    //
    //
    bool IsButtonPress( Button button ) const;

    //
    //
    //
    //
    //
    //
    bool IsButtonDown( Button button ) const;

    //
    //
    //
    //
    //
    //
    bool IsButtonUp( Button button ) const;

    //
    //
    //
    //
    //
    //
    bool IsButtonRepeat( Button button ) const;

    //
    //
    //
    //
    //
    //
    //
    bool IsButtonRepeatFast( Button button ) const;

    //
    //
    //
    //
    s16 GetAnalogStickX() const { return m_AnalogStickX; }

    //
    //
    //
    //
    s16 GetAnalogStickY() const { return m_AnalogStickY; }

    //
    //
    //
    //
    //
    nw::math::VEC2 GetAnalogStick() const { return m_AnalogStick; }

    //
    //
    //
    //
    //
    //
    void EnableTestMode(bool enabled) { m_IsTestMode = enabled; }

    //
    //
    //
    void SetTestPadStatus(u32 status) { m_TestPadStatus = status; }

protected:
    //
    //
    //
    //
    static Pad* Create(os::IAllocator* allocator);

    //
    //
    void Destroy();

private:
    //
    //
    Pad()
    : m_AnalogStickX(0),
      m_AnalogStickY(0),
      m_AnalogStick(math::VEC2::Zero()),
      m_IsTestMode(false),
      m_TestPadStatus(0)
    {}

private:
    internal::PadButton m_PadButton;
    os::IAllocator* m_Allocator;
    nn::hid::PadStatus m_PadStatus;

    s16 m_AnalogStickX;
    s16 m_AnalogStickY;
    nw::math::VEC2 m_AnalogStick;

    bool m_IsTestMode;
    u32 m_TestPadStatus;

};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class PadFactory
{
public:
    //
    //
    //
    //
    static void Initialize(os::IAllocator* allocator);

    //
    //
    static void Finalize();

    //
    //
    //
    //
    static Pad* GetPad();

private:
    //
    //
    PadFactory(){}
    static Pad* s_pPad;

};

} // namespace nw::demo
} // namespace nw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif /* NW_DEMO_PAD_H_ */

