/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_DEMO_SIMPLEAPP_H_
#define NW_DEMO_SIMPLEAPP_H_

#include <nw/sdk.h>
#include <nw/types.h>
#include <nw/demo/demo_GraphicsSystem.h>
#include <nw/demo/demo_Memory.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw {
namespace demo {

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class SimpleApp
{
public:
    //
    enum Display
    {
        DISPLAY0,       //
        DISPLAY1,       //
        DISPLAY_BOTH,   //
        DISPLAY_MAX     //
    };

    //
    enum DisplaySize
    {
        DISPLAY0_WIDTH = 400,   //
        DISPLAY0_HEIGHT = 240,  //
        DISPLAY1_WIDTH = 320,   //
        DISPLAY1_HEIGHT = 240   //
    };

    //
    //
    //
    static SimpleApp& GetInstance()
    {
        static SimpleApp s_Instance;
        return s_Instance;
    }

    //
    //
    //
    void Initialize();

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetRenderingTarget(Display display);

    //
    //
    //
    //
    //
    const gfx::FrameBufferObject& GetFrameBufferObject() const;

    //
    //
    //
    //
    //
    void SwapBuffer(Display display);

    //

    //
    //
    //
    void Finalize();

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    static void* Allocate(size_t size, u8 alignment = 4);

    //
    //
    //
    //
    //
    //
    //
    //
    static void* AllocateDeviceMemory(size_t size, u8 alignment = 4);

    //
    //
    //
    //
    //
    //
    static void Free(void* ptr);

    //
    //
    //
    //
    //
    nw::os::IAllocator& GetAllocator()
    {
        return m_DeviceAllocator;
    }

    //
    //
    //
    //
    //
    nw::os::IAllocator& GetDeviceAllocator()
    {
        return m_DeviceAllocator;
    }

    //

    Display m_CurrentDisplay;
    nw::demo::RenderSystem* m_pRenderSystem;
    nw::gfx::IRenderTarget* m_pRenderTargetUpper;
    nw::gfx::IRenderTarget* m_pRenderTargetLower;
    nw::demo::DemoAllocator m_DeviceAllocator;

protected:
    //
    //
    //
    //
    //
    SimpleApp()
    {
        if (s_pInstance != NULL)
        {
            NW_FATAL_ERROR("nw::demo::SimpleApp instance already exists.");
        }
        s_pInstance = this;
    }

    //
    //
    //
    void TransferBuffer();

    //
    static SimpleApp* s_pInstance;
};

} /* namespace demo */
} /* namespace nw   */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif /* NW_DEMO_SIMPLEAPP_H_ */

