﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/demo/demo_PadButton.h>
#include <algorithm>

namespace nw {
namespace demo {
namespace internal {

//--------------------------------------------------------------------------
PadButton::PadButton()
{
    Reset();
}

//--------------------------------------------------------------------------
void
PadButton::Update( u32 buttonStatus )
{
    if ( m_ResetFlag )
    {
        m_ButtonDown = m_ButtonUp = m_ButtonRepeat = m_ButtonRepeatFast = 0;
        m_ResetFlag = false;
    }
    else
    {
        m_ButtonDown = ( m_ButtonPress ^ buttonStatus ) & buttonStatus;
        m_ButtonUp = ( m_ButtonPress ^ buttonStatus ) & m_ButtonPress;
        m_ButtonRepeat = m_ButtonDown;
        m_ButtonRepeatFast = m_ButtonDown;
    }
    m_ButtonPress = buttonStatus;

    u32 mask = 1;
    for ( int i = 0; i < BUTTON_COUNT_MAX; ++i, mask <<= 1 )
    {
        if ( m_ButtonPress & mask )
        {
            --m_RepeatCounter[ i ];
            --m_RepeatCounterFast[ i ];

            if ( m_RepeatCounter[ i ] == 0 )
            {
                m_ButtonRepeat |= mask;
                m_RepeatCounter[ i ] = REPEAT_SPAN;
            }
            if ( m_RepeatCounterFast[ i ] == 0 )
            {
                m_ButtonRepeatFast |= mask;
                m_RepeatCounterFast[ i ] = 1;
            }
        }
        else
        {
            m_RepeatCounter[ i ] = REPEAT_START;
            m_RepeatCounterFast[ i ] = REPEAT_START;
        }
    }
}

//--------------------------------------------------------------------------
void
PadButton::Reset()
{
    m_ResetFlag = true;

    m_ButtonPress = 0;
    m_ButtonDown = 0;
    m_ButtonUp = 0;
    m_ButtonRepeat = 0;
    m_ButtonRepeatFast = 0;

    std::fill_n(m_RepeatCounter, BUTTON_COUNT_MAX, static_cast<u8>(REPEAT_START));
    std::fill_n(m_RepeatCounterFast, BUTTON_COUNT_MAX, static_cast<u8>(REPEAT_START));
}

//--------------------------------------------------------------------------
void
PadButton::ClearTriggerFlag()
{
    m_ButtonDown = 0;
    m_ButtonUp = 0;
    m_ButtonRepeat = 0;
    m_ButtonRepeatFast = 0;
}

} // namespace nw::demo::internal
} // namespace nw::demo
} // namespace nw
