/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_FONT_PAIRFONT_H_
#define NW_FONT_PAIRFONT_H_

#include <nn/types.h>
#include <nw/font/font_Font.h>


namespace nw {
namespace font {


//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class PairFont
    : public Font
{
public:
    /* ------------------------------------------------------------------------
            Functions
       ------------------------------------------------------------------------ */

    //
    //

    //
    //
    //
    //
    //
                                PairFont(
                                    Font*   primary,
                                    Font*   secondary);

    //
    virtual                     ~PairFont();

    //


    //
    //

    virtual int                 GetWidth() const;

    virtual int                 GetHeight() const;

    virtual int                 GetAscent() const;

    virtual int                 GetDescent() const;

    virtual int                 GetMaxCharWidth() const;

    virtual Type                GetType() const;

    virtual TexFmt              GetTextureFormat() const;

    virtual int                 GetLineFeed() const;

    virtual const CharWidths    GetDefaultCharWidths() const;

    //


    //
    //

    virtual void                SetDefaultCharWidths(const CharWidths& widths);

    virtual bool                SetAlternateChar(CharCode c);

    virtual void                SetLineFeed(int linefeed);

    //


    //
    //

    virtual int                 GetCharWidth(
                                    CharCode c                      // Character whose width is to be found
                                ) const;

    virtual const CharWidths    GetCharWidths(
                                    CharCode c                      // Character whose width is to be found
                                ) const;

    virtual void                GetGlyph(
                                    Glyph*      glyph,              // Buffer used to receive glyph information
                                    CharCode    c                   // Character whose glyph information is to be gotten
                                ) const;
    virtual bool                HasGlyph(
                                    CharCode    c                   // Character whose glyph information availability is to be gotten
                                ) const;

    //


    //
    //

    virtual CharacterCode       GetCharacterCode() const;

    //


    //
    //

    virtual int                 GetBaselinePos() const;

    virtual int                 GetCellHeight() const;

    virtual int                 GetCellWidth() const;

    //


    //
    //

    virtual void            EnableLinearFilter(
                                bool    atSmall,
                                bool    atLarge);

    virtual bool            IsLinearFilterEnableAtSmall() const;

    virtual bool            IsLinearFilterEnableAtLarge() const;

    virtual u32             GetTextureWrapFilterValue() const;

    //

private:
    /* ------------------------------------------------------------------------
            Variables
       ------------------------------------------------------------------------ */
    // Pointer to the font instance
    Font*                       m_Primary;
    Font*                       m_Secondary;
    // Whether to get the substitution character from the first font
    bool                        m_AlternateWithPrimary;
    u8                          m_Padding[3];
};




}   // namespace font
}   // namespace nw
#endif //  NW_FONT_PAIRFONT_H_
