/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_FONT_RECTDRAWER_H_
#define NW_FONT_RECTDRAWER_H_

//--------------------------------------
// Operation Settings

// Is it faster to render with glDrawArrays
#define NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS 1

// If saving in the buffer, save and render simultaneously.
#define NW_FONT_RECTDRAWER_USE_DRAW_BUFFER 1


#include <nn/assert.h>
#include <nw/ut/ut_CmdCache.h>
#include <nw/font/font_Types.h>
#if defined(NW_PLATFORM_CTR)
    #include <nn/gx/CTR/gx_CommandAccess.h>
#endif

#include <nw/font/font_RectDrawerCommand.h>

#ifdef NW_PLATFORM_CTR
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
#endif
#define NW_FONT_COMMAND_SET_BLEND_FUNC( eq, src, dst ) \
    PICA_CMD_DATA_COLOR_OPERATION( \
        PICA_DATA_FRAGOP_MODE_DMP, \
        PICA_DATA_ENABLE_BLEND ), \
    PICA_CMD_HEADER_SINGLE( PICA_REG_COLOR_OPERATION ), \
    PICA_CMD_DATA_BLEND_FUNC( eq, src, dst ), \
    PICA_CMD_HEADER_SINGLE( PICA_REG_BLEND_FUNC )

#define NW_FONT_COMMAND_SET_BLEND_DEFAULT \
    NW_FONT_COMMAND_SET_BLEND_FUNC( \
        PICA_DATA_BLEND_EQUATION_ADD, \
        PICA_DATA_BLEND_FUNC_SRC_ALPHA, \
        PICA_DATA_BLEND_FUNC_ONE_MINUS_SRC_ALPHA )

namespace nw {
namespace font {

struct DispStringBuffer;
class CharWriter;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class RectDrawer
{
public:
    //
    //
    //
    //
    //
    enum DrawFlag
    {
        //
        DONT_USE_SETUP_COMMAND = 0x1
    };

    //
    //
    //
    //
    //
    //
    //
    static u32          GetVertexBufferCommandBufferSize(
                            const void* shaderBinary,
                            u32         size);

    //
    //
    //
    //
    static void*        GetVertexBufferData();

    //
    //
    //
    //
    static u32          GetVertexBufferSize();

    //
    //
    //
    //
    //
    //
    //
    static u32          GetCommandBufferSize(
                            const void* shaderBinary,
                            u32         size);

    //
    //

    //
                        RectDrawer();

    //
    //
    //
    //
    //
    //
    //
    //
    virtual             ~RectDrawer();

    //

    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void                Initialize(
                            void*       vtxBufCmdBuf,
                            const void* shaderBinary,
                            u32         size);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void                Initialize(
                            void*       vertexBuffer,
                            void*       commandBuffer,
                            const void* shaderBinary,
                            u32         size);

    //
    //
    //
    //
    //
    //
    virtual void        Finalize();

    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void                SetCommandJumpInfo(void* command, void* sizePtr);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void*                GetCommandJumpInfo(void** commandPtr) const
    {
        if (commandPtr)
        {
            *commandPtr = m_CommandJumpReturnAddr;
        }

        return m_CommandJumpSizePtr;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void                FixCommandJump(void* command);


    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void*               GetCommandJumpFirstReturnAddr() const
    {
        return m_CommandJumpFirstReturnAddr;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void        DrawBegin(u32 flag = 0);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u32*        DrawBegin(u32* pCmdStream, u32 flag = 0);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void        DrawEnd(u32 flag = 0);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u32*        DrawEnd(u32* pCmdStream, u32 flag = 0);

    //
    //
    //
    //
    //
    //
    //
    //
    void                SetProjectionMtx(const nn::math::MTX44& mtx);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32*                SetProjectionMtx(u32* pCmdStream, const nn::math::MTX44& mtx);

    //
    //
    //
    //
    //
    //
    //
    //
    void                SetViewMtxForText(const nn::math::MTX34& mtx);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32*                SetViewMtxForText(u32* pCmdStream, const nn::math::MTX34& mtx);

    //
    //
    //
    //
    //
    //
    void                SetParallax(
                            f32     parallax,
                            f32     dLevel,
                            f32     addDist     = 0.0f
                        );

    //
    //
    //
    //
#if !defined(NW_FONT_ENABLE_EDGE_FONT)
    void                BuildTextCommand(CharWriter* pCharWriter);
#else
    void                BuildTextCommand(CharWriter* pCharWriter, FontCombinerType combiner = FONT_COMBINER_TYPE_DEFAULT );
#endif

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    static u32*         BuildPaddingCommand(u32* command, size_t size);

    //
    //
    //
    //
    //
    //
    static void         BuildReturnCommand(DispStringBuffer* pDispStringBuffer);

    //
    void                UniformAndDrawText(DispStringBuffer* pDispStringBuffer);

    //
    u32*                UseBeginCommand(u32* pCmdStream);

    //
    u32*                UseEndCommand(u32* pCmdStream);

    //
    //
    static u32*         AddTextVertexColorCommand(u32* pCmdStream, const u32* cmdBuf);

    //
    //
#if defined(NW_FONT_ENABLE_EDGE_FONT)
    static u32*         AddTextMappingColorCommand(u32* pCmdStream, u32 white, u32 black, FontCombinerType combiner = FONT_COMBINER_TYPE_DEFAULT);
#else
    static u32*         AddTextMappingColorCommand(u32* pCmdStream, u32 white, u32 black);
#endif

protected:

    //
    static u32          SetUniformCommand(
                            u32*    buffer,
                            u32     num
                        )
    {
#if defined(NW_PLATFORM_CTR)
        buffer[2] = buffer[3];
        buffer[3] = PICA_CMD_HEADER_VS_F32(num);
        buffer[(4 + 4 * num) - 1] = 0;
#else
        NN_ASSERT(false);
        (void)buffer;
#endif
        return sizeof(u32) * (4 + 4 * num);
    }

    //
    u32*                DisableTextures(u32* pCmdStream);

    //
    u32*                AddUniformMtx(u32* pCmdStream);

#if defined(NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS)
    //
    u32                 GetVertexIndexAddressOffset(u32 vtxNum) const;
#endif

    //
    const u32*          GetDrawCommands() const;

    //
    u32                 GetDrawCommandSize() const;

    //
    //
    //
    //
    //
    //
    static u32*         GetCurrentCommandBuffer();

    //
    //
    //
    //
    //
    static void         SetCurrentCommandBuffer(u32 *pCmdStream);

private:
    static void         InitializeStatic();

    void                InitializeCMD(
                            void*       vertexBuffer,
                            void*       commandBuffer,
                            const void* shaderBinary,
                            u32         size,
                            bool        initVertexBuffer);

    void                InitUniformBuffer(
                            u32* __restrict buf,
                            u32             addr);

    void                AddTextColor(
                            math::VEC4* __restrict          dst,
                            const ut::Color8* __restrict    src,
                            int                             mode);

protected:
    ////////////////////////////// Vertex Shader Info

    //
    enum
    {
        UNIFORM_MTX_USE_MAX     =  3 /* mtx row */ + 4 /* vtx color */,
        UNIFORM_DATA_USE_MAX    =  1 /* rect */ + 3/* TexMapMax */ * 2,

        UNIFORM_TEXT_MTX_MAX    =  3,   // mtx34 row

        UNIFORM_PROJ_NUM        =  4,   // mtx34 row

        UNIFORM_PROJ_START      =  0,
        UNIFORM_ULCDDATA_START  = UNIFORM_PROJ_START + UNIFORM_PROJ_NUM,
        UNIFORM_CONST_START,
        UNIFORM_ADDR_START,
        UNIFORM_MTX_START       = 32,
        UNIFORM_DATA_START      = 64,

        UNIFORM_TEXTCOLOR_START = UNIFORM_MTX_START + UNIFORM_TEXT_MTX_MAX,

        UNIFORM_MAX             = 96
    };

    //
    enum
    {
#ifdef NW_FONT_RECTDRAWER_USE_DRAW_BUFFER

        UNIFORM_ADDR_NUM        = UNIFORM_MTX_START  - UNIFORM_ADDR_START,
        UNIFORM_MTX_NUM         = UNIFORM_DATA_START - UNIFORM_MTX_START,
        UNIFORM_DATA_NUM        = UNIFORM_MAX        - UNIFORM_DATA_START,

        UNIFORM_TEXTCOLOR_NUM   = (UNIFORM_DATA_START - UNIFORM_TEXTCOLOR_START) & ~((1 << 2) - 1),
#else

        UNIFORM_RESISTER_ADDR = 5,

        // A maximum of 8 registers if not rendering all together.
        UNIFORM_DATA_NUM  = 8,

        UNIFORM_ADDR_NUM      = UNIFORM_DATA_NUM - UNIFORM_RESISTER_ADDR,
#endif

        DRAW_VTX_NUM          = 6,
        BIND_ATTRIB_LOCATION  = 0
    };


    ////////////////////////////// Command

    //
    ut::internal::CmdCache
                        m_CmdCacheDrawBegin;

    //
    ut::internal::CmdCache
                        m_CmdCacheDrawEnd;

    //
    u8                  m_UniformAddrIndex;

    //
    u8                  m_UniformMtxIndex;

    //
    u8                  m_UniformDataIndex;

    //
    u8                  m_UniformTextColorIndex;

    //
    u32                 m_UniformProjBuffer[ 4 + 4 * (UNIFORM_PROJ_NUM + 1) ];

    //
    u32                 m_UniformAddrBuffer[ 4 + 4 * UNIFORM_ADDR_NUM ];

    //
    union
    {
        //
        u32                 m_UniformMtxBuffer[ 4 + 4 * UNIFORM_MTX_NUM  ];

        //
        struct
        {

            //
            u32                 m_UniformTextMtxBuffer  [ 4 + 4 * UNIFORM_TEXT_MTX_MAX ];

            //
            u32                 m_UniformTextColorBuffer[ 4 + 4 * UNIFORM_TEXTCOLOR_NUM ];
        };
    };

    //
    u32                 m_UniformDataBuffer[ 4 + 4 * UNIFORM_DATA_NUM ];

    //
    struct UniformAddress
    {
        //
                            UniformAddress()
                            :   addrMtx(0.f),
                                addrSizeAndVtx(0.f),
                                addrTexCoord(0.f),
                                addrColor(0.f)
                            {}

        //
        f32                 addrMtx;

        //
        f32                 addrSizeAndVtx;

        //
        f32                 addrTexCoord;

        //
        f32                 addrColor;
    };

    //
    UniformAddress      m_UniformAddress;

    //
    UniformAddress *const
                        m_UniformAddr;

    //
    math::VEC4 *const   m_UniformMtx;

    //
    math::VEC4 *const   m_UniformData;

    //
    math::VEC4 *const   m_UniformTextColor;

    //
    void*               m_CommandBuffer;

    ////////////////////////////// Vertex Buffer Object

    //
    void*               m_VertexBufferArray;

    ////////////////////////////// Uniform

#if defined(NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS)
    //
    u32                 m_VertexBufferBaseAddr;
#endif

    //
    void*               m_CommandJumpFirstReturnAddr;

    //
    void*               m_CommandJumpReturnAddr;

    //
    void*               m_CommandJumpSizePtr;

    //
    //
    u8                  m_IsColorDirty;

    //
    u8 padding[3];

private:
    //

    struct VertexBuffer
    {
#if defined(NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS)
        f32                 position[UNIFORM_ADDR_NUM][DRAW_VTX_NUM][4];
#else
        f32                 position[UNIFORM_ADDR_NUM][VERTEX_MAX][4];
        u16                 indexes[UNIFORM_ADDR_NUM][DRAW_VTX_NUM];
#endif
    };

    //

    static bool         s_StaticInitialized;
    static VertexBuffer s_VertexBuffer;

    friend class CharWriter;
};

inline
u32
RectDrawer::GetVertexBufferSize()
{
    return sizeof(s_VertexBuffer);
}

namespace internal
{

#if defined(NW_PLATFORM_CTR)
inline
u32*
SetVertexNumCmd(
    u32*    pCmdStream,
    u32     addrOffset,
    u32     vtxNum
    )
{
#if defined(NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS)
    // Address/type of the vertex index
    // 0x227
    NW_FONT_ADD_SINGLE_COMMAND_PTR(
        pCmdStream,
        PICA_CMD_HEADER_SINGLE( PICA_REG_INDEX_ARRAY_ADDR_OFFSET ),
        PICA_CMD_DATA_INDEX_ARRAY_ADDR_OFFSET( addrOffset, 1 /* type */ ));
#else
    (void)addrOffset;
#endif

    // Number of vertices to draw
    // 0x228
    NW_FONT_ADD_SINGLE_COMMAND_PTR(
        pCmdStream,
        PICA_CMD_HEADER_SINGLE( PICA_REG_DRAW_VERTEX_NUM ),
        PICA_CMD_DATA_DRAW_VERTEX_NUM( vtxNum ));

    return pCmdStream;
}
#endif

}   // namespace internal

}   // namespace font
}   // namespace nw
#ifdef NW_PLATFORM_CTR
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif
#endif

#endif //  NW_FONT_RECTDRAWER_H_
