/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/
/*                                                                              */
/*  Copyright (c) 2009, Digital Media Professionals Inc. All rights reserved.   */
/*                                                                              */
/*------------------------------------------------------------------------------*/

#include <nw/gfx/gfx_GlImplement.h>


//===============================================================
//
// WARNING: Do not directly reference the API or structure contained in this header.
//
//===============================================================

namespace nw {
namespace gfx {
namespace internal {

//---------------------------------------------------------------------------
void
nwgfxAddVramDmaCommand(void* srcaddr, void* dstaddr, GLsizei size)
{
    nngxAddVramDmaCommand( srcaddr, dstaddr, size);
}

//---------------------------------------------------------------------------
void
nwgfxClear(
    u32 colorAddr, u32 colorSize, u32 clearColor, u32 colorWidth,
    u32 depthAddr, u32 depthSize, u32 clearDepth, u32 depthWidth
)
{
    nngxAddMemoryFillCommand(
        reinterpret_cast<GLvoid*>(colorAddr), colorSize, clearColor, colorWidth,
        reinterpret_cast<GLvoid*>(depthAddr), depthSize, clearDepth, depthWidth
    );
}

//---------------------------------------------------------------------------
void
GetFrameBufferState(
    GLuint fboID,
    u32* pColorAddr,
    u32* pDepthAddr
)
{
    glBindFramebuffer( GL_FRAMEBUFFER, fboID );

    if ( pColorAddr )
    {
        s32 renderID;
        s32 addr;

        glGetFramebufferAttachmentParameteriv(
            GL_FRAMEBUFFER,
            GL_COLOR_ATTACHMENT0,
            GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
            &renderID );

        glBindRenderbuffer(GL_RENDERBUFFER, renderID);

        glGetRenderbufferParameteriv(
            GL_RENDERBUFFER,
            GL_RENDERBUFFER_DATA_ADDR_DMP,
            &addr );

        *pColorAddr = static_cast<u32>(addr);
    }

    if ( pDepthAddr )
    {
        s32 objectType;

        glGetFramebufferAttachmentParameteriv(
            GL_FRAMEBUFFER,
            GL_DEPTH_ATTACHMENT,
            GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE,
            &objectType );

        // If the depth buffer has been generated
        if (objectType != GL_NONE)
        {
            s32 renderID;
            s32 addr;

            glGetFramebufferAttachmentParameteriv(
                GL_FRAMEBUFFER,
                GL_DEPTH_ATTACHMENT,
                GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME,
                &renderID );

            glBindRenderbuffer(GL_RENDERBUFFER, renderID);

            glGetRenderbufferParameteriv(
                GL_RENDERBUFFER,
                GL_RENDERBUFFER_DATA_ADDR_DMP,
                &addr );

            *pDepthAddr = static_cast<u32>(addr);
        }
        else
        {
            *pDepthAddr = NULL;
        }
    }
}

} // namespace internal
} // namespace gfx
} // namespace nw


