/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/gfx/gfx_ParticleRandom.h>

namespace nw
{
namespace gfx
{

#ifndef NW_GFX_DISABLE_ASM
#include <nn/hw/ARM/code32.h>
#include <nn/hw/ARM/VFPv2_reg.h>
#endif

#ifdef NW_GFX_DISABLE_ASM
f32
ParticleRandom::NextFloatSignedOne()
{
    m_Seed = (m_Seed * 214013u) + 2531011u;

    f32 sign;
    if ((m_Seed & 0x80000000) == 0)
    {
        sign = 1.0f;
    }
    else
    {
        sign = -1.0f;
    }

    uint work = (m_Seed >> 16) & 0x7fff;

    const f32 BaseValue = 1.0f / 32768;

    if (work == 0)
    {
        return sign * BaseValue * 0.25f;
    }

    return sign * BaseValue * work;
}
#else
//----------------------------------------
asm f32
ParticleRandom::NextFloatSignedOne()
{
    LDR      r2, [r0, #__cpp(offsetof(ParticleRandom, m_Seed))]

    LDR      r3, mix1
    LDR      r1, mix2
    MLA      r1, r2, r3, r1

      LDR      r3, mask
    STR      r1, [r0, #0]
      AND      r2,r3,r1,LSR #16

        CLZ      r0, r2
      AND      r12, r1, #0x80000000
        LSL      r2, r2, r0 // Shift to the left until the uppermost bit becomes 1
          RSB      r1, r0, #142 // When the uppermost bit (17), it becomes 126 (-1 + 127)
        ADD      r2, r12, r2, LSR #8
          ADD      r2, r2, r1, LSL #23 /// Set the exponent portion
          VMOV     s0, r2

    BX       lr

mix1 DCD      0x343fd
mix2 DCD      0x269ec3
mask DCD      0x00007fff
}
#endif

#ifdef NW_GFX_DISABLE_ASM
f32
ParticleRandom::NextFloatSignedHalf()
{
    return NextFloatSignedOne() * 0.5f;
}

#else
//----------------------------------------
asm f32
ParticleRandom::NextFloatSignedHalf()
{
    LDR      r2, [r0, #__cpp(offsetof(ParticleRandom, m_Seed))]

    LDR      r3, mixh1
    LDR      r1, mixh2
    MLA      r1, r2, r3, r1

      LDR      r3, maskh
    STR      r1, [r0, #0]
      AND      r2,r3,r1,LSR #16

        CLZ      r0, r2
      AND      r12, r1, #0x80000000
        LSL      r2, r2, r0 // Shift to the left until the uppermost bit becomes 1
          RSB      r1, r0, #141 // When the uppermost bit (17), it becomes 126 (-1 + 127)
        ADD      r2, r12, r2, LSR #8
          ADD      r2, r2, r1, LSL #23 /// Set the exponent portion
          VMOV     s0, r2

    BX       lr

mixh1 DCD      0x343fd
mixh2 DCD      0x269ec3
maskh DCD      0x00007fff
}
#endif

#ifndef NW_GFX_DISABLE_ASM
#include <nn/hw/ARM/codereset.h>
#endif

} // namespace gfx
} // namespace nw
