﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/gfx/gfx_SceneTraverser.h>
#include <nw/gfx/gfx_SceneContext.h>
#include <nw/gfx/gfx_SceneNode.h>
#include <nw/gfx/gfx_TransformNode.h>
#include <nw/gfx/gfx_Model.h>
#include <nw/gfx/gfx_SkeletalModel.h>
#include <nw/gfx/gfx_Camera.h>
#include <nw/gfx/gfx_Fog.h>
#include <nw/gfx/gfx_FragmentLight.h>
#include <nw/gfx/gfx_VertexLight.h>
#include <nw/gfx/gfx_AmbientLight.h>
#include <nw/gfx/gfx_HemiSphereLight.h>
#include <nw/gfx/gfx_ParticleSet.h>
#include <nw/gfx/gfx_IMaterialIdGenerator.h>

namespace nw
{
namespace gfx
{

NW_UT_RUNTIME_TYPEINFO_DEFINITION(SceneInitializer, ISceneVisitor);

//----------------------------------------
SceneInitializer*
SceneInitializer::Builder::Create(
    os::IAllocator* allocator
)
{
    NW_NULL_ASSERT(allocator);

    void* memory = allocator->Alloc(sizeof(SceneInitializer));
    NW_NULL_ASSERT(memory);

    NW_ASSERTMSG(
        m_Description.materialIdGenerator != NULL, "SceneInitializer need to set an IMaterialIdGenerator instance.");

    SceneInitializer* initializer = new(memory) SceneInitializer(allocator, m_Description);

    return initializer;
}

//----------------------------------------
void SceneInitializer::VisitModel(Model* model)
{
    if (m_MaterialIdGenerator.Get() != NULL)
    {
        nw::gfx::MaterialArray::iterator materialEnd = model->GetMaterials().second;
        for (nw::gfx::MaterialArray::iterator material = model->GetMaterials().first;
            material != materialEnd;
            ++material)
        {
            m_MaterialIdGenerator.Get()->Accept(*material);
        }
    }
}

//----------------------------------------
void SceneInitializer::VisitSkeletalModel(SkeletalModel* model)
{
    if (m_MaterialIdGenerator.Get() != NULL)
    {
        nw::gfx::MaterialArray::iterator materialEnd = model->GetMaterials().second;
        for (nw::gfx::MaterialArray::iterator material = model->GetMaterials().first;
            material != materialEnd;
            ++material)
        {
            m_MaterialIdGenerator.Get()->Accept(*material);
        }
    }
}

//----------------------------------------
void SceneInitializer::VisitParticleModel(ParticleModel* model)
{
    if (m_MaterialIdGenerator.Get() != NULL)
    {
        nw::gfx::MaterialArray::iterator materialEnd = model->GetMaterials().second;
        for (nw::gfx::MaterialArray::iterator material = model->GetMaterials().first;
            material != materialEnd;
            ++material)
        {
            m_MaterialIdGenerator.Get()->Accept(*material);
        }
    }
}

//----------------------------------------
void SceneInitializer::Begin()
{
}

//----------------------------------------
void SceneInitializer::End()
{
    m_MaterialIdGenerator.Get()->Generate();
}

} // namespace gfx
} // namespace nw
