/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/ut/ut_ResDictionary.h>

namespace nw {
namespace ut {


s32
ResDicLinear::GetIndex(const char* key) const
{
    if ( (! this->IsValid()) || (! key) ) { return -1; }

    const ResDicLinearData::ResDicNodeData* x = ref().data;

    for ( int i = 0; i < s32(ref().numData); ++i )
    {
        if ( x->toName.offset != 0 && std::strcmp(key, x->toName.to_ptr()) == 0 )
        {
            return static_cast<s32>( i );
        }
        ++x;
    }
    return -1;
}

ResDicPatriciaData::ResDicNodeData*
ResDicPatricia::Get(const char* s, size_t len) const
{
    const ResDicPatriciaData::ResDicNodeData* p;
    const ResDicPatriciaData::ResDicNodeData* x;
    const ResDataType& r = this->ref();

    p = &r.data[0];
    x = &r.data[p->idxLeft];

    while(p->ref > x->ref)
    {
        p = x;
        // If the bit is 1, trace right; if the bit is 0, trace left.

        u32 wd = u32(x->ref) >> 3;
        u32 pos = u32(x->ref) & 7;

        if (wd < len && ((s[wd] >> pos) & 1))
        {
            x = &r.data[x->idxRight];
        }
        else
        {
            x = &r.data[x->idxLeft];
        }
    }

    if (x->ofsString != 0 && !std::strcmp(s, x->ofsString.to_ptr<char>()))
    {
        return const_cast<ResDicPatriciaData::ResDicNodeData*>(x);
    }
    return NULL;
}



/* ------------------------------------------------------------------------
    ResDic
   ------------------------------------------------------------------------ */

} /* namespace ut */
} /* namespace nw */

