﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_DRAWER_H_
#define NW_LYT_DRAWER_H_

// Enables accelerated testing.
#ifdef NW_PLATFORM_CTR
#define NW_LYT_DRAWER_ENABLE 1
#endif

#include <nw/font/font_RectDrawer.h>

#ifdef NW_LYT_DRAWER_ENABLE
//--------------------------------------
// Operation Settings

// ASSERT
#if 1
#define NW_LYT_DRAWER_GL_ASSERT()      NW_GL_ASSERT()
#define NW_LYT_DRAWER_ASSERT( x )      NW_ASSERT( x )
#define NW_LYT_DRAWER_NULL_ASSERT( x ) NW_NULL_ASSERT( x )
#else
#define NW_LYT_DRAWER_ASSERT( x )
#define NW_LYT_DRAWER_GL_ASSERT()
#define NW_LYT_DRAWER_NULL_ASSERT( x )
#endif

// Might be faster is unnecessary features are removed.
#define NW_LYT_DRAWER_ENABLE_ALPHA_TEST   1
#define NW_LYT_DRAWER_ENABLE_BLEND_FUNC   1
#define NW_LYT_DRAWER_ENABLE_USER_TEX_ENV 1

//----------------------------------------------------------
// Macros

// nngxValidateState
#ifdef NW_FONT_RECTDRAWER_USE_DRAW_ARRAYS
#define NW_LYT_DRAWER_VALIDATE( state_mask ) \
    nngxValidateState( state_mask, GL_FALSE )
#else
#define NW_LYT_DRAWER_VALIDATE( state_mask ) \
    nngxValidateState( state_mask, GL_TRUE )
#endif

#if 0
#define NW_LYT_DRAWER_ALIGN_32 __attribute__((aligned(32)))
#else
#define NW_LYT_DRAWER_ALIGN_32
#endif

//----------------------------------------------------------
// For testing.

// Disables Picture Draw.
// #define NW_LYT_DRAWER_DISABLE_DRAW_PICTURE           1

// Disables TextBox Draw.
// #define NW_LYT_DRAWER_DISABLE_DRAW_TEXTBOX           1

// Disables Window Draw.
// #define NW_LYT_DRAWER_DISABLE_DRAW_WINOOW            1

// Prints UniformData.
// #define NW_LYT_DRAWER_PRINT_UNIFORM_DATA             1


//----------------------------------------------------------

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <nw/lyt/lyt_Types.h>
#include <nw/ut/ut_Color.h>

namespace nw { namespace lyt {

class Layout;
class Pane;
class Material;
class DrawInfo;
class Material;
class TexMap;
class TextBox;
class GraphicsResource;

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class Drawer : public font::RectDrawer
{
    typedef RectDrawer  Base;

public:

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    Drawer();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit Drawer( GraphicsResource& graphicsResource );

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Initialize( GraphicsResource& graphicsResource,
                     void* vertexBuffer = NULL );

protected:
    //
    void Initialize( void*       vtxBufCmdBuf,
                     const void* shaderBinary,
                     u32         size
    )
    {
        Base::Initialize(vtxBufCmdBuf, shaderBinary, size);
    }

    //
    void Initialize( void*       vertexBuffer,
                     void*       commandBuffer,
                     const void* shaderBinary,
                     u32         size
    )
    {
        Base::Initialize(vertexBuffer, commandBuffer, shaderBinary, size);
    }

public:
    //
    //
    //
    //
    //
    //
    virtual void        Finalize();

    //

public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void DrawBegin( const DrawInfo& drawInfo, u32 flag = 0 );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32* DrawBegin( u32* pCmdStream, const DrawInfo& drawInfo, u32 flag = 0 );

    //
    NW_DEPRECATED_FUNCTION(virtual void DrawBegin(u32 flag = 0));

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Draw( const Layout* pLayout, DrawInfo& drawInfo );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32* Draw( u32* pCmdStream, const Layout* pLayout, DrawInfo& drawInfo );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void Draw( const Pane* pPane, DrawInfo& drawInfo );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32* Draw( u32* pCmdStream, const Pane* pPane, DrawInfo& drawInfo );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void DrawEnd( const DrawInfo& drawInfo, u32 flag = 0 );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u32* DrawEnd( u32* pCmdStream, const DrawInfo& drawInfo, u32 flag = 0 );

    //
    NW_DEPRECATED_FUNCTION(virtual void DrawEnd(u32 flag = 0));

protected:
    virtual u32* DrawBegin(u32* pCmdStream, u32 flag);

    virtual u32* DrawEnd(u32* pCmdStream, u32 flag);

    //

public:

    //
    void DumpCmdList();

    //
    u32* SetUpTexEnv(
        u32* pCmdStream,
        const Material* pMaterial );

    //
    u32* SetUpTextures(
        u32* pCmdStream,
        const Material* pMaterial,
        bool addDisableCommand = true );

    //
    u32* SetUpTextureCoords(
        u32* pCmdStream,
        const nw::math::VEC4 texCoords[ TexMapMax * 2 ],
        const int texCoordNum );

    //
    void SetUpVtxColors( const nw::ut::Color8 vtxColors[VERTEX_MAX],
                         const u8 globalAlpha );

    //
#if !defined(NW_FONT_ENABLE_EDGE_FONT)
    u32* SetUpTextBox(
        u32* pCmdStream,
        const TextBox* pTextBox,
        const Material* pMaterial,
        const DrawInfo& drawInfo );
#else
    u32* SetUpTextBox(
        u32* pCmdStream,
        const TextBox* pTextBox,
        const Material* pMaterial,
        const DrawInfo& drawInfo,
        font::FontCombinerType combiner = font::FONT_COMBINER_TYPE_DEFAULT );
#endif

    //
    void SetUpMtx( const nw::math::MTX34& mtx );

    //
    void SetUpQuad( const nw::lyt::Size& size,
                    const nw::math::VEC2& basePt );

    //
    u32* SetUniformDataEnd(u32* pCmdStream);

    //
    void SetCurrentUniformAlpha( const f32 alpha );

    //
    u8 CalcTextureCoords( const Material* pMaterial,
                          const TexCoordQuad* pTexCoordQuad,
                          nw::math::VEC4 texCoords[ TexMapMax * 2 ] ) const;

    //
    //
    //
    //
    //
    //
    //
    //
    u32* FlushBuffer(u32* pCmdStream)
    {
#ifdef NW_FONT_RECTDRAWER_USE_DRAW_BUFFER
        if (m_UniformAddrIndex > 0)
        {
            pCmdStream = UniformAndDraw(pCmdStream);
        }
#endif
        return pCmdStream;
    }

    //
    //
    void FlushBuffer()
    {
        SetCurrentCommandBuffer(FlushBuffer(GetCurrentCommandBuffer()));
    }

protected:
    //
    u32* UniformAndDraw(u32* pCmdStream);

    //
    u32* SetUpTexEnvType2( u32* pCmdStream, const Material* pMaterial );

    //
    u32* SetUpTexEnvType3( u32* pCmdStream, const Material* pMaterial );

#ifdef NW_LYT_DRAWER_ENABLE_USER_TEX_ENV
    //
    u32* SetUpGLTexEnvUser( u32* pCmdStream, const Material* pMaterial );
#endif

#ifdef NW_LYT_DRAWER_ENABLE_BLEND_FUNC
    //
    u32* SetUpBlendMode( u32* pCmdStream, const Material* pMaterial );
#endif

#ifdef NW_LYT_DRAWER_ENABLE_ALPHA_TEST
    //
    u32* SetUpAlphaTest( u32* pCmdStream, const Material* pMaterial );
#endif

protected: ////////////////////////////// Command
    //
    GLuint m_PrevTexObj;

    //
    int m_ActiveTexureNum;

    //
    int m_TexCoordNum;

    //
    u8 m_IsBlendDefault;

    //
    u8 m_IsAlphaTestDefault;

protected: ////////////////////////////// Texture Combiner

    //
    enum TEX_ENV_TYPE
    {
        TEX_ENV_TYPE_0_TEX,
        TEX_ENV_TYPE_1_TEX,
        TEX_ENV_TYPE_2_TEX,
        TEX_ENV_TYPE_3_TEX,
        TEX_ENV_TYPE_TEXT,
        TEX_ENV_TYPE_USER,
        TEX_ENV_TYPE_NUM
    };

    //
    u8 m_CurrentTexEnvType; // TEX_ENV_TYPE

#if ! defined(NW_RELEASE)
    //
    u8 m_PreviousTexEnvType; // TEX_ENV_TYPE
#else
    //
    u8 padding[1];
#endif

    //
    enum
    {
        TEX_ENV_UNIFORM_NUM = 8,
#ifdef NW_TARGET_CTR_GL_FINAL
        TEX_ENV_STAGE_NUM = 6,
#else
        TEX_ENV_STAGE_NUM = 3,
#endif
        TEX_ENV_UNIFORM_VALUE_NUM = 16
    };
};

}}

#endif // NW_LYT_DRAWER_ENABLE
#endif // NW_LYT_DRAWER_H_
