/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_GROUP_H_
#define NW_LYT_GROUP_H_

#include <cstddef>
#include <nw/ut/ut_LinkList.h>

#include <nw/lyt/lyt_Pane.h>
#include <nw/lyt/lyt_Types.h>

namespace nw
{
namespace lyt
{

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
struct PaneLink
{
    //
    //
    ut::LinkListNode link;

    //
    Pane* target;
};

//
//

//
//
//
//
typedef ut::LinkList<PaneLink, offsetof(PaneLink, link)> PaneLinkList;

//

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class Group
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    Group();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Group(const res::Group* pResGroup, Pane* pRootPane);

    //
    //
    //
    //
    virtual ~Group();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    const char* GetName() const
    {
        return m_Name;
    }

    //
    //
    //
    //
    //
    //
    PaneLinkList& GetPaneList()
    {
        return m_PaneLinkList;
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    void AppendPane(Pane* pPane);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    bool IsUserAllocated() const
    {
        return m_UserAllocated != 0;
    }

    //
    //
    //
    //
    //
    //
    //
    void SetUserAllocated()
    {
        m_UserAllocated = 1;
    }

    //

    //
    //
    ut::LinkListNode m_Link_;

protected:
    //
    void Init();

    //
    PaneLinkList m_PaneLinkList;

    //
    char m_Name[ResourceNameStrMax + 1];

    //
    //
    u8 m_UserAllocated;

    //
    u8 m_Padding[2];
};

//
//

//
//
//
//
typedef ut::LinkList<Group, offsetof(Group, m_Link_)> GroupList;

//

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class GroupContainer
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    GroupContainer()
    {}

    //
    //
    //
    //
    ~GroupContainer();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    void AppendGroup(Group* pGroup);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    GroupList& GetGroupList()
    {
        return m_GroupList;
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    Group* FindGroupByName(const char* findName);

    //

protected:
    //
    GroupList m_GroupList;

};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_GROUP_H_

