/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_TEXRESOURCE_H_
#define NW_LYT_TEXRESOURCE_H_

#include <nw/lyt/lyt_Arc.h>
#include <nw/lyt/lyt_Resources.h>
#include <nw/lyt/lyt_Types.h>

namespace nw
{
namespace lyt
{

//---------------------------------------------------------------------------
//
//
//
//
//
//
//
//
//
//
//
//
//---------------------------------------------------------------------------
class TexResource
{
public:
    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    TexResource()
    : m_pTop(NULL)
    , m_pImage(NULL)
    , m_pImageSize(NULL)
    {}

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TexResource(void* pTexRes, u32 size)
    {
        this->Set(pTexRes, size);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit TexResource(ARCFileInfo& fileInfo)
    {
        this->Set(fileInfo);
    }

    // @}

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    bool IsValid() const
    {
        return m_pTop != NULL;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool Set(void* pTexRes, u32 size);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool Set(ARCFileInfo& fileInfo)
    {
        return this->Set(
            ARCGetStartAddrInMem(&fileInfo),
            ARCGetLength(&fileInfo));
    }

    //
    //
    //
    //
    //
    //
    const void* GetImageAddress() const
    {
        NW_NULL_ASSERT(m_pTop);

        return m_pTop;
    }

    //
    //
    //
    //
    //
    //
    void* GetImageAddress()
    {
        NW_NULL_ASSERT(m_pTop);

        return m_pTop;
    }

    //
    //
    //
    //
    //
    //
    u32 GetImageSize() const
    {
        NW_NULL_ASSERT(m_pImageSize);

        return m_pImageSize->imageSize;
    }

    //
    //
    //
    //
    //
    //
    lyt::TexFormat GetFormat() const
    {
        NW_NULL_ASSERT(m_pImage);

        return static_cast<lyt::TexFormat>(m_pImage->format);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool Is4bitFormat() const
    {
        NW_NULL_ASSERT(m_pImage);

        return m_pImage->format == lyt::TEXFORMAT_L4
            || m_pImage->format == lyt::TEXFORMAT_A4;
    }

    //
    //
    //
    //
    //
    //
    u16 GetWidth() const
    {
        NW_NULL_ASSERT(m_pImage);

        return m_pImage->width;
    }

    //
    //
    //
    //
    //
    //
    u16 GetHeight() const
    {
        NW_NULL_ASSERT(m_pImage);

        return m_pImage->height;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    MemArea GetImageArea() const
    {
        NW_NULL_ASSERT(m_pImage);

        return static_cast<lyt::MemArea>(
            internal::GetBits(
                m_pImage->flag,
                lyt::IMAGEFLAG_AREA_POS,
                lyt::IMAGEFLAG_AREA_LEN));
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetImageArea(lyt::MemArea area)
    {
        NW_NULL_ASSERT(m_pImage);

        m_pImage->flag = internal::SetBits(
            m_pImage->flag,
            lyt::IMAGEFLAG_AREA_POS,
            lyt::IMAGEFLAG_AREA_LEN,
            static_cast<u8>(area));
    }

    //
    //
    //
    //
    //
    //
    res::Image* GetResImage()
    {
        return m_pImage;
    }

    //
    //
    //
    //
    //
    //
    const res::Image* GetResImage() const
    {
        return m_pImage;
    }

protected:
    //
    void* m_pTop;

    //
    res::Image* m_pImage;

    //
    res::ImageSize* m_pImageSize;
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_TEXRESOURCE_H_
