/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_TEXTBOX_H_
#define NW_LYT_TEXTBOX_H_

#include <nw/ut/ut_Color.h>
#include <nw/ut/ut_RuntimeTypeInfo.h>
#include <nw/font/font_DispStringBuffer.h>

#include <nw/lyt/lyt_Config.h>
#include <nw/lyt/lyt_Pane.h>

namespace nw
{
namespace font
{

class Font;

template <typename CharType>
class TagProcessorBase;

template <typename CharType>
class TextWriterBase;

} // namespace nw::font

namespace lyt
{

class DrawInfo;
struct ResBlockSet;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class TextBox : public Pane
{
    typedef Pane Base;

public:
    //
    NW_UT_RUNTIME_TYPEINFO;

    //
    typedef font::TagProcessorBase<wchar_t> TagProcessor;

    //
    typedef nw::font::DispStringBuffer::CommandBufferMode CommandBufferMode;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    explicit TextBox(u16 allocStrLen = 0,
        CommandBufferMode cmdBuffMode = font::DispStringBuffer::CBM_DYNAMIC);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TextBox(
        u16 allocStrLen,
        const wchar_t* str,
        const font::Font* pFont,
        CommandBufferMode cmdBuffMode = font::DispStringBuffer::CBM_DYNAMIC);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TextBox(
        u16 allocStrLen,
        const wchar_t* str,
        u16 strLen,
        const font::Font* pFont,
        CommandBufferMode cmdBuffMode = font::DispStringBuffer::CBM_DYNAMIC);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TextBox(
        const res::TextBox* pBlock,
        const ResBlockSet& resBlockSet,
        CommandBufferMode cmdBuffMode = font::DispStringBuffer::CBM_DYNAMIC);

    //
    //
    //
    //
    //
    //
    //
    //
    virtual ~TextBox();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const wchar_t* GetString() const
    {
        return m_TextBuf;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const wchar_t* GetStringBuffer() const
    {
        return m_TextBuf;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    u16 GetStringLength() const
    {
        return m_TextLen;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    u16 GetStringBufferLength() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void AllocStringBuffer(
        u16 minLen,
        CommandBufferMode cmdBuffMode = font::DispStringBuffer::CBM_DYNAMIC);

    //
    //
    //
    //
    //
    //
    virtual void FreeStringBuffer();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u16 SetString(
        const wchar_t*  str,
        u16             dstIdx = 0);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u16 SetString(
        const wchar_t*  str,
        u16             dstIdx,
        u16             strLen);

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const ut::Color8 GetTextColor(u32 type) const
    {
        NW_ASSERT(type < TEXTCOLOR_MAX); return m_TextColors[type];
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextColor(u32 type, ut::Color8 value)
    {
        NW_ASSERT(type < TEXTCOLOR_MAX);

        m_TextColors[type] = value;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextColor(
        ut::Color8 top,
        ut::Color8 bottom
    )
    {
        m_TextColors[TEXTCOLOR_TOP   ] = top;
        m_TextColors[TEXTCOLOR_BOTTOM] = bottom;
    }

    //
    //
    //
    //
    //
    //
    //
    const font::Font* GetFont() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetFont(const font::Font* pFont);

    //
    //
    //
    //
    //
    //
    //
    //
    const Size& GetFontSize() const
    {
        return m_FontSize;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetFontSize(const Size& fontSize)
    {
        if (UpdatePTDirty(!(m_FontSize == fontSize)))
        {
            m_FontSize = fontSize;
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetLineSpace() const
    {
        return m_LineSpace;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetLineSpace(f32 space)
    {
        if (UpdatePTDirty(m_LineSpace != space))
        {
            m_LineSpace = space;
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetCharSpace() const
    {
        return m_CharSpace;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetCharSpace(f32 space)
    {
        if (UpdatePTDirty(m_CharSpace != space))
        {
            m_CharSpace = space;
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    HorizontalPosition GetTextPositionH() const
    {
        return internal::GetHorizontalPosition(m_TextPosition);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextPositionH(HorizontalPosition val)
    {
        if (UpdatePTDirty(GetTextPositionH() != val))
        {
            m_TextPosition = internal::SetHorizontalPosition(m_TextPosition, u8(val));
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    VerticalPosition GetTextPositionV() const
    {
        return internal::GetVerticalPosition(m_TextPosition);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextPositionV(VerticalPosition val)
    {
        if (UpdatePTDirty(GetTextPositionV() != val))
        {
            m_TextPosition = internal::SetVerticalPosition(m_TextPosition, u8(val));
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    TextAlignment GetTextAlignment() const
    {
        return static_cast<TextAlignment>(m_TextAlignment);
    }

    //
    //
    //
    //
    //
    //
    //
    //
    void SetTextAlignment(TextAlignment val)
    {
        if (UpdatePTDirty(GetTextAlignment() != val))
        {
            m_TextAlignment = val;
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    TagProcessor* GetTagProcessor() const
    {
        return m_pTagProcessor;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTagProcessor(TagProcessor* pTagProcessor)
    {
        if (UpdatePTDirty(m_pTagProcessor != pTagProcessor))
        {
            m_pTagProcessor = pTagProcessor;
        }
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual const ut::Color8 GetVtxColor(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColor(u32 idx, ut::Color8 value);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u8 GetVtxColorElement(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColorElement(u32 idx, u8 value);

#ifdef NW_LYT_DRAWER_ENABLE
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void UpdateDrawCharData(Drawer* pDrawer)
    {
        if (m_TextLen <= 0 || !m_pFont)
        {
            return;
        }

        UpdateDrawCharDataImpl(pDrawer);
    }
#endif

    //
    //
    //
    //
    //
    //
    const ut::Rect GetTextDrawRect() const;

    using Base::GetMaterial;

    //
    //
    //
    //
    //
    //
    virtual u8 GetMaterialNum() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual Material* GetMaterial(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetMaterial(Material* pMaterial);

#if defined(NW_FONT_ENABLE_EDGE_FONT)
    //
    //
    //
    //
    //
    //
    //
    //
    void SetCombinerType(font::FontCombinerType combiner) { m_CombinerType = combiner; }
#endif

    //

    //
    font::DispStringBuffer* GetDispStringBuffer() const
    {
        return m_pDispStringBuf;
    }

    //
    void GetTextGlobalMtx(nw::math::MTX34* pMtx) const;

#ifdef NW_LYT_DRAWER_ENABLE
    virtual u32* MakeUniformDataSelf( u32* pCmdStream, DrawInfo* pDrawInfo, Drawer* pDrawer ) const;
#endif

protected:
#ifdef NW_LYT_DMPGL_ENABLED
    virtual void DrawSelf(const DrawInfo& drawInfo);
    virtual void LoadMtx(const DrawInfo& drawInfo);
#endif

protected:
    //
    void SetFontInfo(font::WideTextWriter* pWriter) const;

    //
    void SetTextPos(font::WideTextWriter* pWriter) const;

    //
    math::VEC2  AdjustTextPos(
                    const Size& size,
                    bool        isCeil
                ) const;

    //
    void Init(u16 allocStrLen, CommandBufferMode cmdBuffMode);

    //
    void InitMaterial();

    //
    u16 SetStringImpl(
        const wchar_t*  str,
        u16             dstIdx,
        u32             strLen);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool UpdatePTDirty(bool isChanged)
    {
        m_IsPTDirty |= isChanged ? 1: 0;
        return isChanged;
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    bool IsPTDirty() const
    {
        return m_IsPTDirty != 0;
    }

#ifdef NW_LYT_DRAWER_ENABLE
    //
    //
    void UpdateDrawCharDataImpl(Drawer* pDrawer)
    {
        if (this->IsPTDirty() || (! m_pDispStringBuf->IsGeneratedCommand() && pDrawer))
        {
            SetupDrawCharData(pDrawer);
        }
    }
#endif

#ifdef NW_LYT_DRAWER_ENABLE
    //
    //
    void SetupDrawCharData(Drawer* pDrawer);
#endif

    //
    void SetupTextWriter(font::WideTextWriter* pWriter);

    //
    wchar_t* m_TextBuf;

    //
    ut::Color8 m_TextColors[TEXTCOLOR_MAX];

    //
    const font::Font* m_pFont;

    //
    Size m_FontSize;

    //
    f32 m_LineSpace;

    //
    f32 m_CharSpace;

    //
    TagProcessor* m_pTagProcessor;

    //
    u16 m_TextBufBytes;

    //
    u16 m_TextLen;

    //
    u8 m_TextPosition;

    //
    u8 m_TextAlignment;

    //
    // If position and texture change, then true.
    u8 m_IsPTDirty;

    //
    u8 padding[1];

    //
    Material* m_pMaterial;

    //
    font::DispStringBuffer* m_pDispStringBuf;

#if defined(NW_FONT_ENABLE_EDGE_FONT)
    font::FontCombinerType  m_CombinerType;
#endif
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_TEXTBOX_H_

