/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW_LYT_WINDOW_H_
#define NW_LYT_WINDOW_H_

#include <nw/ut/ut_Color.h>
#include <nw/ut/ut_RuntimeTypeInfo.h>

#include <nw/lyt/lyt_Config.h>
#include <nw/lyt/lyt_Pane.h>

namespace nw
{
namespace lyt
{

class DrawInfo;
class AnimationLink;

//---------------------------------------------------------------------------
//
//
//
//
//
//---------------------------------------------------------------------------
class Window : public Pane
{
    typedef Pane Base;

    //
    struct Content
    {
        ut::Color8 vtxColors[VERTEXCOLOR_MAX];
        internal::TexCoordAry texCoordAry;
    };

    //
    struct Frame
    {
        Frame()
        :   textureFlip(0),
            pMaterial(0)
        {}

        ~Frame();

        TextureFlip GetTextureFlip() const
        {
            return (TextureFlip) this->textureFlip;
        }

        u8 textureFlip;
        u8 padding_[3];
        Material* pMaterial;
    };

public:
    //
    NW_UT_RUNTIME_TYPEINFO;

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Window(
        u8  contentTexNum,
        u8  frameTexNum);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Window(
        u8  contentTexNum,
        u8  frameLTTexNum,
        u8  frameRTTexNum,
        u8  frameRBTexNum,
        u8  frameLBTexNum);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Window(
        u8  contentTexNum,
        u8  cornerLTTexNum,
        u8  cornerRTTexNum,
        u8  cornerRBTexNum,
        u8  cornerLBTexNum,
        u8  frameLTexNum,
        u8  frameTTexNum,
        u8  frameRTexNum,
        u8  frameBTexNum);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Window(
        const res::Window*  pBlock,
        const ResBlockSet&  resBlockSet);

    //
    //
    //
    //
    virtual ~Window();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual const ut::Color8 GetVtxColor(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColor(u32 idx, ut::Color8 value);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u8 GetVtxColorElement(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual void SetVtxColorElement(u32 idx, u8 value);

    using Base::GetMaterial;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual u8 GetMaterialNum() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual Material* GetMaterial(u32 idx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Material* GetContentMaterial() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetContentMaterial(Material* pMaterial);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    Material* GetFrameMaterial(WindowFrame frameIdx) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetFrameMaterial(WindowFrame frameIdx, Material* pMaterial);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void ReserveTexCoord(u8 num);

    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetTexCoordNum() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetTexCoordNum(u8 num);

    //
    //
    //
    //
    //
    //
    //
    void GetTexCoord(u32 idx, TexCoordQuad coords) const;

    //
    //
    //
    //
    //
    //
    //
    void SetTexCoord(u32 idx, const TexCoordQuad coords);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    u8 GetFrameNum() const
    {
        return m_FrameNum;
    }

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    virtual Material* FindMaterialByName(
        const char* findName,
        bool bRecursive = true);

    //

#ifdef NW_LYT_DMPGL_ENABLED
protected:
    virtual void DrawSelf(const DrawInfo& drawInfo);

protected:
    //
    virtual void DrawContent(
        const DrawInfo& draInfo,
        const math::VEC2& basePt,
        const WindowFrameSize& frameSize,
        u8 alpha);

    //
    virtual void DrawFrame(
        const DrawInfo& draInfo,
        const math::VEC2& basePt,
        const Frame& frame,
        const WindowFrameSize& frameSize,
        u8 alpha);

    //
    virtual void DrawFrame4(
        const DrawInfo& draInfo,
        const math::VEC2& basePt,
        const Frame* frames,
        const WindowFrameSize& frameSize,
        u8 alpha);

    //
    virtual void DrawFrame8(
        const DrawInfo& draInfo,
        const math::VEC2& basePt,
        const Frame* frames,
        const WindowFrameSize& frameSize,
        u8 alpha);

#endif // NW_LYT_DMPGL_ENABLED

protected:
    //
    const WindowFrameSize GetFrameSize(
        u8 frameNum,
        const Frame* frames) const;

    //
    void InitTexNum(
        u8 contentTexNum,
        u8 frameTexNums[],
        u8 frameNum);

    //
    void InitContent(u8 texNum);

    //
    void InitFrame(u8 frameNum);

#ifdef NW_LYT_DRAWER_ENABLE
public:

    virtual u32* MakeUniformDataSelf( u32* pCmdStream, DrawInfo* pDrawInfo, Drawer* pDrawer ) const;

protected: // Drawer is used to reduce processing.

    //
    //
    mutable u32 m_UniformTexCoordNum;

    //
    //
    mutable nw::math::VEC4 m_UniformTexCoords[ TexMapMax * 2 ];

#endif

protected:
    //
    InflationLRTB m_ContentInflation;

    //
    Content m_Content;

    //
    Frame* m_Frames;

    //
    Material* m_pMaterial;

    //
    u8 m_FrameNum;

#ifdef NW_LYT_DRAWER_ENABLE
protected: // Drawer is used to reduce processing.
    //
    //
    mutable u8 m_IsTexCoordInited;

    //
    u8 padding[2];
#else
    //
    u8 padding[3];
#endif

private:
    // Prohibits calling the copy constructor and assignment operator. This must not be implemented.
    Window(const Window& other);
    Window& operator = (const Window& other);
};

} // namespace nw::lyt
} // namespace nw

#endif // NW_LYT_WINDOW_H_

