﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/lyt/lyt_Supplement.h>


nw::math::MTX44* nw::lyt::internal::MTX44Transpose(nw::math::MTX44* pOut, const nw::math::MTX34* pSrc)
{
    NW_NULL_ASSERT(pOut);
    NW_NULL_ASSERT(pSrc);

    *pOut = math::MTX44(
        pSrc->f._00, pSrc->f._10, pSrc->f._20, 0.f,
        pSrc->f._01, pSrc->f._11, pSrc->f._21, 0.f,
        pSrc->f._02, pSrc->f._12, pSrc->f._22, 0.f,
        pSrc->f._03, pSrc->f._13, pSrc->f._23, 1.f);

    return pOut;
}


nw::math::MTX44* nw::lyt::internal::MTX44Mult(
    nw::math::MTX44* pOut,
    const nw::math::MTX34* p1,
    const nw::math::MTX44* p2)
{
    NW_NULL_ASSERT(pOut);
    NW_NULL_ASSERT(p1);
    NW_NULL_ASSERT(p2);

    nw::math::MTX44 tmp;

    for (int i = 0; i < 3; ++i)
    {
        for (int j = 0; j < 4; ++j)
        {
            f32 value = 0;
            for (int k = 0; k < 4; ++k)
            {
                value += p1->m[i][k] * p2->m[k][j];
            }
            tmp.m[i][j] = value;
        }
    }
    for (int j = 0; j < 4; ++j)
    {
        tmp.m[3][j] = p2->m[3][j];
    }

    *pOut = tmp;

    return pOut;
}
