﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include "precompiled.h"

#include <nw/lyt/lyt_Types.h>
#include <nw/lyt/lyt_TexMap.h>
#include <nw/lyt/lyt_Common.h>
#include <nw/lyt/lyt_DrawerCommand.h>

namespace nw
{
namespace lyt
{

TexMap::TexMap()
: m_TexObject(0)
, m_PhysicalAddress(NULL)
, m_Width(0)
, m_Height(0)
, m_RealWidth(0)
, m_RealHeight(0)
{
    m_Bits.format = TEXFORMAT_MAX;
    m_Bits.wrapS = TEXWRAP_CLAMP;
    m_Bits.wrapT = TEXWRAP_CLAMP;
    m_Bits.minFilter = TEXFILTER_LINEAR;
    m_Bits.magFilter = TEXFILTER_LINEAR;

#ifdef NW_LYT_DRAWER_ENABLE
    ResetU32Info();
#endif
}

TexMap::TexMap(const TextureInfo& textureInfo)
{
    m_Bits.wrapS = TEXWRAP_CLAMP;
    m_Bits.wrapT = TEXWRAP_CLAMP;
    m_Bits.minFilter = TEXFILTER_LINEAR;
    m_Bits.magFilter = TEXFILTER_LINEAR;

    this->Set(textureInfo);
}


#ifdef NW_LYT_DRAWER_ENABLE
void TexMap::ResetU32Info()
{
    switch ( GetFormat() )
    {
      case TEXFORMAT_RGBA8  : m_u32Format = PICA_DATA_TEXTURE_FORMAT_RGBA_UNSIGNED_BYTE;  break;
      case TEXFORMAT_RGB8   : m_u32Format = PICA_DATA_TEXTURE_FORMAT_RGB_UNSIGNED_BYTE;   break;
      case TEXFORMAT_RGB5A1 : m_u32Format = PICA_DATA_TEXTURE_FORMAT_RGBA_UNSIGNED_SHORT_5_5_5_1; break;
      case TEXFORMAT_RGB565 : m_u32Format = PICA_DATA_TEXTURE_FORMAT_RGB_UNSIGNED_SHORT_5_6_5; break;
      case TEXFORMAT_RGBA4  : m_u32Format = PICA_DATA_TEXTURE_FORMAT_RGBA_UNSIGNED_SHORT_4_4_4_4;  break;
      case TEXFORMAT_LA8    : m_u32Format = PICA_DATA_TEXTURE_FORMAT_LUMINANCE_ALPHA_UNSIGNED_BYTE;    break;
      case TEXFORMAT_HILO8  : m_u32Format = PICA_DATA_TEXTURE_FORMAT_HILO8_DMP_UNSIGNED_BYTE_DMP;  break;
      case TEXFORMAT_L8     : m_u32Format = PICA_DATA_TEXTURE_FORMAT_LUMINANCE_UNSIGNED_BYTE;     break;
      case TEXFORMAT_A8     : m_u32Format = PICA_DATA_TEXTURE_FORMAT_ALPHA_UNSIGNED_BYTE;     break;
      case TEXFORMAT_LA4    : m_u32Format = PICA_DATA_TEXTURE_FORMAT_LUMINANCE_ALPHA_UNSIGNED_BYTE_4_4_DMP;    break;
      case TEXFORMAT_L4     : m_u32Format = PICA_DATA_TEXTURE_FORMAT_LUMINANCE_UNSIGNED_4BITS_DMP;     break;
      case TEXFORMAT_A4     : m_u32Format = PICA_DATA_TEXTURE_FORMAT_ALPHA_UNSIGNED_4BITS_DMP;     break;
      case TEXFORMAT_ETC1   : m_u32Format = PICA_DATA_TEXTURE_FORMAT_ETC1_RGB8_NATIVE_DMP;   break;
      case TEXFORMAT_ETC1A4 : m_u32Format = PICA_DATA_TEXTURE_FORMAT_ETC1_ALPHA_RGB8_A4_NATIVE_DMP; break;
    }

    m_u32WidthHeight = PICA_CMD_DATA_TEXTURE_SIZE( GetRealWidth(), GetRealHeight() );

    const u32 minFilter[] =
    {
        PICA_DATA_TEXTURE_MIN_FILTER_NEAREST,
        PICA_DATA_TEXTURE_MIN_FILTER_LINEAR,
    };

    const u32 wrap[] =
    {
        PICA_DATA_TEXTURE_WRAP_CLAMP_TO_EDGE,
        PICA_DATA_TEXTURE_WRAP_REPEAT,
        PICA_DATA_TEXTURE_WRAP_MIRRORED_REPEAT,
    };

    m_u32WrapFilter = PICA_CMD_DATA_TEXTURE_WRAP_FILTER(
        GetMagFilter() /* textureMagFilter */,
        minFilter[ GetMinFilter() ] /* textureMinFilter */,
        m_u32Format /* textureFormat*/,
        wrap[ GetWrapModeT() ] /* textureWrapT */,
        wrap[ GetWrapModeS() ] /* textureWrapS */,
        0 /* useShadowTexture */,
        0 /* textureSamplerType */);
}
#endif

} // namespace nw::lyt
} // namespace nw
