/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_MODULE3D_PUB_H_
#define NW4C_VIEWER_G3D_MODULE3D_PUB_H_

#include <nw/gfx.h>
#include <nw/ut.h>
#include <nw/anim.h>
#include <nw/viewer/types.h>
#include <nw/viewer/sys/Module.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace nw     {
namespace viewer {

namespace sys    {
class App;
} /* Namespace */

namespace g3d    {

class NodeEntity;
class NodeIter;
class ModelIter;
class CameraIter;
class LightIter;
class FogIter;
class SceneEnvironmentSettingIter;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
enum AnimType
{
    Anim_Skeletal = 0,    //
    Anim_Material,        //
    Anim_Visibility,      //
    Anim_Camera,          //
    Anim_Light,           //
    Anim_Fog              //
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class IResourceListener
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnLoadResource( nw::gfx::res::ResGraphicsFile /* file */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddModel( nw::gfx::Model* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddCamera( nw::gfx::Camera* /* Camera */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddLight( nw::gfx::Light* /* light */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddFog( nw::gfx::Fog* /* Fog */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddSceneEnvironmentSetting( nw::gfx::SceneEnvironmentSetting* /* setting */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnAddAnimation( nw::anim::res::ResAnim /* anim */, AnimType /* type */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateModel( nw::gfx::Model* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateMaterial( nw::gfx::Model* /* Model */, nw::gfx::Material* /* Material */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateCamera( nw::gfx::Camera* /* Camera */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateLight( nw::gfx::Light* /* light */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnUpdateFog( nw::gfx::Fog* /* Fog */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveModel( nw::gfx::Model* /* Model */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveCamera( nw::gfx::Camera* /* Camera */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveLight( nw::gfx::Light* /* light */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveFog( nw::gfx::Fog* /* Fog */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveSceneEnvironmentSetting( nw::gfx::SceneEnvironmentSetting* /* setting */ ) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    virtual void OnRemoveAnimation( nw::anim::res::ResAnim /* anim */, AnimType /* type */ ) {}

    //
    //
    //
    virtual /* dtor */ ~IResourceListener() {}
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
enum RenderFlag
{
    RenderGrid                     = 1<<1,  //
    RenderLightDir                 = 1<<2,  //
    RenderParticleEmitterForm      = 1<<3,  //
    RenderTargetCameraAxis         = 1<<4,  //
    RenderModelAxis                = 1<<5,  //
    RenderBoneAxis                 = 1<<6,  //
    RenderParticleEmitterAxis      = 1<<7,  //
    RenderConstraintTargetBoneAxis = 1<<8,  //
    RenderCameraIcon               = 1<<9,  //
    RenderLightIcon                = 1<<10, //
    RenderTargetLine               = 1<<11, //
    RenderViewFrustum              = 1<<12, //
    RenderBoundingBox              = 1<<13, //
    RenderDepthLevelSurface        = 1<<14, //
    RenderAll                      = 0xFFFF //
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
enum ScreenKind
{
    LEFT_SCREEN                    = 0x1 << 1,   //
    RIGHT_SCREEN                   = 0x1 << 2    //
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class Module3D : public sys::Module
{
//---- Public members -----
public:

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void DrawScene(nw::gfx::IRenderTarget* renderTarget, ScreenKind screen);

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void Draw3DInfo(nw::gfx::Camera* viewCamera, nw::gfx::Camera* baseCamera, u32 flag);

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    ModelIter GetModel( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetModelNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    LightIter GetLight( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetLightNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    CameraIter GetCamera( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetCameraNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    FogIter GetFog( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetFogNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter GetSceneEnvironmentSetting( u32 idx = 0 ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetSceneEnvironmentSettingNum() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter GetCurrentSceneEnvironmentSetting();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Camera* GetDefaultCamera();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Camera* GetViewCamera();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Camera* GetLeftViewCamera();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Camera* GetRightViewCamera();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::SceneEnvironment* GetSceneEnvironment();

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void GetSceneEnvironmentArraySize( int& cameraSize, int& lightSetSize, int& fogSize );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    int GetCameraIndex( nw::gfx::Camera* camera );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void AddCamera( nw::gfx::Camera* camera );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void RemoveCamera( nw::gfx::Camera* camera );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void AddLight( nw::gfx::Light* light );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void RemoveLight( nw::gfx::Light* light );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetRootTransform( const nw::math::Transform3& transform );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetRootTransform( const math::VEC3& scale, const math::MTX34& rotateTranslate );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    IResourceListener* GetResourceListener();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetResourceListener( IResourceListener* listener );

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    u32  GetRenderFlag();

//---- Private members -----
private:
    //
    //
    //
    Module3D();

    NW4C_VIEWER_FORBIDDEN_FUNC( Module3D );

    //
    //
    //
    IResourceListener* mResListener;

    friend class sys::App;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::SceneObject* GetSceneObject() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResSceneObject GetResSceneObject() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    bool IsValid() const { return NULL != m_pEntity; }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::ut::MoveArray<nw::gfx::BaseAnimEvaluator*>::iterator GetCurrentAnimEvaluatorsBegin();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::ut::MoveArray<nw::gfx::BaseAnimEvaluator*>::iterator GetCurrentAnimEvaluatorsEnd();

protected:
    //
    //
    //
    NodeIter(NodeEntity *pNode);

    //
    //
    //
    virtual ~NodeIter(){};

    //
    //
    //
    NodeEntity *m_pEntity;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class ModelIter : public NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    ModelIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Model* GetModel() const
    {
        return nw::ut::DynamicCast<nw::gfx::Model*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResModel GetResModel() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResModel>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ModelIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ModelIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ModelIter operator++(int)
    {
        ModelIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const ModelIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const ModelIter& it ) const
    {
        return !(*this == it);
    }
protected:
    //
    //
    //
    ModelIter(NodeEntity *pModel) : NodeIter(pModel) {}
    friend class Module3D;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class LightIter : public NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    LightIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Light* GetLight() const
    {
        return nw::ut::DynamicCast<nw::gfx::Light*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResLight GetResLight() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResLight>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    LightIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    LightIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    LightIter operator++(int)
    {
        LightIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const LightIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const LightIter& it ) const
    {
        return !(*this == it);
    }
protected:
    //
    //
    //
    LightIter(NodeEntity *pLight) : NodeIter(pLight) {}
    friend class Module3D;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class CameraIter : public NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    CameraIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Camera* GetCamera() const
    {
        return nw::ut::DynamicCast<nw::gfx::Camera*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResCamera GetResCamera() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResCamera>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    CameraIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    CameraIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    CameraIter operator++(int)
    {
        CameraIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const CameraIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const CameraIter& it ) const
    {
        return !(*this == it);
    }
protected:
    //
    //
    //
    CameraIter(NodeEntity *pCamera) : NodeIter(pCamera) {}
    friend class Module3D;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class FogIter : public NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    FogIter(): NodeIter(NULL){}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::Fog* GetFog() const
    {
        return nw::ut::DynamicCast<nw::gfx::Fog*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResFog GetResFog() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResFog>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    FogIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    FogIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    FogIter operator++(int)
    {
        FogIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const FogIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const FogIter& it ) const
    {
        return !(*this == it);
    }
protected:
    //
    //
    //
    FogIter(NodeEntity *pFog) : NodeIter(pFog) {}
    friend class Module3D;
};

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class SceneEnvironmentSettingIter : public NodeIter
{
public:
    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter() : NodeIter(NULL) {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::SceneEnvironmentSetting* GetSceneEnvironmentSetting() const
    {
        return nw::ut::DynamicCast<nw::gfx::SceneEnvironmentSetting*>(GetSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::gfx::res::ResSceneEnvironmentSetting GetResSceneEnvironmentSetting() const
    {
        return nw::ut::ResDynamicCast<nw::gfx::res::ResSceneEnvironmentSetting>(GetResSceneObject());
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter GetNext() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter &operator++()
    {
        *this = this->GetNext(); return *this;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    SceneEnvironmentSettingIter operator++(int)
    {
        SceneEnvironmentSettingIter it = *this; *this = this->GetNext(); return it;
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator==( const SceneEnvironmentSettingIter& it ) const
    {
        return (this->m_pEntity == it.m_pEntity);
    }

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    bool operator!=( const SceneEnvironmentSettingIter& it ) const
    {
        return !(*this == it);
    }

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void Activate() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    static void Deactivate();
protected:
    //
    //
    //
    SceneEnvironmentSettingIter(NodeEntity *pScene) : NodeIter(pScene) {}
    friend class Module3D;
};

} /* namespace g3d    */
} /* namespace viewer */
} /* namespace nw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif


#endif /* NW4C_VIEWER_G3D_MODULE3D_PUB_H_ */

