/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_LYT_LAYOUTACCESS_PUB_H_
#define NW4C_VIEWER_LYT_LAYOUTACCESS_PUB_H_

#include <nw/lyt/lyt_Types.h>
//#include <nw/lyt/lyt_DrawInfo.h>
//#include <nw/lyt/lyt_Pane.h>
//#include <nw/lyt/lyt_Layout.h>
#include <nw/ut/ut_Rect.h>
#include <nw/math/math_Types.h>

namespace nw {
namespace lyt {

class Layout;
class Pane;
class AnimResource;

} // namespace nw::lyt
} // namespace nw


//
namespace vw  {
//
namespace g2d {

class ResLayout;
class AnimCtrl;

} // namespace vw::g2d
} // namespace vw


namespace nw {
namespace viewer {
namespace lyt {

class PaneInfo;
class AnimInfo;

//====================================================
//
//
//
//
//====================================================
class LayoutInfo
{
//--- public static members --------
public:
    //
    //
    //
    //
    //
    //
    //
    //
    static u32          GetLayoutNum();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    static LayoutInfo   GetLayout( u32 id );

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    static LayoutInfo   GetFirstLayout();

//--- public members --------
    //
    //
    //
    //
    //
    const char*     GetName()           const;

    //
    //
    //
    //
    //
    //
    //
    u32             GetID()             const;

    //
    //
    //
    //
    //
    bool            IsVisible()         const { return true; }

    //
    //
    //
    //
    //
    const nw::lyt::Size& GetLayoutSize()     const;

    //
    //
    //
    //
    //
    const nw::ut::Rect   GetLayoutRect()     const;

    //
    //
    //
    //
    //
    bool            IsValid()           const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    LayoutInfo      GetNext()           const;

    //
    //
    //
    //
    //
    //
    PaneInfo        GetRootPaneInfo()               const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    PaneInfo        GetPaneInfo( u32 idx )          const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    PaneInfo        GetPaneInfo( const char* name ) const;

    //
    //
    //
    //
    //
    //
    //
    //
    s32             GetPaneNum()                    const;

    //
    //
    //
    //
    //
    u32             GetAnimNum()                const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    AnimInfo        GetAnim( u32 idx )          const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    AnimInfo        GetAnim( const char* name ) const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    AnimInfo        GetFirstAnim()              const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    AnimInfo        GetCurrentAnim()            const;

    //
    //
    //
    //
    void            SetCurrentAnim( const char* name );

    //
    //
    //
    //
    void            SetCurrentAnim( u32 idx );

    //
    //
    //
    //
    //
    bool            HasAnimation()      const;

    //
    //
    //
    //
    //
    f32             GetCurrentFrame()   const;

    //
    //
    //
    //
    void            SetCurrentFrame(f32 frame);

    //
    //
    //
    //
    //
    f32             GetTotalFrame()     const;

    //
    //
    //
    //
    //
    f32             GetAbsoluteFrame()  const;

    //
    //
    //
    //
    //
    nw::lyt::Layout* GetLayout() const
    {
        return const_cast<nw::lyt::Layout*>(this->ptr_lyt());
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const LayoutInfo  operator=( const LayoutInfo info ) { this->mpLayout = info.mpLayout; return *this; }

//--- private members -------
private:
    /* ctor */      LayoutInfo( vw::g2d::ResLayout* layout = NULL ) : mpLayout( layout ) {}
    const vw::g2d::ResLayout& ref() const { NW_NULL_ASSERT( mpLayout ); return *mpLayout; }
    const vw::g2d::ResLayout* ptr() const { return mpLayout; }
    vw::g2d::ResLayout&       ref()       { NW_NULL_ASSERT( mpLayout ); return *mpLayout; }
    vw::g2d::ResLayout*       ptr()       { return mpLayout; }
    const nw::lyt::Layout& ref_lyt() const;
    const nw::lyt::Layout* ptr_lyt() const;

    vw::g2d::ResLayout* mpLayout;
};


//====================================================
//
//
//
//
//
//====================================================
class PaneInfo
{
//--- public members --------
public:
    //
    //
    //
    //
    //
    const char*               GetName()      const;

    //
    //
    //
    //
    //
    const nw::math::MTX34&  GetMtx()       const;

    //
    //
    //
    //
    //
    const nw::math::MTX34&  GetGlobalMtx() const;

    //
    //
    //
    //
    //
    const nw::ut::Rect      GetPaneRect()  const;

    //
    //
    //
    //
    //
    const nw::lyt::Size&    GetSize()      const;

    //
    //
    //
    //
    //
    //
    void                      GetSRT( nw::math::VEC2* pScale, nw::math::VEC3* pRot, nw::math::VEC3* pTrans ) const;

    //
    //
    //
    //
    bool                      IsVisible()    const;


    //
    //
    //
    //
    //
    bool                IsValid() const { return (mpPane != NULL); }

    //
    //
    //
    //
    //
    //
    PaneInfo            GetParent()          const;

    //
    //
    //
    //
    //
    //
    //
    //
    PaneInfo            GetChild( u32 idx )  const;

    //
    //
    //
    //
    //
    u32                 GetChildNum()        const;

    //
    //
    //
    //
    //
    //
    PaneInfo            GetNextSibling()     const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    PaneInfo            GetNext()            const;

    //
    //
    //
    //
    //
    nw::lyt::Pane* GetPane() const
    {
        return const_cast<nw::lyt::Pane*>(this->ptr());
    }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    const PaneInfo      operator=( const PaneInfo info ) { this->mpPane = info.mpPane; return *this; }

    //
    /* ctor */          PaneInfo( nw::lyt::Pane* pPane = NULL ) : mpPane( pPane ) {}

//--- private members -------
private:
    const nw::lyt::Pane& ref() const { NW_ASSERT( this->IsValid() ); return *mpPane; }
    const nw::lyt::Pane* ptr() const { return mpPane; }
    nw::lyt::Pane&       ref()       { NW_ASSERT( this->IsValid() ); return *mpPane; }
    nw::lyt::Pane*       ptr()       { return mpPane; }

    nw::lyt::Pane*    mpPane;
};


//====================================================
//
//
//
//
//
//====================================================
class AnimInfo
{
//--- public members --------
public:
    //
    //
    //
    //
    //
    const char*         GetName()   const;

    //
    //
    //
    //
    bool                IsValid()   const;

    //
    //
    //
    //
    //
    bool                IsCurrent() const;

    //
    //
    //
    //
    //
    u32                 GetIndex()  const { NW_ASSERT( this->IsValid() ); return u32(mIdx); }

    //
    //
    //
    //
    //
    f32                 GetFrameMax() const;

    //
    //
    //
    //
    //
    bool                IsLoop()      const;

    //
    //
    //
    //
    //
    //
    AnimInfo            GetNext()     const;

    //
    /* ctor */          AnimInfo( const vw::g2d::ResLayout* lyt = NULL, u32 idx = 0 ) : mpLayout( lyt ), mIdx( s32(idx) ) {}

    //
    //
    //
    //
    //
    const nw::lyt::AnimResource& GetAnimResource() const;

//--- private members -------
private:
    const vw::g2d::ResLayout& ref_lyt() const { NW_NULL_ASSERT( mpLayout ); return *mpLayout; }
    const vw::g2d::ResLayout* ptr_lyt() const { return mpLayout; }
    const vw::g2d::AnimCtrl& ref() const;

    const vw::g2d::ResLayout*    mpLayout;
    s32                 mIdx;
};

} /* namespace nw::viewer::lyt */
} /* namespace nw::viewer */
} /* namespace nw  */

/* NW4C_VIEWER_LYT_LAYOUTINFO_PUB_H_ */
#endif


