/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_MCSSTREAM_PUB_H_
#define NW4C_VIEWER_SYSTEM_MCSSTREAM_PUB_H_

#include <nn/hio.h>
#include <nw/viewer/types.h>
#include <nw/viewer/sys/Heap.h>
//#include <nw/viewer/sys/VideoSetting.h>
//#include <nw/viewer/sys/PADInfo.h>
#include <nw/viewer/g3d/Module3D.h>

namespace nw     {
namespace viewer {
namespace sys    {

class McsManager;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class McsChannel
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 Read( void* data, u32 size );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 Write( const void* data, u32 size );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 Peek( void* data, u32 size );

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 Skip( u32 size );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetReadableBytes() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    u32 GetWritableBytes( u32* pWritableBytes ) const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    bool Polling();

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void Destroy();

private:
    //----------------------------------------
    //
    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    McsChannel( u16 channel );

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    ~McsChannel();

    //

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    u16     mChannel;

    friend class McsManager;
};

//
const u32    STREAM_DEFAULT_BUF_SIZE = 0x20000;

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class McsManager
{
public:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    bool IsServerConnected() const;

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    McsChannel* CreateChannel( u16 channel, u32 bufSize = STREAM_DEFAULT_BUF_SIZE  );

private:
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    McsManager() {}

    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    NW4C_VIEWER_FORBIDDEN_FUNC( McsManager );

    friend class App;
};


} // namespace sys
} // namespace viewer
} // namespace nw


#endif /* NW4C_VIEWER_SYSTEM_MCSSTREAM_PUB_H_ */

