/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_COMMON_CANVAS_H_
#define NW4R_VIEWER_COMMON_CANVAS_H_

//#include <dolphin/types.h>
#include <nn/types.h>


namespace vw     {
namespace common {

class Canvas
{
private:
    static const u32 DEFAULT_COLOR = 0xffffffff;
public:

    static void SetDither( bool dither ) { sDither = dither; }

    Canvas() : mTevSettingFunc( NULL ) {}
    ~Canvas(){}

    void SetupGX( void );
    static void SetupCamera( f32 width = CANVAS_WIDTH, f32 height = CANVAS_HEIGHT, Mtx44 transformMtx = NULL );

    void DrawLine( s16 x1, s16 y1, s16 x2, s16 y2, GXColor color1, GXColor color2, u8 lineWidth = 0 );
    void DrawLine( s16 x1, s16 y1, s16 x2, s16 y2, GXColor color, u8 lineWidth = 0 )
    {
        DrawLine( x1, y1, x2, y2, color, color, lineWidth );
    }

    void FrameRect( s16 posh, s16 posv, s16 sizeh, s16 sizev,
                    GXColor colorUL, GXColor colorUR, GXColor colorDL, GXColor colorDR, u8 lineWidth = 0 );
    void FrameRect( s16 posh, s16 posv, s16 sizeh, s16 sizev, GXColor color, u8 lineWidth = 0 )
    {
        FrameRect( posh, posv, sizeh, sizev, color, color, color, color, lineWidth );
    }

    void FillRect(  s16 posh, s16 posv, s16 sizeh, s16 sizev,
                    GXColor colorUL, GXColor colorUR, GXColor colorDL, GXColor colorDR );
    void FillRect(  s16 posh, s16 posv, s16 sizeh, s16 sizev, GXColor color )
    {
        FillRect( posh, posv, sizeh, sizev, color, color, color, color );
    }

    void FillRect( s16 posh, s16 posv, s16 sizeh, s16 sizev, GXColor colorU, GXColor colorD )
    {
        FillRect( posh, posv, sizeh, sizev, colorU, colorU, colorD, colorD );
    }

    void FillRectWithTex( s16 posh, s16 posv, s16 sizeh, s16 sizev, GXTexObj* obj, s16 texh, s16 texv, GXColor color = *(GXColor*)&DEFAULT_COLOR );

    void SetTevSetting( void (*func)( void )  ) { mTevSettingFunc = func; }

private:
    static bool  sDither;       // Existence of dither processing

    //---- Prepare to draw
    void         SetupVertexFormat();
    void         SetupTevDefault();
    void         SetupTev();
    void         SetupPE();
    void         SetupMisc();

    void         SetupVertexFormatForTex();
    void         SetupTevForTex();

    void (*mTevSettingFunc)( void );

};


}   /* namespace common */
}   /* namespace vw */


/* NW4R_VIEWER_COMMON_CANVAS_H_ */
#endif
