/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_COMMON_CHARARRAY_H_
#define NW4R_VIEWER_COMMON_CHARARRAY_H_

//#include <revolution/types.h>
//#include <revolution/mem.h>
#include <nn/types.h>
#include <nw/os/os_Memory.h>

namespace vw     {
namespace common {


/*---------------------------------------------------------------------------*
  Name:         CharArray
  Description:  Variable length string class
 *---------------------------------------------------------------------------*/
class CharArray
{
public:
    /* ctor */ explicit       CharArray( nw::os::IAllocator* allocator = NULL );                           // Constructor
    /* ctor */ /* implicit */ CharArray( const char* str, nw::os::IAllocator* allocator = NULL );          // Constructor
    /* ctor */                CharArray( const char* str, u32 len, nw::os::IAllocator* allocator = NULL ); // Constructor
    /* ctor */ /* implicit */ CharArray( const CharArray& str, nw::os::IAllocator* allocator = NULL );     // Constructor
    virtual /* dtor */        ~CharArray();                                                          // Destructor
    char*               Get() const                 { return mStr; }        // Get string
    void                Set( const char* str );                             // String settings
    void                Set( const char* str, u32 len );                    // String settings (includes size specification)
    const char*         Cat( const char* str );                             // Add string
    const char*         Cat( const char* str, u32 len );                    // Add string (with size specification)

    void                Clear() { this->Set( NULL ); }                      // Release string and set to NULL
    void                SetAllocator( nw::os::IAllocator* allocator ) { mAllocator = allocator; } // Allocator settings

    const CharArray&    operator = ( const char*      str ) { this->Set( str );      return *this; }
    const CharArray&    operator = ( const CharArray& str ) { this->Set( str.mStr ); return *this; }
    const CharArray&    operator += ( const char*      str ) { this->Cat( str );      return *this; }
    const CharArray&    operator += ( const CharArray& str ) { this->Cat( str.mStr ); return *this; }
                        operator const char* () const { return mStr; }
          char          operator[] ( int idx ) const { return ( idx < mLen ) ? mStr[ idx ] : '\0'; }

private:
    nw::os::IAllocator* mAllocator;
    char*               mStr;                                               // Pointer to text buffer
    u32                 mLen;                                               // Buffer size

    void                Free_( char* str );
    char*               Alloc_( u32 size );
};



}   /* namespace common */
}   /* namespace vw */


/* NW4R_VIEWER_COMMON_CANVAS_H_ */
#endif
