/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_G3D_CAMERA_MGR_H_
#define NW4R_VIEWER_G3D_CAMERA_MGR_H_

#include <nn/ulcd/CTR/ulcd_StereoCamera.h>
#include "Camera.h"
#include "../../../src/g3d/include/SceneElemSelector.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw  {
namespace g3d {

class CameraMgr
{
public:
    // <CODE>GetInstance</CODE>
    static CameraMgr*   GetInstance();
    // Initialize
    static void         Initialize();
    // Finalize
    static void         Finalize();
    // Setup
    void                Setup( SceneElemSelector::CameraList::iterator camerasBegin,
                               SceneElemSelector::CameraList::iterator camerasEnd);
    // MoveCamera3D
    void                MoveCamera3D(vw::system::HidStatus& hidStatus);
    // GetActiveViewCameraIdx
    int                 GetActiveViewCameraIdx(){ return m_ViewIdx; }
    // GetActiveTaretCameraIdx
    int                 GetActiveTargetCameraIdx(){ return m_TargetIdx; }
    // GetViewCamera
    nw::gfx::Camera*    GetViewCamera(){ return m_CameraArray[m_ViewIdx].GetCamera(); }
    // GetViewCameraName
    const char*         GetViewCameraName(){ return m_CameraArray[m_ViewIdx].GetName(); }
    // GetTargetCamera
    nw::gfx::Camera*    GetTargetCamera(){ return m_CameraArray[m_TargetIdx].GetCamera(); }
    // GetTargetCameraName
    const char*         GetTargetCameraName(){ return m_CameraArray[m_TargetIdx].GetName(); }
    // GetLeftViewCamera
    nw::gfx::Camera*    GetLeftViewCamera(){ return m_LeftViewCamera; }
    // GetRightViewCamera
    nw::gfx::Camera*    GetRightViewCamera(){ return m_RightViewCamera; }
    // GetCameraNum
    u32                 GetCameraNum(){ return m_CameraArray.size(); }
    // IsFirstViewCamera()
    bool                IsFirstViewCamera();
    // IsLastViewCamera()
    bool                IsLastViewCamera();
    // SwitchToFirstViewCamera()
    void                SwitchToFirstViewCamera();
    // SwitchToLastViewCamera()
    void                SwitchToLastViewCamera();
    // SwitchToPrevViewCamera()
    void                SwitchToPrevViewCamera();
    // SwitchToNextViewCamera()
    void                SwitchToNextViewCamera();
    // IsFirstTargetCamera()
    bool                IsFirstTargetCamera();
    // IsLastTargetCamera()
    bool                IsLastTargetCamera();
    // SwitchToFirstTargetCamera()
    void                SwitchToFirstTargetCamera();
    // SwitchToLastTargetCamera()
    void                SwitchToLastTargetCamera();
    // SwitchToPrevTargetCamera()
    void                SwitchToPrevTargetCamera();
    // SwitchToNextTargetCamera()
    void                SwitchToNextTargetCamera();
    // <CODE>UpdateCameraMatrix</CODE>
    void                UpdateCameraMatrix();
    // Focus
    void                Focus(nw::math::VEC3);
    // Focus
    void                Focus(nw::math::AABB);
    // SynchronizeViewCameraToTargetCamera
    void                SynchronizeViewCameraToTargetCamera();
    // SynchronizeTargetCameraToViewCamera
    void                SynchronizeTargetCameraToViewCamera();
    // ApplySetting
    void                ReloadSetting();

private:
    /* ctor */
    CameraMgr();
    /* dtor */
    ~CameraMgr();
    // Stereo camera
    nn::ulcd::CTR::StereoCamera     m_StereoCamera;
    // Left eye camera
    nw::gfx::Camera*                m_LeftViewCamera;
    // Right eye camera
    nw::gfx::Camera*                m_RightViewCamera;
    // View camera index
    u16                             m_ViewIdx;
    // Operation target camera index
    s16                             m_TargetIdx;
    // Camera
    nw::ut::MoveArray<g3d::Camera>  m_CameraArray;
    // Name of view camera used immediately before
    char                            m_PrevViewCameraName[512];
    // Name of operation target camera used immediately before
    char                            m_PrevTargetCameraName[512];
    // Static instance
    static CameraMgr*               m_Instance;
};

}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif

/* NW4R_VIEWER_G3D_CAMERA_MGR_H_ */
#endif
