/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_LIGHT_H_
#define NW4C_VIEWER_G3D_LIGHT_H_

#include "DirectionalObj.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw  {
namespace g3d {

/*-------------------------------------------------------
  Name:         Light

  Description:  Light management class
  -------------------------------------------------------*/
class Light
{
//--public members-----------------------------------------
public:
    Light(nw::gfx::Light* light, const char *name);
    nw::gfx::Light*     GetLight();
    void                Reset();                           // In the end, returns the position to the pre-set position.
    void                Preset();                          // Stores the current settings so that the status can return if there is a reset.
    void                MoveLight3D(vw::system::HidStatus& hidStatus);
    const char *        GetName(){ return m_Name; }
    nw::math::VEC3      GetPosition() const;
    nw::math::VEC3      GetTargetPos() const{ return m_TargetPos; }
    nw::math::VEC3      GetDirection() const;
    void                SetTargetPos(const nw::math::VEC3& targetPos) { m_TargetPos = targetPos; }
    void                SetupLight();

private:
    void                Rotate( DirectionalObj& directionalObj );
    void                Dolly( DirectionalObj& directionalObj );
    void                Track( DirectionalObj& directionalObj );
    void                Slide( DirectionalObj& directionalObj );
    void                Tilt( DirectionalObj& directionalObj );
    void                SetDirection(const nw::math::VEC3& direction);

private:
    Light();
    nw::gfx::Light*     m_Light;
    char                m_Name[512];
    nw::math::VEC3      m_TargetPos;
    bool                m_CanTranslate;
    bool                m_CanRotate;
};

}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif


/* NW4C_VIEWER_G3D_LIGHT_H_ */
#endif
