/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4C_VIEWER_SYSTEM_APP_H_
#define NW4C_VIEWER_SYSTEM_APP_H_

#include <viewer.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw     {
namespace system {

class Module;

class App
{
//--------public members ------------------------
public:
    // Get instance
    static App&             GetInstance();
    static Module*          GetModuleInstance( u32 id );

    static void             SetEmbeddedFlg( bool flag ) { sIsEmbedded = flag; }
    static void             SetUseMenuFlg( bool flag ) { sUseMenuFlag = flag; }
    static void             SetUseGraphicsFlg( bool flag ) { sUseGraphicsFlag = flag; }
    static void             SetUseGraphicsMemoryAllocator( bool flag ) { sUseGraphicsMemoryAllocator = flag; }
    static bool             IsEmbeddedViewer() { return sIsEmbedded; }
    static bool             GetUseMenuFlg() { return sUseMenuFlag; }
    static bool             GetUseGraphicsFlg() { return sUseGraphicsFlag; }
    static bool             GetUseGraphicsMemoryAllocator() { return sUseGraphicsMemoryAllocator; }

    // System initialization routine
    static void             InitializeSystem( nw::os::IAllocator* allocator );

    // Initialization routine
    static void             Initialize( nw::os::IAllocator* allocator );

    // Menu initialization routine
    static void             InitializeMenu();

    // Graphics initialization routine
    static void             InitializeGraphics();

    // Finalization routine
    static void             Finalize();

    // MCS stream processing
    void                    PollStream();

    // Main routine pre-processing
    void                    PreMain();
    // Main routine
    void                    Main();

    // Update the animation frame
    void                    UpdateFrame();

    // Execute Calc and Draw alternately for each module
    void                    CalcAndDraw( bool sync = true );

    // Frame update routine
    void                    Calc();

    // Render routine
    void                    Draw( bool sync = true );
    void                    Redraw( bool sync = true ) { this->Draw( sync ); } // Old API alias

    // Information display routine
    void                    DrawInfo();

    // Frame synchronization
    void                    WaitNextFrame();

    // Get current focus module
    u16                     GetFocusID( void )          { return mFocusID;  }
    void                    SetFocusID( u16 focus );
    bool                    HasFocus( u16 id )          { return ( mFocusID == id ); }
    Module&                 GetFocusModule()            { return *this->GetModuleInstance( mFocusID ); }
    const Module&           GetFocusModule() const      { return *this->GetModuleInstance( mFocusID ); }
    HidStatus               GetHidStatus() const        { return mHidStatus; }

    bool                    IsVisibleInfo(void) const   { return (mIsVisibleInfo)/* && GetCapture().IsVisibleInfo()*/; }
    bool                    ChangeVisibleInfo( bool visible )
        {
            bool before = mIsVisibleInfo;
            mIsVisibleInfo = visible;
            return before;
        }

    // Calls enter playback command for all animations for each module
    void                    OnAnmCmd( u32   command,        // Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                                      u32   target,         // Target (ALL/3D_MDL/2D_LYT)
                                      u32   targetID,       // Target ID (do not use when target is ANM_TGT_ALL)
                                      void* arg,            // Option parameters
                                      u32   trigModule );   // Module ID that is the trigger

    bool                    IsRenderScene() const { return mIsRenderScene; }
    bool                    IsUpdateScene() const { return mIsUpdateScene; }
    bool                    IsSoundOnlyMode() const { return mIsSoundOnlyMode; }


////--------private members ------------------------
private:
    /* ctor */              App();
    /* dtor */              ~App();
    void                    HidRead();

private:
    static App*             m_pInstance;            // Pointer to application class

    nw::os::IAllocator*     mAllocator;             // Allocator
    u16                     mFocusID;               // Module with current focus
    u16                     mConnectFlg;            // Flag for tool being connected

    bool                    mChangeFocusEnable;     // Flag for whether mode change is possible
    bool                    mIsVisibleInfo;         // Flag for whether information display is valid
    enum
    {
        ANIM_STATE_STOP,
        ANIM_STATE_PLAY
    };
    u32                     mAnimState;             // Animation status

    static bool             sIsEmbedded;
    static bool             sUseMenuFlag;
    static bool             sUseGraphicsFlag;
    static bool             sUseGraphicsMemoryAllocator;

    HidStatus               mHidStatus;

    s32                     mSkipCount;
    bool                    mIsRequestRenderScene;
    bool                    mIsRenderScene;
    u32                     mUpdateCount;
    bool                    mIsUpdateScene;
    bool                    mIsSoundOnlyMode;
    f32                     mSkipTime;
};

}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.namespace vw     {
#endif



/* NW4C_VIEWER_SYSTEM_APP_H_ */
#endif
