/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_APPID_H_
#define NW4C_VIEWER_SYSTEM_APPID_H_

#include <viewer.h>

namespace vw     {
namespace system {


/* All Module IDs */
enum ModuleId
{
    SYSTEM_ID   = 0,
    G3D_ID      = 1,
    EFFECT_ID   = 2,
    G2D_ID      = 3,
    SOUND_ID    = 4,
#if defined(USE_SAMPLE)
    SAMPLE_ID     = 5,
#endif
    TEMPLATE_ID     = 6,
    MODULE_NUM,
    USER_ID     = MODULE_NUM
};


static const u32    SYSTEM_PRIORITY = 0;
static const u32    SOUND_PRIORITY  = 0x100;
static const u32    G3D_PRIORITY    = 0x200;
static const u32    EFFECT_PRIORITY = 0x300;
static const u32    G2D_PRIORITY    = 0x400;
static const u32    TEMPLATE_PRIORITY    = 0x500;

#if defined(USE_SAMPLE)
static const u32    SAMPLE_PRIORITY   = 0xF00;
#endif

static const u32    G3D_SETUP_ORDER     = 0x100;
static const u32    EFFECT_SETUP_ORDER  = 0x200;
static const u32    G2D_SETUP_ORDER     = 0x300;
static const u32    SOUND_SETUP_ORDER   = 0x400;
static const u32    DEFAULT_SETUP_ORDER = 0x500;



}   /* namespace system */

namespace g3d {
    enum
    {
        G3D_ID    = vw::system::G3D_ID,
        EFFECT_ID = vw::system::EFFECT_ID,
        G2D_ID    = vw::system::G2D_ID,
        SOUND_ID  = vw::system::SOUND_ID
    };
} /* namespace g3d */

namespace g2d {
    enum
    {
        G3D_ID    = vw::system::G3D_ID,
        EFFECT_ID = vw::system::EFFECT_ID,
        G2D_ID    = vw::system::G2D_ID,
        SOUND_ID  = vw::system::SOUND_ID
    };
} /* namespace g2d */

namespace effect {
    enum
    {
        G3D_ID    = vw::system::G3D_ID,
        EFFECT_ID = vw::system::EFFECT_ID,
        G2D_ID    = vw::system::G2D_ID,
        SOUND_ID  = vw::system::SOUND_ID
    };
} /* namespace effect */

namespace sound {
    enum
    {
        G3D_ID    = vw::system::G3D_ID,
        EFFECT_ID = vw::system::EFFECT_ID,
        G2D_ID    = vw::system::G2D_ID,
        SOUND_ID  = vw::system::SOUND_ID
    };
} /* namespace sound */

#if defined(USE_SAMPLE)
namespace sample {
    enum
    {
        G3D_ID    = vw::system::G3D_ID,
        EFFECT_ID = vw::system::EFFECT_ID,
        G2D_ID    = vw::system::G2D_ID,
        SOUND_ID  = vw::system::SOUND_ID
        SAMPLE_ID = vw::system::SAMPLE_ID
    };
} /* namespace sample */
#endif

}   /* namespace vw     */



/* NW4R_VIEWER_SYSTEM_APPID_H_ */
#endif
