/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef VIEWER_SYSTEM_CAPTURE_H_
#define VIEWER_SYSTEM_CAPTURE_H_

#include <viewer.h>


namespace vw     {
namespace system {

//============================================
// Interface class for screen capture
//============================================
class CaptureInterface
{
//--- Public members ------
public:
    static CaptureInterface& GetInstance();

    static void        Capture(const char* filePath = NULL);             // Executes capture.
    //static void        Main();             // Capture main function
    //static bool        IsExecuting();      // Flag indicating whether the capture image is being generated
    //static bool        IsBusy();           // Determines whether capture is being executed
    //static bool        IsVisibleInfo();    // Gets whether it is OK for each module to display information

    static void  Finalize();
    static void  Main();
    static bool  IsExecuting();
    static f32   GetMagnify();
    static void  SetMagnify( f32 magnify );

//--- Private members ------
private:
    /* ctor */              CaptureInterface() {}
    /* dtor */              ~CaptureInterface() {}
    static CaptureInterface sCaptureInterface;
};


/*---------------------------------------------------------------------------*
  Name:         GetCapture

  Description:  Gets the screen capture management class instance.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
inline CaptureInterface&
GetCapture()
{
    return CaptureInterface::GetInstance();
}

}   /* namespace system */
}   /* namespace vw     */



/* VIEWER_SYSTEM_CAPTURE_H_ */
#endif
