/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW_DEMO_COMMANDLISTSWAPPER_H_
#define NW_DEMO_COMMANDLISTSWAPPER_H_

#include <viewer.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif


namespace vw
{
namespace system
{

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------
class CommandListSwapper
{
    NW_DISALLOW_COPY_AND_ASSIGN(CommandListSwapper);

public:

    //---------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    struct Description
    {
        size_t commandListCount;            //
        size_t bufferSize;                  //
        size_t requestCount;                //
        size_t reusableBufferSize;          //
        size_t reusableRequestCount;        //
        size_t maxGpuProfilingEntryCount;   //
    };

    //
    static CommandListSwapper* GetInstance();

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    static CommandListSwapper* Create(nw::os::IAllocator* allocator, Description& description);

    //
    void Destroy();

    //

    //----------------------------------------
    //
    //

    //
    void Bind();

    //
    void Run()
    {
        RunAsync();
        WaitDone();
    }

    //
    void RunAsync();

    //
    void WaitDone();

    //
    //
    //
    void Swap();

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    void ResetGpuProfiling();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    s32 AddGpuProfilingStartPoint(bool IsInTotal);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void SetGpuProfilingEndPoint(u32 profilingId);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetGpuProfilingCostTime(u32 profilingId);

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    f32 GetGpuProfilingTotalCostTime();

    //}

    //----------------------------------------
    //
    //

    //
    GLuint GetCommandListId() const { return m_CommandLists[m_BoundIndex]; }

    //
    size_t GetListCount() const { return this->m_CommandLists.size(); }

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    int GetCommandBufferSize() const;

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    int GetReusableCommandBufferSize() const;

    //

    //----------------------------------------
    //
    //

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void StartCommandSave();

    //
    //
    //
    //
    //
    //
    void EndCommandSave();

    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    //
    void ReuseCommand(bool isCopyBuffer);

    //

private:
    CommandListSwapper(nw::os::IAllocator* allocator, Description& description);

    ~CommandListSwapper() {}

    //
    void CalcGpuProfilingCostTime();

    //
    void SetGpuProfilingTick(GLint id);

    //
    static void CommandListCallback(GLint id);

    //
    //
    //
    GLuint RegisterCallback();

    struct GpuProfilingEntry {
        GLuint beginId;
        GLuint endId;

        s64 beginTick;
        s64 endTick;

        f32 costTime;

        bool isInTotal;
        GLuint splitId;

        GpuProfilingEntry()
        : beginId(0),
          endId(0),
          beginTick(0),
          endTick(0),
          costTime(0.0f),
          isInTotal(false),
          splitId(0)
        {}
    };

    nw::os::IAllocator* m_Allocator;
    typedef nw::ut::MoveArray<GLuint> CommandListArray;
    CommandListArray m_CommandLists;
    int m_BoundIndex;

    int m_RunningIndex;
    bool m_IsRunning;

    typedef nw::ut::MoveArray<GpuProfilingEntry> GpuProfilingEntryArray;
    GpuProfilingEntryArray m_GpuProfilingEntries;
    GpuProfilingEntryArray m_GpuProfilingResults;

    CommandListArray m_ReusableCommandLists;
    GLuint m_ReusableBufferOffset;
    GLsizei m_ReusableBufferSize;
    GLuint m_ReusableRequestId;
    GLsizei m_ReusableRequestSize;

    static CommandListSwapper*   s_CommandListSwapper;
};



} // namespace system
} // namespace vw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif // NW_DEMO_COMMANDLISTSWAPPER_H_
