/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4CVIEWER_DRAW2DHELPER_H_
#define NW4CVIEWER_DRAW2DHELPER_H_


#include <viewer.h>
#include <nn/tpl.h>
#include "Draw3DHelper.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw     {
namespace system {



/*
  
 */
class FontBuffer
{
    friend class FontDrawer;
    nw::font::DispStringBuffer* mDrawStringBuf;
public:
    FontBuffer() : mDrawStringBuf(0){}
    ~FontBuffer()
    {
        ReleaseBuffer();
    }
    void ReleaseBuffer()
    {
        if(0 != mDrawStringBuf)
        {
            Memory_Free(mDrawStringBuf);
            mDrawStringBuf = 0;
        }
    }
};

/*
  
 */
class FontDrawer
{
public:
    //!--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    static FontDrawer& GetInstance();

    //---------------------------------------------------------------------------
    //
    //
    static void Initialize();

    //!--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    static void Finalize();
    //!--------------------------------------------------------------------------
    //
    //---------------------------------------------------------------------------
    void BeginDraw();
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    f32  DrawString( s32 posh, s32 posv, const char* format, ... );
    //---------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void  DrawString( const char* format, ... );
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    f32  DrawString( s32 posh, s32 posv, const char* format, std::va_list vargs );
    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void  DrawString( const char* format, std::va_list vargs );
    //!--------------------------------------------------------------------------

    //
    //---------------------------------------------------------------------------
    void EndDraw();
    //!--------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::ut::Rect GetDrawRect( s32 posh, s32 posv, const char* format, ... );
    //!--------------------------------------------------------------------------

    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetScreenSize(u32 width, u32 height);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetColor(nw::ut::Color8 color);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    nw::ut::Color8 GetColor() const
    {
        return mColor;
    }
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetCursorX(f32 x)
    {
        mWriter.SetCursorX(x);
    }
    //!--------------------------------------------------------------------------
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetFontSize( f32 width, f32 height);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void SetFixedWidth( f32 width);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void CreateCache(FontBuffer& buf, u32 charMax);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void BeginCache(FontBuffer& buf);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawCache(s32 posh, s32 posv, const char* format, ... );
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawCache(const char* format, ... );
    //!--------------------------------------------------------------------------
    //
    //
    //
    //---------------------------------------------------------------------------
    void EndCache(FontBuffer& buf);
    //!--------------------------------------------------------------------------
    //
    //
    //
    //
    //
    //---------------------------------------------------------------------------
    void DrawBuffer(const FontBuffer& buf, s32 posh=0, s32 posv=0);

private:
    FontDrawer();
    ~FontDrawer();
    void InitShaders();
    void InitFont();
    void InitDraw(u32 width, u32 height);
    void SetupTextCamera(nw::font::RectDrawer* pDrawer, u32 width, u32 height);
    nw::font::DispStringBuffer* AllocDispStringBuffer(u32 charMax);
    void CleanupFont(nw::font::ResFont* pFont);

private:
    static FontDrawer*          m_pInstance;
    nw::math::VEC2              mScreenSize;
    nw::font::ResFont           mFont;
    nw::font::RectDrawer        mDrawer;
    nw::font::DispStringBuffer* mDrawStringBuf;
    nw::font::TextWriter        mWriter;
    void*                       mVtxBufCmdBuf;
    nw::ut::Color8              mColor;
};



/*
  
 */
class Draw2DHelper
{
public:
    static Draw2DHelper& GetInstance();

    /*
      
     */
    static void Initialize( u32 TriangleBufferSize = 1024 );

    /*
      
     */
    static void Finalize();

    /*
      

      
      
     */
    void SetScreenSize(u32 width, u32 height);

    /*
      
     */
    void Begin();

    /*
      
     */
    void BeginDraw();

    /*
      
     */
    void EndDraw();

    /*
      
     */
    void End();

    /*
      
     */
    void Flush();

    /*
      

      
     */
    void SetLineWidth(f32 width) { m_LineWidth = width; }

    /*
      

      
      
      
      
      
     */
    void DrawLine( s32 x1, s32 y1, s32 x2, s32 y2, nw::ut::Color8 color )
    {
        this->DrawLine(
            nw::math::VEC2(static_cast<f32>(x1), static_cast<f32>(y1)),
            nw::math::VEC2(static_cast<f32>(x2), static_cast<f32>(y2)),
            color
        );
    }

    /*
      

      
      
      
     */
    void DrawLine( const nw::math::VEC2& p1, const nw::math::VEC2& p2, nw::ut::Color8 color );

    /*
      

      
      
      
      
      
     */
    void DrawRectangle( s32 posh, s32 posv, s32 sizeh, s32 sizev, nw::ut::Color8 color )
    {
        this->DrawRectangle(
            nw::math::VEC2(static_cast<f32>(posh), static_cast<f32>(posv)),
            nw::math::VEC2(static_cast<f32>(sizeh), static_cast<f32>(sizev)),
            color
        );
    }

    /*
      

      
      
      
     */
    void DrawRectangle( const nw::math::VEC2& pos, const nw::math::VEC2& size, nw::ut::Color8 color );

    /*
      

      
      
      
      
      
     */
    void FillRectangle( s32 posh, s32 posv, s32 sizeh, s32 sizev, nw::ut::Color8 color )
    {
        this->FillRectangle(
            nw::math::VEC2(static_cast<f32>(posh), static_cast<f32>(posv)),
            nw::math::VEC2(static_cast<f32>(sizeh), static_cast<f32>(sizev)),
            color
        );
    }

    /*
      

      
      
      
     */
    void FillRectangle( const nw::math::VEC2& pos, const nw::math::VEC2& size, nw::ut::Color8 color );

    /*
      

      
      
      
      
     */
    void FillTriangle( const nw::math::VEC2& pt1, const nw::math::VEC2& pt2, const nw::math::VEC2& pt3, nw::ut::Color8 color );

    /*
      

      
      
      
     */
    void FillTriangles( const nw::math::VEC2* pPointArray, s32 pointCount, nw::ut::Color8 color );

    const nn::gr::BindSymbolVSInput* GetSymbolPos() const{return &m_SymbolPos;}
    const nn::gr::BindSymbolVSInput* GetSymbolTex() const{return &m_SymbolTex;}
    const nn::gr::BindSymbolVSInput* GetSymbolClr() const{return &m_SymbolClr;}
    u32*& GetCommand(){return m_Command;}

    enum State{
        ALPHABLEND = 1 << 0,
        DEPTHWRITE = 1 << 1,
        CULLFACE   = 1 << 2,
        DEPTHTEST  = 1 << 3,
        ALPHATEST  = 1 << 4
    };
    typedef u32 STATE;
    void Enable(STATE state);   // (ALPHABLEND | DEPTHWRITE | CULLFACE | DEPTHTEST | ALPHATEST)
    void Disable(STATE staet);  // (ALPHABLEND | DEPTHWRITE | CULLFACE | DEPTHTEST | ALPHATEST)

    void SetDepth(f32 depth){m_Depth = depth;}
    void SetUVScale(f32 u, f32 v){m_UVScale.x=u; m_UVScale.y=v;}

private:
    void BuildTriangle( const nw::math::VEC2& pt1, const nw::math::VEC2& pt2, const nw::math::VEC2& pt3, u32 color );
    void BuildRectangle( const nw::math::VEC2& pt1, const nw::math::VEC2& pt2,
                         const nw::math::VEC2& pt3, const nw::math::VEC2& pt4, u32 color );
    void BuildRectangle( const nw::math::VEC2& pos, const nw::math::VEC2& size, u32 color );
    void BuildLine( const nw::math::VEC2& pt1, const nw::math::VEC2& pt2, f32 lineWidth, u32 color );
    void BuildTriangle( const nw::math::VEC2* pos, const nw::math::VEC2* uv, u32 color );

private:
    Draw2DHelper(u32 TriangleBufferSize);
    ~Draw2DHelper();

private:
    static Draw2DHelper* m_pInstance;

    u8*                         m_ShaderBinary;
    nn::gr::Shader              m_Shader;
    nn::gr::BindSymbolVSInput   m_SymbolPos;
    nn::gr::BindSymbolVSInput   m_SymbolTex;
    nn::gr::BindSymbolVSInput   m_SymbolClr;
    nn::gr::BindSymbolVSFloat   m_SymbolScale;
    u32*                        m_Command;
    const u32*                  m_StartAddr;
    nn::gr::Viewport            m_Viewport;
    nn::gr::Scissor             m_Scissor;
    nn::gr::CTR::RenderState    m_RenderState;
    u32                         m_PrimitiveCount;
    u32                         m_PrimitiveSize;
    u32                         m_PrimitiveStart;
    u32                         m_Width;
    u32                         m_Height;
    f32                         m_LineWidth;
    f32                         m_Depth;
    nn::math::VEC2              m_UVScale;
};

/*
  
 */
class Texture
{
public:
    Texture();
    Texture(const char* filename, s16 index = 0);
    Texture(const char* filename, const char* tex_name);
    Texture(const void* data, u32 width, u32 height);
    ~Texture();
    bool SetData(const void* data, const char* tex_name);
    void SetState();
    void SetCombiner(nn::gr::Combiner combiner);
    void SetFilter(PicaDataTextureMagFilter magFilter=PICA_DATA_TEXTURE_MAG_FILTER_NEAREST,
                   PicaDataTextureMinFilter minFilter=PICA_DATA_TEXTURE_MIN_FILTER_NEAREST);
    void SetWrapMode(PicaDataTextureWrap T=PICA_DATA_TEXTURE_WRAP_REPEAT,
                     PicaDataTextureWrap S=PICA_DATA_TEXTURE_WRAP_REPEAT);
    static void ResetState();
private:
    void LoadTexture(const char* filename, s16 index);
    void LoadTexture(const char* filename, const char* tex_name);
    void SetupData(const void* buffer, s16 index);
    void* LoadData(const char* filename);
    void InitState();

private:
    void*                   m_Data;
    nn::tpl::CtrTextureInfo m_Info;
    nn::gr::Texture         m_Texture;
    nn::gr::Combiner        m_Combiner;
};



} // namespace system
} // namespace vw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif // NW4CVIEWER_DRAW2DHELPER_H_
