/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/


#ifndef NW4C_VIEWER_SYSTEM_HID_H_
#define NW4C_VIEWER_SYSTEM_HID_H_

#include <nn/hid.h>
#include <nn/os.h>
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw     {
namespace system {

#define HOLD(key) \

#define PAD_TRIGGER(x) (this->pad.trigger & nn::hid::CTR::BUTTON_##x)
#define PAD_HOLD(x)    (this->pad.hold    & nn::hid::CTR::BUTTON_##x)
#define HID_RELEASE(x) (this->pad.release & nn::hid::CTR::BUTTON_##x)
#define HID_STICK(x)   (this->pad.stick   & nn::hid::CTR::BUTTON_##x)
#define DBG_TRIGGER(x) (this->debugPad.trigger & nn::hid::CTR::DEBUG_PAD_##x)
#define DBG_HOLD(x)    (this->debugPad.hold    & nn::hid::CTR::DEBUG_PAD_##x)
#define DBG_RELEASE(x) (this->debugPad.release & nn::hid::CTR::DEBUG_PAD_##x)
#define DBG_STICK(x)   (this->debugPad.stick   & nn::hid::CTR::DEBUG_PAD_##x)

struct HidStatus
{
    nn::hid::CTR::PadStatus             pad;
    nn::hid::CTR::AccelerometerStatus   accelerometer;
    NN_PADDING2;
    nn::hid::CTR::DebugPadStatus        debugPad;
    nn::hid::CTR::TouchPanelStatus      touchPanel;
    nn::hid::CTR::PadReader*            padReader;

    bool                                isClicking;
    bool                                isClicked;
    u16                                 clickPosX;
    u16                                 clickPosY;

    HidStatus()
     : isClicking(false)
     , isClicked(false)
     , clickPosX(0)
     , clickPosY(0)
     , m_Flag(true) {}

    bool Get()
    {
        bool tmp = m_Flag;
        m_Flag = true;
        return tmp;
    }

    bool GetHoldR(){ return PAD_HOLD(R) || DBG_HOLD(TRIGGER_ZR); }
    bool GetHoldL(){ return PAD_HOLD(L) || DBG_HOLD(TRIGGER_ZL); }

    bool GetHoldStart(){ return PAD_HOLD(START) || DBG_HOLD(BUTTON_MINUS) || DBG_HOLD(BUTTON_PLUS); }

    bool GetHoldLeft() { return PAD_HOLD(LEFT)  || DBG_HOLD(BUTTON_LEFT); }
    bool GetHoldRight(){ return PAD_HOLD(RIGHT) || DBG_HOLD(BUTTON_RIGHT); }
    bool GetHoldUp()   { return PAD_HOLD(UP)    || DBG_HOLD(BUTTON_UP); }
    bool GetHoldDown() { return PAD_HOLD(DOWN)  || DBG_HOLD(BUTTON_DOWN); }

    bool GetHoldA(){ return PAD_HOLD(A) || DBG_HOLD(BUTTON_A); }
    bool GetHoldB(){ return PAD_HOLD(B) || DBG_HOLD(BUTTON_B); }
    bool GetHoldX(){ return PAD_HOLD(X) || DBG_HOLD(BUTTON_X); }
    bool GetHoldY(){ return PAD_HOLD(Y) || DBG_HOLD(BUTTON_Y); }

    bool GetTriggerLeft() { return PAD_TRIGGER(LEFT)  || DBG_TRIGGER(BUTTON_LEFT); }
    bool GetTriggerRight(){ return PAD_TRIGGER(RIGHT) || DBG_TRIGGER(BUTTON_RIGHT); }
    bool GetTriggerUp()   { return PAD_TRIGGER(UP)    || DBG_TRIGGER(BUTTON_UP); }
    bool GetTriggerDown() { return PAD_TRIGGER(DOWN)  || DBG_TRIGGER(BUTTON_DOWN); }

    bool GetTriggerA(){ return PAD_TRIGGER(A) || DBG_TRIGGER(BUTTON_A); }
    bool GetTriggerB(){ return PAD_TRIGGER(B) || DBG_TRIGGER(BUTTON_B); }
    bool GetTriggerX(){ return PAD_TRIGGER(X) || DBG_TRIGGER(BUTTON_X); }
    bool GetTriggerY(){ return PAD_TRIGGER(Y) || DBG_TRIGGER(BUTTON_Y); }

    bool GetTriggerStart(){ return PAD_TRIGGER(START) || DBG_TRIGGER(BUTTON_MINUS) || DBG_TRIGGER(BUTTON_PLUS); }

    HidStatus&
    HoldR(bool b = true)
    {
        m_Flag &= (PAD_HOLD(R) || DBG_HOLD(TRIGGER_ZR)) == b;
        return *this;
    }

    HidStatus&
    HoldL(bool b = true)
    {
        m_Flag &= (PAD_HOLD(L) || DBG_HOLD(TRIGGER_ZL)) == b;
        return *this;
    }

    HidStatus&
    HoldStart(bool b = true)
    {
        m_Flag &= (PAD_HOLD(START) || DBG_HOLD(BUTTON_MINUS) || DBG_HOLD(BUTTON_PLUS)) == b;
        return *this;
    }

    HidStatus&
    HoldLeft(bool b = true)
    {
        m_Flag &= (PAD_HOLD(LEFT) || DBG_HOLD(BUTTON_LEFT)) == b;
        return *this;
    }

    HidStatus&
    HoldRight(bool b = true)
    {
        m_Flag &= (PAD_HOLD(RIGHT) || DBG_HOLD(BUTTON_RIGHT)) == b;
        return *this;
    }

    HidStatus&
    HoldUp(bool b = true)
    {
        m_Flag &= (PAD_HOLD(UP) || DBG_HOLD(BUTTON_UP)) == b;
        return *this;
    }

    HidStatus&
    HoldDown(bool b = true)
    {
        m_Flag &= (PAD_HOLD(DOWN) || DBG_HOLD(BUTTON_DOWN)) == b;
        return *this;
    }

    HidStatus&
    HoldA(bool b = true)
    {
        m_Flag &= (PAD_HOLD(A) || DBG_HOLD(BUTTON_A)) == b;
        return *this;
    }

    HidStatus&
    HoldB(bool b = true)
    {
        m_Flag &= (PAD_HOLD(B) || DBG_HOLD(BUTTON_B)) == b;
        return *this;
    }

    HidStatus&
    HoldX(bool b = true)
    {
        m_Flag &= (PAD_HOLD(X) || DBG_HOLD(BUTTON_X)) == b;
        return *this;
    }

    HidStatus&
    HoldY(bool b = true)
    {
        m_Flag &= (PAD_HOLD(Y) || DBG_HOLD(BUTTON_Y)) == b;
        return *this;
    }

    HidStatus&
    TriggerA()
    {
        m_Flag &= (PAD_TRIGGER(A) || DBG_TRIGGER(BUTTON_A));
        return *this;
    }

    HidStatus&
    TriggerB()
    {
        m_Flag &= (PAD_TRIGGER(B) || DBG_TRIGGER(BUTTON_B));
        return *this;
    }

    HidStatus&
    TriggerX()
    {
        m_Flag &= (PAD_TRIGGER(X) || DBG_TRIGGER(BUTTON_X));
        return *this;
    }

    HidStatus&
    TriggerY()
    {
        m_Flag &= (PAD_TRIGGER(Y) || DBG_TRIGGER(BUTTON_Y));
        return *this;
    }

    HidStatus&
    TriggerRight()
    {
        m_Flag &= (PAD_TRIGGER(RIGHT) || DBG_TRIGGER(BUTTON_RIGHT));
        return *this;
    }

    HidStatus&
    TriggerLeft()
    {
        m_Flag &= (PAD_TRIGGER(LEFT) || DBG_TRIGGER(BUTTON_LEFT));
        return *this;
    }

    HidStatus&
    TriggerUp()
    {
        m_Flag &= (PAD_TRIGGER(UP) || DBG_TRIGGER(BUTTON_UP));
        return *this;
    }

    HidStatus&
    TriggerDown()
    {
        m_Flag &= (PAD_TRIGGER(DOWN) || DBG_TRIGGER(BUTTON_DOWN));
        return *this;
    }

    bool m_Flag;
    NN_PADDING3;
};


#undef PAD_TRIGGER
#undef PAD_HOLD
#undef HID_RELEASE
#undef HID_STICK
#undef DBG_TRIGGER
#undef DBG_HOLD
#undef DBG_RELEASE
#undef DBG_STICK


/*---------------------------------------------------------------------------*
  Name:         ButtonEnums
  Description:
 *---------------------------------------------------------------------------
*/
enum ButtonEnums
{
    HLD_AWHILE = 0x01 << 0, // This bit is set if held for more than a fixed time.
    HLD_L     =  0x01 << 1,
    HLD_R     =  0x01 << 2,
    HLD_LR    =  HLD_L | HLD_R,
    HLD_Start =  0x01 << 3,
    //HLD_Select=  0x01 << 4, // As of 2010/12/14, determination input is not possible with Select.
    HLD_A     =  0x01 << 5,
    HLD_B     =  0x01 << 6,
    HLD_X     =  0x01 << 7,
    HLD_Y     =  0x01 << 8,
    HLD_Left  =  0x01 << 9,
    HLD_Right =  0x01 << 10,
    HLD_Up    =  0x01 << 11,
    HLD_Down  =  0x01 << 12,
    TRG_A     =  0x01 << 13 | HLD_A,
    TRG_B     =  0x01 << 14 | HLD_B,
    TRG_X     =  0x01 << 15 | HLD_X,
    TRG_Y     =  0x01 << 16 | HLD_Y,
    TRG_Left  =  0x01 << 17 | HLD_Left,
    TRG_Right =  0x01 << 18 | HLD_Right,
    TRG_Up    =  0x01 << 19 | HLD_Up,
    TRG_Down  =  0x01 << 20 | HLD_Down,
    TRG_Start =  0x01 << 21 | HLD_Start
};

/*---------------------------------------------------------------------------*
  Name:         GetHidBit
  Description:  Generates an unique bit sequence from the Pad input information.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
inline
u32 GetHidBit( vw::system::HidStatus &hidStatus )
{
    static u32 prevBit = 0;
    u32 padBit = 0;
    if ( hidStatus.GetTriggerA() )      padBit |= TRG_A;
    if ( hidStatus.GetTriggerB() )      padBit |= TRG_B;
    if ( hidStatus.GetTriggerX() )      padBit |= TRG_X;
    if ( hidStatus.GetTriggerY() )      padBit |= TRG_Y;
    if ( hidStatus.GetTriggerUp() )     padBit |= TRG_Up;
    if ( hidStatus.GetTriggerDown() )   padBit |= TRG_Down;
    if ( hidStatus.GetTriggerRight() )  padBit |= TRG_Right;
    if ( hidStatus.GetTriggerLeft() )   padBit |= TRG_Left;
    if ( hidStatus.GetTriggerStart() )  padBit |= TRG_Start;

    if ( hidStatus.GetHoldA() )         padBit |= HLD_A;
    if ( hidStatus.GetHoldB() )         padBit |= HLD_B;
    if ( hidStatus.GetHoldX() )         padBit |= HLD_X;
    if ( hidStatus.GetHoldY() )         padBit |= HLD_Y;
    if ( hidStatus.GetHoldUp() )        padBit |= HLD_Up;
    if ( hidStatus.GetHoldDown() )      padBit |= HLD_Down;
    if ( hidStatus.GetHoldRight() )     padBit |= HLD_Right;
    if ( hidStatus.GetHoldLeft() )      padBit |= HLD_Left;

    if ( hidStatus.GetHoldR() )         padBit |= HLD_R;
    if ( hidStatus.GetHoldL() )         padBit |= HLD_L;
    if ( hidStatus.GetHoldStart() )     padBit |= HLD_Start;

    static nn::os::Tick startTick;
    if ( padBit != 0)
    {
        // If the state in which bits excluding HLD_AWHILE do not change continues for more than a fixed time, the HLD_AWHILE flag is set
        // 
        if ( (prevBit & ~HLD_AWHILE) == (padBit & ~HLD_AWHILE) )
        {
            nn::os::Tick currentTick = nn::os::Tick::GetSystemCurrent();

            const f32 NANO_TO_MILLI = 1.0f / (1000.0f * 1000.0f);
            s64 span = (currentTick - startTick).ToTimeSpan().GetNanoSeconds();
            f64 elapsedTime = static_cast<f64>(span) * NANO_TO_MILLI;
            // The threshold is 300 ms
            if (elapsedTime > 300)
            {
                padBit |= HLD_AWHILE;
            }
        }
        else
        {
            if (padBit) startTick = nn::os::Tick::GetSystemCurrent();
        }
    }
    prevBit = padBit;

    return padBit;
}

}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif



/* NW4C_VIEWER_SYSTEM_HID_H_ */
#endif
