/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_LOAD_FILE_H_
#define NW4C_VIEWER_SYSTEM_LOAD_FILE_H_

namespace vw     {
namespace system {

/* If loading the file failed, the following negative value can be acquired in pSize as an error code.       *
                           *
 * When loading succeeds, a positive value file size is stored */
enum
{
    LOAD_FILE_RESULT_NOT_FOUND       = -1,  // Cannot find file
    LOAD_FILE_RESULT_TRANSFER_ERROR  = -2,  // Failed to transfer file
    LOAD_FILE_RESULT_MEM_ALLOC_ERROR = -3   // Failed to allocate memory
};

//void*   LoadFile_FromDVD( u16 moduleID, const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL );
void*   LoadFile_FromHIO( u16 moduleID, const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL );

}   /* namespace system */

namespace g3d {

//inline void*
//LoadFile_FromDVD( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromROM( system::G3D_ID, path, pSize, allocator ); }
inline void*
LoadFile_FromHIO( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromHIO( system::G3D_ID, path, pSize, allocator ); }

} /* namespace g3d */

namespace effect {

//inline void*
//LoadFile_FromDVD( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromROM( system::EFFECT_ID, path, pSize, allocator ); }
inline void*
LoadFile_FromHIO( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromHIO( system::EFFECT_ID, path, pSize, allocator ); }

} /* namespace effect */

namespace g2d {

//inline void*
//LoadFile_FromDVD( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromROM( system::G2D_ID, path, pSize, allocator ); }
inline void*
LoadFile_FromHIO( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromHIO( system::G2D_ID, path, pSize, allocator ); }

} /* namespace g2d */

namespace sound {

//inline void*
//LoadFile_FromDVD( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromROM( system::SOUND_ID, path, pSize, allocator ); }
inline void*
LoadFile_FromHIO( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromHIO( system::SOUND_ID, path, pSize, allocator ); }

} /* namespace sound */

#if defined(USE_SAMPLE)
namespace sample {

//inline void*
//LoadFile_FromDVD( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromROM( system::SAMPLE_ID, path, pSize, allocator ); }
inline void*
LoadFile_FromHIO( const char* path, s32* pSize, nw::os::IAllocator* allocator = NULL ) { return system::LoadFile_FromHIO( system::SAMPLE_ID, path, pSize, allocator ); }

} /* namespace sample */
#endif



}   /* namespace vw     */



/* NW4C_VIEWER_SYSTEM_LOAD_FILE_H_ */
#endif
