/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_TOUCH_PANEL_H_
#define NW4C_VIEWER_SYSTEM_TOUCH_PANEL_H_

#include <viewer/system/AppId.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw     {
namespace system {


struct TouchPanelInfo
{
    u16    x;           // 5 <= x <= 314
    u16    y;           // 5 <= y <= 234
    u16    startX;      // 5 <= x <= 314 Start drag X
    u16    startY;      // 5 <= y <= 234 Start drag Y
    u8     touch;       // 0 : Pen up, 1: Touch
    u8     press;       // Input value
    u8     trig;        // Push trigger input
    u8     release;     // Release trigger input
    u8     repeat;      // Drag repeat input
    u8     repeatAccel; // accelerated Repeat input
    u16    disabled;    // Flag of whether invalid
};


void    TouchPanel_Init( void );

void    TouchPanel_Read( void );

void    TouchPanel_GetInfo             ( u16 appId, TouchPanelInfo* panelInfo );

bool    TouchPanel_IsPressed           ( u16 appId );

bool    TouchPanel_IsTriggered         ( u16 appId );

bool    TouchPanel_IsReleased          ( u16 appId );

bool    TouchPanel_IsRepeated          ( u16 appId );

bool    TouchPanel_IsClicked           ( u16 appId );

bool    TouchPanel_IsDragged           ( u16 appId );

void    TouchPanel_GetXY               ( u16 appId, u16* posX, u16* posY );

void    TouchPanel_GetStartXY          ( u16 appId, u16* posX, u16* posY );

bool    TouchPanel_IsEnable            ( u16 appId );

bool    TouchPanel_GetRawInfo_         ( nn::hid::CTR::TouchPanelStatus* pad );



inline bool
TouchPanel_IsTriggerToMenu( u16 appId )
{
    return false;
}



/*---------------------------------------------------------------------------*
  Name:         TouchPanel
  Description:  Touch panel-related functions wrapper class
 *---------------------------------------------------------------------------*/
class TouchPanel
{
public:
    /* ctor */  explicit TouchPanel ( u16 appId  ) : mAppId( appId ) {}
    virtual /* dtor */ ~TouchPanel() {}
    bool         IsPressed      ( void ) const { return TouchPanel_IsPressed( mAppId );   }
    bool         IsTriggered    ( void ) const { return TouchPanel_IsTriggered( mAppId ); }
    bool         IsReleased     ( void ) const { return TouchPanel_IsReleased( mAppId );  }
    bool         IsRepeated     ( void ) const { return TouchPanel_IsRepeated( mAppId );  }
    bool         IsClicked      ( void ) const { return TouchPanel_IsClicked( mAppId );   }
    bool         IsDragged      ( void ) const { return TouchPanel_IsDragged( mAppId );   }

    void        GetXY     ( u16* posX, u16* posY ) const { TouchPanel_GetXY( mAppId, posX, posY );         }
    void        GetStartXY( u16* posX, u16* posY ) const { TouchPanel_GetStartXY( mAppId, posX, posY );    }
    void        GetInfo   ( TouchPanelInfo* panelInfo ) const { TouchPanel_GetInfo( mAppId, panelInfo );   }

    bool        IsTriggerToMenu( void ) const { return TouchPanel_IsTriggerToMenu( mAppId ); }
    bool        IsEnable       ( void ) const { return TouchPanel_IsEnable( mAppId ); }
private:
    u16         mAppId;     // Module ID
};


}   /* namespace system */

namespace g3d {

class TouchPanel : public system::TouchPanel
{
    /* ctor */ TouchPanel() : system::TouchPanel( G3D_ID ) {}
    static TouchPanel      GetInstance() { return TouchPanel(); }
};

inline void TouchPanel_GetInfo      ( system::TouchPanelInfo* panelInfo ) { system::TouchPanel_GetInfo( G3D_ID, panelInfo ); }
inline bool TouchPanel_IsPressed    ( void ) { return system::TouchPanel_IsPressed( G3D_ID ); }
inline bool TouchPanel_IsTriggered  ( void ) { return system::TouchPanel_IsTriggered( G3D_ID ); }
inline bool TouchPanel_IsReleased   ( void ) { return system::TouchPanel_IsReleased( G3D_ID ); }
inline bool TouchPanel_IsRepeated   ( void ) { return system::TouchPanel_IsRepeated( G3D_ID ); }
inline bool TouchPanel_IsClicked    ( void ) { return system::TouchPanel_IsClicked( G3D_ID ); }
inline bool TouchPanel_IsDragged    ( void ) { return system::TouchPanel_IsDragged( G3D_ID );    }
inline void TouchPanel_GetXY        ( u16* posX, u16* posY ) { system::TouchPanel_GetXY( G3D_ID, posX, posY ); }
inline void TouchPanel_GetStartXY   ( u16* posX, u16* posY ) { system::TouchPanel_GetStartXY( G3D_ID, posX, posY ); }
inline bool TouchPanel_IsEnable     ( void ) { return system::TouchPanel_IsEnable( G3D_ID ); }

} /* namespace g3d */

namespace effect {

class TouchPanel : public system::TouchPanel
{
    /* ctor */ TouchPanel() : system::TouchPanel( EFFECT_ID ) {}
    static TouchPanel      GetInstance() { return TouchPanel(); }
};

inline void TouchPanel_GetInfo      ( system::TouchPanelInfo* panelInfo ) { system::TouchPanel_GetInfo( EFFECT_ID, panelInfo ); }
inline bool TouchPanel_IsPressed    ( void ) { return system::TouchPanel_IsPressed( EFFECT_ID ); }
inline bool TouchPanel_IsTriggered  ( void ) { return system::TouchPanel_IsTriggered( EFFECT_ID ); }
inline bool TouchPanel_IsReleased   ( void ) { return system::TouchPanel_IsReleased( EFFECT_ID ); }
inline bool TouchPanel_IsRepeated   ( void ) { return system::TouchPanel_IsRepeated( EFFECT_ID ); }
inline bool TouchPanel_IsClicked    ( void ) { return system::TouchPanel_IsClicked( EFFECT_ID ); }
inline bool TouchPanel_IsDragged    ( void ) { return system::TouchPanel_IsDragged( EFFECT_ID );    }
inline void TouchPanel_GetXY        ( u16* posX, u16* posY ) { system::TouchPanel_GetXY( EFFECT_ID, posX, posY ); }
inline void TouchPanel_GetStartXY   ( u16* posX, u16* posY ) { system::TouchPanel_GetStartXY( EFFECT_ID, posX, posY ); }
inline bool TouchPanel_IsEnable     ( void ) { return system::TouchPanel_IsEnable( EFFECT_ID ); }

} /* namespace effect */

namespace g2d {

class TouchPanel : public system::TouchPanel
{
    /* ctor */ TouchPanel() : system::TouchPanel( G2D_ID ) {}
    static TouchPanel      GetInstance() { return TouchPanel(); }
};

inline void TouchPanel_GetInfo      ( system::TouchPanelInfo* panelInfo ) { system::TouchPanel_GetInfo( G2D_ID, panelInfo ); }
inline bool TouchPanel_IsPressed    ( void ) { return system::TouchPanel_IsPressed( G2D_ID ); }
inline bool TouchPanel_IsTriggered  ( void ) { return system::TouchPanel_IsTriggered( G2D_ID ); }
inline bool TouchPanel_IsReleased   ( void ) { return system::TouchPanel_IsReleased( G2D_ID ); }
inline bool TouchPanel_IsRepeated   ( void ) { return system::TouchPanel_IsRepeated( G2D_ID ); }
inline bool TouchPanel_IsClicked    ( void ) { return system::TouchPanel_IsClicked( G2D_ID ); }
inline bool TouchPanel_IsDragged    ( void ) { return system::TouchPanel_IsDragged( G2D_ID );    }
inline void TouchPanel_GetXY        ( u16* posX, u16* posY ) { system::TouchPanel_GetXY( G2D_ID, posX, posY ); }
inline void TouchPanel_GetStartXY   ( u16* posX, u16* posY ) { system::TouchPanel_GetStartXY( G2D_ID, posX, posY ); }
inline bool TouchPanel_IsEnable     ( void ) { return system::TouchPanel_IsEnable( G2D_ID ); }

} /* namespace g2d */

namespace sound {

class TouchPanel : public system::TouchPanel
{
    /* ctor */ TouchPanel() : system::TouchPanel( SOUND_ID ) {}
    static TouchPanel      GetInstance() { return TouchPanel(); }
};

inline void TouchPanel_GetInfo      ( system::TouchPanelInfo* panelInfo ) { system::TouchPanel_GetInfo( SOUND_ID, panelInfo ); }
inline bool TouchPanel_IsPressed    ( void ) { return system::TouchPanel_IsPressed( SOUND_ID ); }
inline bool TouchPanel_IsTriggered  ( void ) { return system::TouchPanel_IsTriggered( SOUND_ID ); }
inline bool TouchPanel_IsReleased   ( void ) { return system::TouchPanel_IsReleased( SOUND_ID ); }
inline bool TouchPanel_IsRepeated   ( void ) { return system::TouchPanel_IsRepeated( SOUND_ID ); }
inline bool TouchPanel_IsClicked    ( void ) { return system::TouchPanel_IsClicked( SOUND_ID ); }
inline bool TouchPanel_IsDragged    ( void ) { return system::TouchPanel_IsDragged( SOUND_ID );    }
inline void TouchPanel_GetXY        ( u16* posX, u16* posY ) { system::TouchPanel_GetXY( SOUND_ID, posX, posY ); }
inline void TouchPanel_GetStartXY   ( u16* posX, u16* posY ) { system::TouchPanel_GetStartXY( SOUND_ID, posX, posY ); }
inline bool TouchPanel_IsEnable     ( void ) { return system::TouchPanel_IsEnable( SOUND_ID ); }

} /* namespace sound */



}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif



/* NW4C_VIEWER_SYSTEM_TOUCH_PANEL_H_ */
#endif
