/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_SYSTEM_VWPAD_H_
#define NW4R_VIEWER_SYSTEM_VWPAD_H_

//#include <nw4r/config.h>
#include <viewer/system/AppId.h>

#include <viewer/system/Pad.h>
#include <viewer/system/WPad.h>

namespace vw     {
namespace system {

#define VWPAD_BUTTON_LEFT      PAD_BUTTON_LEFT      // 0x0001
#define VWPAD_BUTTON_RIGHT     PAD_BUTTON_RIGHT     // 0x0002
#define VWPAD_BUTTON_DOWN      PAD_BUTTON_DOWN      // 0x0004
#define VWPAD_BUTTON_UP        PAD_BUTTON_UP        // 0x0008
#define VWPAD_BUTTON_Z         PAD_TRIGGER_Z        // 0x0010
#define VWPAD_BUTTON_R         PAD_TRIGGER_R        // 0x0020
#define VWPAD_BUTTON_L         PAD_TRIGGER_L        // 0x0040
#define VWPAD_BUTTON_A         PAD_BUTTON_A         // 0x0100
#define VWPAD_BUTTON_B         PAD_BUTTON_B         // 0x0200
#define VWPAD_BUTTON_X         PAD_BUTTON_X         // 0x0400
#define VWPAD_BUTTON_Y         PAD_BUTTON_Y         // 0x0800
#define VWPAD_BUTTON_START     PAD_BUTTON_START     // 0x1000
#define VWPAD_STICK_LEFT       PAD_STICK_LEFT       // 0x0080
#define VWPAD_STICK_RIGHT      PAD_STICK_RIGHT      // 0x2000
#define VWPAD_STICK_DOWN       PAD_STICK_DOWN       // 0x4000
#define VWPAD_STICK_UP         PAD_STICK_UP         // 0x8000

#define VWPAD_WBUTTON_LEFT     VWPAD_BUTTON_LEFT    // 0x0001
#define VWPAD_WBUTTON_RIGHT    VWPAD_BUTTON_RIGHT   // 0x0002
#define VWPAD_WBUTTON_DOWN     VWPAD_BUTTON_DOWN    // 0x0004
#define VWPAD_WBUTTON_UP       VWPAD_BUTTON_UP      // 0x0008

#define VWPAD_WBUTTON_HOME     VWPAD_BUTTON_START   // 0x1000

#define VWPAD_WSTICK_LEFT      WPAD_STICK_LEFT      // 0x00010000
#define VWPAD_WSTICK_RIGHT     WPAD_STICK_RIGHT     // 0x00020000
#define VWPAD_WSTICK_DOWN      WPAD_STICK_DOWN      // 0x00040000
#define VWPAD_WSTICK_UP        WPAD_STICK_UP        // 0x00080000

#define VWPAD_WBUTTON_A        (WPAD_BUTTON_A     << 16) // 0x08000000
#define VWPAD_WBUTTON_B        (WPAD_BUTTON_B     << 16) // 0x04000000
#define VWPAD_WBUTTON_1        (WPAD_BUTTON_1     << 16) // 0x02000000
#define VWPAD_WBUTTON_2        (WPAD_BUTTON_2     << 16) // 0x01000000
#define VWPAD_WBUTTON_PLUS     (WPAD_BUTTON_PLUS  << 16) // 0x00100000
#define VWPAD_WBUTTON_MINUS    (WPAD_BUTTON_MINUS << 16) // 0x10000000
#define VWPAD_WBUTTON_Z        (WPAD_BUTTON_Z     << 16) // 0x20000000
#define VWPAD_WBUTTON_C        (WPAD_BUTTON_C     << 16) // 0x40000000

#define VWPAD_BUTTON_PLUS_MASK  (VWPAD_BUTTON_LEFT | VWPAD_BUTTON_RIGHT | VWPAD_BUTTON_UP | VWPAD_BUTTON_DOWN)
#define VWPAD_STICK_PLUS_MASK   (VWPAD_STICK_LEFT | VWPAD_STICK_RIGHT | VWPAD_STICK_UP | VWPAD_STICK_DOWN)
#define VWPAD_WBUTTON_PLUS_MASK (VWPAD_WBUTTON_LEFT | VWPAD_WBUTTON_RIGHT | VWPAD_WBUTTON_UP | VWPAD_WBUTTON_DOWN)
#define VWPAD_WSTICK_PLUS_MASK  (VWPAD_WSTICK_LEFT  | VWPAD_WSTICK_RIGHT  | VWPAD_WSTICK_UP  | VWPAD_WSTICK_DOWN )
#define VWPAD_WBUTTON_BTN_MASK  ((WPAD_BUTTON_A | WPAD_BUTTON_B | WPAD_BUTTON_1 | WPAD_BUTTON_2 | \
                                   WPAD_BUTTON_PLUS | WPAD_BUTTON_MINUS | WPAD_BUTTON_Z | WPAD_BUTTON_C) << 16)

/*---------------------------------------------------------------------------*
  Name:         VWPad
  Description:  Wrapper class for GCPad + Remote-related functions
 *---------------------------------------------------------------------------*/
class VWPad
{
public:
    /* ctor */ explicit     VWPad( u16 appId  ) : mPad ( appId ), mWPad ( appId ){}
    virtual /* dtor */      ~VWPad    () {}

    // Merge functions.
    u32         IsPressed           ( u32 button ) const;
    u32         IsPressedHold       ( u32 button ) const;
    u32         IsTriggered         ( u32 button ) const;
    u32         IsReleased          ( u32 button ) const;
    u32         IsRepeated          ( u32 button ) const;
    u32         IsRepeatedAccel     ( u32 button ) const;
    u32         IsClicked           ( u32 button ) const;
    u32         IsLongPressed       ( u32 button ) const;

    bool        IsTriggerToMenu     ( void ) const { return ( mPad.IsTriggerToMenu() || mWPad.IsTriggerToMenu() );    }
    bool        IsEnable            ( void ) const { return mPad.IsEnable();    }

    // Pad-related
    // These functions merge with the classic controller results.
    u32         IsPressed_PAD       ( u32 button ) const    { return mPad.IsPressed( (u16)button );        }
    u32         IsPressedHold_PAD   ( u32 button ) const    { return mPad.IsPressedHold( (u16)button );    }
    u32         IsTriggered_PAD     ( u32 button ) const    { return mPad.IsTriggered( (u16)button );      }
    u32         IsReleased_PAD      ( u32 button ) const    { return mPad.IsPressed( (u16)button );        }
    u32         IsRepeated_PAD      ( u32 button ) const    { return mPad.IsRepeated( (u16)button );       }
    u32         IsRepeatedAccel_PAD ( u32 button ) const    { return mPad.IsRepeatedAccel( (u16)button );  }
    u32         IsClicked_PAD       ( u32 button ) const    { return mPad.IsClicked( (u16)button );        }
    u32         IsLongPressed_PAD   ( u32 button ) const    { return mPad.IsLongPressed( (u16)button );    }

    void        GetStickXY_PAD      ( s8* stickX, s8* stickY ) const    { mPad.GetStickXY( stickX, stickY );     }
    void        GetSubStickXY_PAD   ( s8* stickX, s8* stickY ) const    { mPad.GetSubStickXY( stickX, stickY );  }
    void        GetKeyInfo          ( PADInfo* keyInfo )       const    { mPad.GetKeyInfo( keyInfo );            }
    u8          GetAnalogL_PAD      ( void ) const                      { return mPad.GetAnalogL(); }
    u8          GetAnalogR_PAD      ( void ) const                      { return mPad.GetAnalogR(); }
    bool        IsTriggerToMenu_PAD ( void ) const { return mPad.IsTriggerToMenu();    }

    // WPad-related
    // Wii controller-specific processing
    bool        GetPointXY_WPAD     ( s16 *px, s16 *py ) const  { return mWPad.GetPointXY( px, py );      }
    bool        GetPrevXY_WPAD      ( s16 *px, s16 *py ) const  { return mWPad.GetPrevXY( px, py );       }
    bool        GetMoveXY_WPAD      ( s16 *px, s16 *py ) const  { return mWPad.GetMoveXY( px, py );       }

    bool        GetTriggeredXY_WPAD ( s16 *px, s16 *py ) const  { return mWPad.GetTriggeredXY( px, py );  }
    bool        GetVectorXY_WPAD    ( s16 *px, s16 *py ) const  { return mWPad.GetVectorXY( px, py );     }

    bool        IsDown_WPAD         ( void )             const  { return mWPad.IsDown();                  }
    u32         IsPressed_WPAD      ( u32 button )       const;
    u32         IsPressedHold_WPAD  ( u32 button )       const;
    u32         IsTriggered_WPAD    ( u32 button )       const;
    u32         IsReleased_WPAD     ( u32 button )       const;
    u32         IsRepeated_WPAD     ( u32 button )       const;
    u32         IsRepeatedAccel_WPAD( u32 button )       const;
    u32         IsClicked_WPAD      ( u32 button )       const;
    u32         IsDoubleClicked_WPAD( u32 button )       const;
    u32         IsLongPressed_WPAD  ( u32 button )       const;

    u32         IsModifiedC_WPAD    () const    { return mWPad.IsModifiedC();     }
    u32         IsModifiedZ_WPAD    () const    { return mWPad.IsModifiedZ();     }

    f32         GetDist_WPAD        () const    { return mWPad.GetDist();         }
    f32         GetDistVec_WPAD     () const    { return mWPad.GetDistVec();      }

    bool        GetStickXY_WPAD     ( f32 *px, f32 *py ) const { return mWPad.GetStickFS( px, py );      }

    u32         GetDevType_WPAD     ( void )             const { return mWPad.GetDevType();              }

    //bool        GetPadStatus_WPAD( KPADStatus* kpad )    const { return mWPad.GetPadStatus( kpad );      }

    bool        IsConnected_WPAD    ( void )             const { return mWPad.IsConnected();             }

    u32         IsPressed_WithDPD      ( u32 button ) const;
    u32         IsPressedHold_WithDPD  ( u32 button ) const;
    u32         IsTriggered_WithDPD    ( u32 button ) const;
    u32         IsReleased_WithDPD     ( u32 button ) const;
    u32         IsRepeated_WithDPD     ( u32 button ) const;
    u32         IsRepeatedAccel_WithDPD( u32 button ) const;
    u32         IsClicked_WithDPD      ( u32 button ) const;
    u32         IsLongPressed_WithDPD  ( u32 button ) const;

    // Merge functions (menu only use)
    u32         IsPressed_MENU  ( u32 button )     const;
    u32         IsTriggered_MENU( u32 button )     const;
    u32         IsRepeated_MENU ( u32 button )     const;
    u32         IsRepeatedAccel_MENU( u32 button ) const;

private:
    Pad         mPad;       // Pad class
    WPad        mWPad;      // WPad class

    static u32  ConvertToWPadButtonForMenu_( u32 vwpad_button );
    static u32  ConvertToWPadButton_( u32 vwpad_button );

    static u32  ConvertToPadButtonForMenu_( u32 vwpad_button );

};


/*---------------------------------------------------------------------------*
  Name:         VWPad_IsTriggerToMenu
  Description:  Determines menu open/close trigger
  Arguments:    appId
  Returns:
 *---------------------------------------------------------------------------*/
inline bool
VWPad_IsTriggerToMenu( u16 appId )
{
    return bool( Pad_IsTriggerToMenu( appId ) | WPad_IsTriggerToMenu( appId ) );
}

/*---------------------------------------------------------------------------*
  Name:         VWPad_IsPressed_MENU

  Description:  Determines +Control Pad in menu

  Arguments:    appId
                button

  Returns:
 *---------------------------------------------------------------------------*/
inline u32
VWPad_IsPressed_MENU( u16 appId, u32 button )
{
    VWPad  pad( appId );
    return pad.IsPressed_MENU( button );
}

inline u32
VWPad_IsTriggered_MENU( u16 appId, u32 button )
{
    VWPad  pad( appId );
    return pad.IsTriggered_MENU( button );
}

inline u32
VWPad_IsRepeated_MENU( u16 appId, u32 button )
{
    VWPad  pad( appId );
    return pad.IsRepeated_MENU( button );
}

inline u32
VWPad_IsRepeatedAccel_MENU( u16 appId, u32 button )
{
    VWPad  pad( appId );
    return pad.IsRepeatedAccel_MENU( button );
}


//=============================================
// Class declaration to perform control using padInfo
//=============================================
class VWPadFromInfo
{
public:
    /* ctor */ explicit     VWPadFromInfo( const PADInfo *padInfo );
    virtual /* dtor */      ~VWPadFromInfo() {}

    u32         IsPressed           ( u32 button )  const  { return IsPressed_PAD( button );        }
    u32         IsPressedHold       ( u32 /*button*/ )  const  { return 0; }
    u32         IsTriggered         ( u32 button )  const  { return IsTriggered_PAD( button );      }
    u32         IsReleased          ( u32 button )  const  { return IsReleased_PAD( button );       }
    u32         IsRepeated          ( u32 button )  const  { return IsRepeated_PAD( button );       }
    u32         IsRepeatedAccel     ( u32 button )  const  { return IsRepeatedAccel_PAD( button );  }
    u32         IsClicked           ( u32 button )  const  { return IsClicked_PAD( button );        }
    u32         IsLongPressed       ( u32 /*button*/ )  const  { return 0;  }

    bool        IsTriggerToMenu     ( void )        const  { return IsTriggerToMenu_PAD();          }
    bool        IsEnable            ( void ) const { return true; }

    u16         IsPressed_PAD      ( u32 button ) const { return u16( (u16)button & mPadInfo.press   ); }
    u16         IsTriggered_PAD    ( u32 button ) const { return u16( (u16)button & mPadInfo.trig    ); }
    u16         IsReleased_PAD     ( u32 button ) const { return u16( (u16)button & mPadInfo.release ); }
    u16         IsRepeated_PAD     ( u32 button ) const { return u16( (u16)button & mPadInfo.repeat  ); }
    u16         IsRepeatedAccel_PAD( u32 button ) const { return u16( (u16)button & mPadInfo.repeatAccel ); }
    u16         IsClicked_PAD      ( u32 /* button */ ) const { return 0; }
    u16         IsPressedHold_PAD  ( u32 /* button */ ) const { return 0; }
    u16         IsLongPressed_PAD  ( u32 /* button */ ) const { return 0; }

    void        GetStickXY_PAD     ( s8* stickX, s8* stickY ) const { *stickX = mPadInfo.stickX; *stickY = mPadInfo.stickY; }
    void        GetSubStickXY_PAD  ( s8* stickX, s8* stickY ) const { *stickX = mPadInfo.substickX; *stickY = mPadInfo.substickY; }
    void        GetKeyInfo         ( PADInfo* keyInfo )       const { *keyInfo = mPadInfo; }
    u8          GetAnalogL_PAD     ( void ) const { return mPadInfo.triggerLeft;  }
    u8          GetAnalogR_PAD     ( void ) const { return mPadInfo.triggerRight; }

    bool        IsTriggerToMenu_PAD( void ) const { return ( mPadInfo.trig & PAD_BUTTON_START )? true : false; }
    bool        IsEnable_PAD       ( void ) const { return true; }

    bool        GetPointXY_WPAD     ( s16 *, s16 * ) const  { return false; }
    bool        GetPrevXY_WPAD      ( s16 *, s16 * ) const  { return false; }
    bool        GetMoveXY_WPAD      ( s16 *, s16 * ) const  { return false; }

    bool        GetTriggeredXY_WPAD ( s16 *, s16 * ) const  { return false; }
    bool        GetVectorXY_WPAD    ( s16 *, s16 * ) const  { return false; }

    bool        IsDown_WPAD         ()            const  { return false; }
    u32         IsPressed_WPAD      ( u32 )       const  { return 0;     }
    u32         IsPressedHold_WPAD  ( u32 )       const  { return 0;     }
    u32         IsTriggered_WPAD    ( u32 )       const  { return 0;     }
    u32         IsReleased_WPAD     ( u32 )       const  { return 0;     }
    u32         IsRepeated_WPAD     ( u32 )       const  { return 0;     }
    u32         IsRepeatedAccel_WPAD( u32 )       const  { return 0;     }
    u32         IsClicked_WPAD      ( u32 )       const  { return 0;     }
    u32         IsDoubleClicked_WPAD( u32 )       const  { return 0;     }
    u32         IsLongPressed_WPAD  ( u32 )       const  { return 0;     }
    u32         IsModifiedC_WPAD    ()            const  { return 0;     }
    u32         IsModifiedZ_WPAD    ()            const  { return 0;     }

    f32         GetDist_WPAD        () const    { return 0.0f;      }
    f32         GetDistVec_WPAD     () const    { return 0.0f;      }
    bool        GetStickXY_WPAD     ( f32 *, f32 * ) const { return false; }
    //u32         GetDevType_WPAD     ( void ) const         { return WPAD_DEV_NOT_FOUND; }
    //bool        GetPadStatus_WPAD( KPADStatus* ) const     { return false; }
    bool        IsEnable_WPAD       ( void ) const         { return false; }

    u32         IsPressed_WithDPD      ( u32 ) const { return 0; }
    u32         IsPressedHold_WithDPD  ( u32 ) const { return 0; }
    u32         IsTriggered_WithDPD    ( u32 ) const { return 0; }
    u32         IsReleased_WithDPD     ( u32 ) const { return 0; }
    u32         IsRepeated_WithDPD     ( u32 ) const { return 0; }
    u32         IsRepeatedAccel_WithDPD( u32 ) const { return 0; }
    u32         IsClicked_WithDPD      ( u32 ) const { return 0; }
    u32         IsLongPressed_WithDPD  ( u32 ) const { return 0; }

private:
    PADInfo     mPadInfo;
};


inline u32  VWPad_IsPressed      ( u16 appID, u32 button ) { return VWPad( appID ).IsPressed( button );       }
inline u32  VWPad_IsPressedHold  ( u16 appID, u32 button ) { return VWPad( appID ).IsPressedHold( button );   }
inline u32  VWPad_IsTriggered    ( u16 appID, u32 button ) { return VWPad( appID ).IsTriggered( button );     }
inline u32  VWPad_IsReleased     ( u16 appID, u32 button ) { return VWPad( appID ).IsReleased( button );      }
inline u32  VWPad_IsRepeated     ( u16 appID, u32 button ) { return VWPad( appID ).IsRepeated( button );      }
inline u32  VWPad_IsRepeatedAccel( u16 appID, u32 button ) { return VWPad( appID ).IsRepeatedAccel( button ); }
inline u32  VWPad_IsClicked      ( u16 appID, u32 button ) { return VWPad( appID ).IsClicked( button );       }
inline u32  VWPad_IsLongPressed  ( u16 appID, u32 button ) { return VWPad( appID ).IsLongPressed( button );   }

inline u32  VWPad_IsPressed_PAD      ( u16 appID, u32 button ) { return VWPad( appID ).IsPressed_PAD( button );       }
inline u32  VWPad_IsPressedHold_PAD  ( u16 appID, u32 button ) { return VWPad( appID ).IsPressedHold_PAD( button );   }
inline u32  VWPad_IsTriggered_PAD    ( u16 appID, u32 button ) { return VWPad( appID ).IsTriggered_PAD( button );     }
inline u32  VWPad_IsReleased_PAD     ( u16 appID, u32 button ) { return VWPad( appID ).IsReleased_PAD( button );      }
inline u32  VWPad_IsRepeated_PAD     ( u16 appID, u32 button ) { return VWPad( appID ).IsRepeated_PAD( button );      }
inline u32  VWPad_IsRepeatedAccel_PAD( u16 appID, u32 button ) { return VWPad( appID ).IsRepeatedAccel_PAD( button ); }
inline u32  VWPad_IsClicked_PAD      ( u16 appID, u32 button ) { return VWPad( appID ).IsClicked_PAD( button );       }
inline u32  VWPad_IsLongPressed_PAD  ( u16 appID, u32 button ) { return VWPad( appID ).IsLongPressed_PAD( button );   }
inline void VWPad_GetStickXY_PAD     ( u16 appID, s8* stickX, s8* stickY ) { VWPad( appID ).GetStickXY_PAD( stickX, stickY );    }
inline void VWPad_GetSubStickXY_PAD  ( u16 appID, s8* stickX, s8* stickY ) { VWPad( appID ).GetSubStickXY_PAD( stickX, stickY ); }
inline void VWPad_GetKeyInfo         ( u16 appID, PADInfo* keyInfo )       { VWPad( appID ).GetKeyInfo( keyInfo );               }
inline u8   VWPad_GetAnalogL_PAD( u16 appID ) { return VWPad( appID ).GetAnalogL_PAD(); }
inline u8   VWPad_GetAnalogR_PAD( u16 appID ) { return VWPad( appID ).GetAnalogR_PAD(); }

inline u32  VWPad_IsPressed_WPAD      ( u16 appID, u32 button ) { return VWPad( appID ).IsPressed_WPAD( button );       }
inline u32  VWPad_IsPressedHold_WPAD  ( u16 appID, u32 button ) { return VWPad( appID ).IsPressedHold_WPAD( button );   }
inline u32  VWPad_IsTriggered_WPAD    ( u16 appID, u32 button ) { return VWPad( appID ).IsTriggered_WPAD( button );     }
inline u32  VWPad_IsReleased_WPAD     ( u16 appID, u32 button ) { return VWPad( appID ).IsReleased_WPAD( button );      }
inline u32  VWPad_IsRepeated_WPAD     ( u16 appID, u32 button ) { return VWPad( appID ).IsRepeated_WPAD( button );      }
inline u32  VWPad_IsRepeatedAccel_WPAD( u16 appID, u32 button ) { return VWPad( appID ).IsRepeatedAccel_WPAD( button ); }
inline u32  VWPad_IsClicked_WPAD      ( u16 appID, u32 button ) { return VWPad( appID ).IsClicked_WPAD( button );       }
inline u32  VWPad_IsLongPressed_WPAD  ( u16 appID, u32 button ) { return VWPad( appID ).IsLongPressed_WPAD( button );   }
inline void VWPad_GetStickXY_WPAD     ( u16 appID, f32* px, f32* py ) { VWPad( appID ).GetStickXY_WPAD( px, py );       }
inline u32  VWPad_GetDevType_WPAD     ( u16 appID )                   { return VWPad( appID ).GetDevType_WPAD();        }
inline bool VWPad_GetPointXY_WPAD     ( u16 appID, s16 *px, s16 *py ) { return VWPad( appID ).GetPointXY_WPAD( px, py ); }
inline bool VWPad_GetPrevXY_WPAD      ( u16 appID, s16 *px, s16 *py ) { return VWPad( appID ).GetPrevXY_WPAD( px, py );  }
inline bool VWPad_GetMoveXY_WPAD      ( u16 appID, s16 *px, s16 *py ) { return VWPad( appID ).GetMoveXY_WPAD( px, py );  }
//inline bool VWPad_GetPadStatus_WPAD   ( u16 appID, KPADStatus* kpad ) { return VWPad( appID ).GetPadStatus_WPAD( kpad ); }

} /* namespace system */

namespace g3d {

class VWPad : public system::VWPad
{
public:
    /* ctor */ VWPad() : system::VWPad( G3D_ID ) {}
};

inline u32  VWPad_IsPressed      ( u32 button ) { return VWPad_IsPressed( system::G3D_ID, button );       }
inline u32  VWPad_IsPressedHold  ( u32 button ) { return VWPad_IsPressedHold( system::G3D_ID, button );   }
inline u32  VWPad_IsTriggered    ( u32 button ) { return VWPad_IsTriggered( system::G3D_ID, button );     }
inline u32  VWPad_IsReleased     ( u32 button ) { return VWPad_IsReleased( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeated     ( u32 button ) { return VWPad_IsRepeated( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeatedAccel( u32 button ) { return VWPad_IsRepeatedAccel( system::G3D_ID, button ); }
inline u32  VWPad_IsClicked      ( u32 button ) { return VWPad_IsClicked( system::G3D_ID, button );       }
inline u32  VWPad_IsLongPressed  ( u32 button ) { return VWPad_IsLongPressed( system::G3D_ID, button );   }

inline u32  VWPad_IsPressed_PAD      ( u32 button ) { return VWPad_IsPressed_PAD( system::G3D_ID, button );       }
inline u32  VWPad_IsPressedHold_PAD  ( u32 button ) { return VWPad_IsPressedHold_PAD( system::G3D_ID, button );   }
inline u32  VWPad_IsTriggered_PAD    ( u32 button ) { return VWPad_IsTriggered_PAD( system::G3D_ID, button );     }
inline u32  VWPad_IsReleased_PAD     ( u32 button ) { return VWPad_IsReleased_PAD( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeated_PAD     ( u32 button ) { return VWPad_IsRepeated_PAD( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeatedAccel_PAD( u32 button ) { return VWPad_IsRepeatedAccel_PAD( system::G3D_ID, button ); }
inline u32  VWPad_IsClicked_PAD      ( u32 button ) { return VWPad_IsClicked_PAD( system::G3D_ID, button );       }
inline u32  VWPad_IsLongPressed_PAD  ( u32 button ) { return VWPad_IsLongPressed_PAD( system::G3D_ID, button );   }
inline void VWPad_GetStickXY_PAD     ( s8* stickX, s8* stickY ) { VWPad_GetStickXY_PAD( system::G3D_ID, stickX, stickY );    }
inline void VWPad_GetSubStickXY_PAD  ( s8* stickX, s8* stickY ) { VWPad_GetSubStickXY_PAD( system::G3D_ID, stickX, stickY ); }
inline void VWPad_GetKeyInfo         ( system::PADInfo* keyInfo ) { VWPad_GetKeyInfo( system::G3D_ID, keyInfo );               }
inline u8   VWPad_GetAnalogL_PAD( void ) { return VWPad_GetAnalogL_PAD( system::G3D_ID ); }
inline u8   VWPad_GetAnalogR_PAD( void ) { return VWPad_GetAnalogR_PAD( system::G3D_ID ); }

inline u32  VWPad_IsPressed_WPAD      ( u32 button ) { return VWPad_IsPressed_WPAD( system::G3D_ID, button );       }
inline u32  VWPad_IsPressedHold_WPAD  ( u32 button ) { return VWPad_IsPressedHold_WPAD( system::G3D_ID, button );   }
inline u32  VWPad_IsTriggered_WPAD    ( u32 button ) { return VWPad_IsTriggered_WPAD( system::G3D_ID, button );     }
inline u32  VWPad_IsReleased_WPAD     ( u32 button ) { return VWPad_IsReleased_WPAD( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeated_WPAD     ( u32 button ) { return VWPad_IsRepeated_WPAD( system::G3D_ID, button );      }
inline u32  VWPad_IsRepeatedAccel_WPAD( u32 button ) { return VWPad_IsRepeatedAccel_WPAD( system::G3D_ID, button ); }
inline u32  VWPad_IsClicked_WPAD      ( u32 button ) { return VWPad_IsClicked_WPAD( system::G3D_ID, button );       }
inline u32  VWPad_IsLongPressed_WPAD  ( u32 button ) { return VWPad_IsLongPressed_WPAD( system::G3D_ID, button );   }
inline void VWPad_GetStickXY_WPAD     ( f32* px, f32* py ) { VWPad_GetStickXY_WPAD( system::G3D_ID, px, py );       }
inline u32  VWPad_GetDevType_WPAD     ( void )             { return VWPad_GetDevType_WPAD( system::G3D_ID );  }
inline bool VWPad_GetPointXY_WPAD     ( s16 *px, s16 *py ) { return VWPad_GetPointXY_WPAD( system::G3D_ID, px, py ); }
inline bool VWPad_GetPrevXY_WPAD      ( s16 *px, s16 *py ) { return VWPad_GetPrevXY_WPAD( system::G3D_ID, px, py );  }
inline bool VWPad_GetMoveXY_WPAD      ( s16 *px, s16 *py ) { return VWPad_GetMoveXY_WPAD( system::G3D_ID, px, py );  }
//inline bool VWPad_GetPadStatus_WPAD   ( KPADStatus* kpad ) { return VWPad_GetPadStatus_WPAD( system::G3D_ID, kpad ); }

} /* namespace g3d */

namespace g2d {

class VWPad : public system::VWPad
{
public:
    /* ctor */ VWPad() : system::VWPad( G2D_ID ) {}
};

} /* namespace g2d */

namespace effect {

class VWPad : public system::VWPad
{
public:
    /* ctor */ VWPad() : system::VWPad( EFFECT_ID ) {}
};

} /* namespace effect */

namespace sound {

class VWPad : public system::VWPad
{
public:
    /* ctor */ VWPad() : system::VWPad( SOUND_ID ) {}
};

} /* namespace effect */

} /* namespace vw     */



#endif
