/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_IF_COMMON_H_
#define NW4C_VIEWER_IF_COMMON_H_

#include "../../g3d/include/SceneRenderer.h"
#include "../../g3d/include/SceneManager.h"
#include "../../g3d/include/SceneElemSelector.h"

namespace nw     {
namespace viewer {


#define SCN_MGR vw::g3d::SceneManager::GetInstance()
#define ELM_SEL vw::g3d::SceneElemSelector::GetInstance()

#define SCENEOBJECT(p)              static_cast<nw::gfx::SceneObject*>((p)->GetSceneObject())

#define MODELELEM(p)                static_cast<vw::g3d::ModelElem*>(p)
#define MODELELEMITER(p)            vw::g3d::SceneManager::ModelList::GetIteratorFromPointer(MODELELEM(p))
#define MODELNODE(p)                static_cast<nw::gfx::Model*>(SCENEOBJECT(p))

#define CAMERAELEM(p)               static_cast<vw::g3d::CameraElem*>(p)
#define CAMERAELEMITER(p)           vw::g3d::SceneManager::CameraList::GetIteratorFromPointer(CAMERAELEM(p))
#define CAMERANODE(p)               static_cast<nw::gfx::Camera*>(SCENEOBJECT(p))

#define LIGHTELEM(p)                static_cast<vw::g3d::LightElem*>(p)
#define LIGHTELEMITER(p)            vw::g3d::SceneManager::LightList::GetIteratorFromPointer(LIGHTELEM(p))
#define LIGHTNODE(p)                static_cast<nw::gfx::Light*>(SCENEOBJECT(p))

#define FOGELEM(p)                  static_cast<vw::g3d::FogElem*>(p)
#define FOGELELEMITER(p)            vw::g3d::SceneManager::FogList::GetIteratorFromPointer(FOGELEM(p))
#define FOGNODE(p)                  static_cast<nw::gfx::Fog*>(SCENEOBJECT(p))

#define SCENESETTINGELEM(p)         static_cast<vw::g3d::SceneEnvironmentSettingElem*>(p)
#define SCENESETTINGELELEMITER(p)   vw::g3d::SceneManager::SceneEnvironmentSettingList::GetIteratorFromPointer(SCENESETTINGELEM(p))
#define SCENESETTINGNODE(p)         static_cast<nw::gfx::SceneEnvironmentSetting*>(SCENEOBJECT(p))

#define PMODELELEM(p)               static_cast<vw::g3d::ParticleModelElem*>(p)
#define PMODELELEMITER(p)           vw::g3d::SceneManager::ParticleModelList::GetIteratorFromPointer(PMODELELEM(p))
#define PMODELNODE(p)               static_cast<nw::gfx::ParticleModel*>(SCENEOBJECT(p))

#define PEMITTERELEM(p)             static_cast<vw::g3d::ParticleEmitterElem*>(p)
#define PEMITTERELEMITER(p)         vw::g3d::SceneManager::ParticleEmitterList::GetIteratorFromPointer(PEMITTERELEM(p))
#define PEMITTERNODE(p)             static_cast<nw::gfx::ParticleEmitter*>(SCENEOBJECT(p))

/*---------------------------------------------------------------------------*
  Name:         advance_iterator

  Description:  Advances the iterator.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
template <class T> T
advance_iterator(T iter, u32 n)
{
    for (; n>0; --n)
    {
        ++iter;
    }

    return iter;
}

} /* namespace viewer */
} /* namespace nw     */


#endif /* NW4C_VIEWER_IF_COMMON_H_ */

