/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_ANIM_MANAGER_H_
#define NW4R_VIEWER_ANIM_MANAGER_H_

#include <viewer.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw
{
namespace effect
{

class AnimManager
{
public:
    static AnimManager* GetInstance();
    static void Initialize();
    static void Finalize();

    // Ctor
    AnimManager();
    // Dtor
    ~AnimManager();

    void HidControl(vw::system::HidStatus &hidStatus);
    void Synchronize();
    void Reset();
    void UpdateAll();
    void StopAll();
    void PlayAll();
    void PauseAll();
    void SpeedUpAll(f32 diff);
    void StepForwardAll();
    void StepBackwardAll();
    void OnTargetModuleChanged(u16 id);
    void OnAnmCmd(u32 command, void* arg);

    // Get step
    s32 GetStartFrame(){ return m_StartFrame; }
    void SetStartFrame(s32 frame);
    f32 GetStepFrame(){ return m_Step; }
    void SetStepFrame(f32 step);
    f32 GetFrame(){ return m_Frame; }
    f32 GetPreFrame(){ return m_PrevFrame; }
    void SetMaxFrameCount( f32 maxFrame ){
        m_MaxFrameCount = maxFrame;
        if (m_MaxFrameCount == 0)
        {
            m_LoopFlag = false;
        }
        else
        {
            m_LoopFlag = true;
        }
    }
    f32  GetMaxFrameCount(){ return m_MaxFrameCount; }
    void SetLoopFlag( bool flag ){ m_LoopFlag = flag; }
    bool GetLoopFlag(){ return m_LoopFlag; }
    bool ActiveAnimationExists(){ return m_IsActive; }
    void SetDefaultRate(f32 rate){ m_DefaultRate = rate; m_Step = rate;}

private:
    // Frame count
    f32 m_Frame;
    // Previous frame
    f32 m_PrevFrame;
    // Playback rate
    f32 m_Step;
    //
    s32 m_StartFrame;
    // Is the animation enabled
    bool m_IsActive;
    // MaxFrameCount
    f32     m_MaxFrameCount;
    // LoopFlag
    bool    m_LoopFlag;
    // Default play rate.
    f32     m_DefaultRate;
    // Static instance
    static AnimManager* m_pInstance;
    void UpdateFrame();
};


} // namespace effect
} // namespace vw
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif /* NW4R_VIEWER_EFFECT_BYTESTREAM_H_ */
