/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_EFFECT_PARAMSETTER_H_
#define NW4C_VIEWER_EFFECT_PARAMSETTER_H_

#include <viewer/effect/ProtocolEM4C.h>
#include <nw/viewer/sys/Application.h>
#include "../include/AnimManager.h"
#include "../../g3d/include/ResourceManager.h"
#include "../../g3d/include/SceneManager.h"
#include "../../g3d/include/AnimManager.h"
#include "../../system/include/ConvertCharEncoding.h"
#include "../../../include/viewer/g3d/CameraMgr.h"
#include "../../system/include/LogUtil.h"

namespace vw {
namespace effect {

#define FOREACH_SELECTED_EM( select ) \
    for (nw::ut::MoveArray<ProtocolEM4C::EditSelectedObject>::iterator select = m_SelectedObject.begin(), end_it = m_SelectedObject.end();\
        select != end_it; ++select)

#define GET_RES_PARTICLE_EMITTER( select, editObject ) \
    nw::gfx::res::ResParticleEmitter editObject; \
    if ( m_SelectedType != ProtocolEM4C::EditTargetType_Emitter ) return; \
    if ( !GetResParticleEmitter( select->Id, select->Index, &editObject ) ) return;

#define GET_RES_PARTICLE_FORM( select, type, editObject ) \
    GET_RES_PARTICLE_EMITTER( select, res_emitter ) \
    nw::gfx::res::ResParticleForm res_particle_form = res_emitter.GetParticleForm(); \
    nw::gfx::res::type editObject; \
    if ( !CheckResourceType( res_particle_form, editObject ) ) return;

#define GET_RES_PARTICLE_SET( select, editObject ) \
    nw::gfx::res::ResParticleSet editObject; \
    if ( m_SelectedType != ProtocolEM4C::EditTargetType_ParticleSet ) return; \
    if ( !GetResParticleSet( select->Id, select->Index, &editObject ) ) return;

#define GET_RES_PARTICLE_INITIALIZER( select, index, type, usage, editObject ) \
    nw::gfx::res::ResParticleInitializer res_particle_initializer; \
    GET_RES_PARTICLE_SET( select, res_particle_set ) \
    nw::gfx::res::ResTypeInfoDefinition typeDef = nw::gfx::res::type##_TYPE_INFO; \
    if ( !GetResParticleInitializer( res_particle_set, index, typeDef, (nw::gfx::res::ParticleUsage)usage, &res_particle_initializer ) ) return; \
    nw::gfx::res::type editObject; \
    if ( !CheckResourceTypeAndUsage( res_particle_initializer, (nw::gfx::res::ParticleUsage)usage, editObject ) ) return;

#define GET_RES_PARTICLE_UPDATER( select, index, type, usage, editObject ) \
    nw::gfx::res::ResParticleUpdater res_particle_updater; \
    GET_RES_PARTICLE_SET( select, res_particle_set ) \
    nw::gfx::res::ResTypeInfoDefinition typeDef = nw::gfx::res::type##_TYPE_INFO; \
    if ( !GetResParticleUpdater( res_particle_set, index, typeDef, (nw::gfx::res::ParticleUsage)usage, &res_particle_updater ) ) return; \
    nw::gfx::res::type editObject; \
    if ( !CheckResourceTypeAndUsage( res_particle_updater, (nw::gfx::res::ParticleUsage)usage, editObject ) ) return;

#define GET_RES_PARTICLE_SHAPE_BUILDER( select, editObject ) \
    GET_RES_PARTICLE_SET( select, res_particle_set ) \
    nw::gfx::res::ResParticleShapeBuilder editObject = res_particle_set.GetParticleShapeBuilder();

#define GET_RES_MATERIAL_COLOR( editObject ) \
    nw::gfx::res::ResMaterialColor editObject = res_material.GetMaterialColor();

#define GET_RES_TEXTURE_COORDINATOR( index, editObject ) \
    if ( index >= res_material.GetTextureCoordinatorsCount() ) return; \
    nw::gfx::res::ResTextureCoordinator editObject = res_material.GetTextureCoordinators( index );

#define GET_RES_TEXTURE_SAMPLER( index, editObject ) \
    if ( index >= res_material.GetTextureMappersCount() ) return; \
    nw::gfx::res::ResTextureSampler editObject = res_material.GetTextureMappers( index ).GetSampler();

#define GET_RES_STANDARD_TEXTURE_SAMPLER( index, editObject ) \
    GET_RES_TEXTURE_SAMPLER( index, res_sampler ) \
    nw::gfx::res::ResStandardTextureSampler editObject = nw::ut::ResDynamicCast<nw::gfx::res::ResStandardTextureSampler>(res_sampler); \
    if ( !editObject.IsValid() ) return;

#define GET_RES_TEXTURE_COMBINER( editObject ) \
    nw::gfx::res::ResFragmentShader res_fragment = res_material.GetFragmentShader(); \
    if ( 0 == res_fragment.GetTextureCombinersCount() ) return; \
    nw::gfx::res::ResTextureCombiner editObject = res_fragment.GetTextureCombiners( 0 );

#define GET_PARTICLE_MODEL( select, editObject ) \
    nw::gfx::ParticleModel* editObject; \
    if ( m_SelectedType != ProtocolEM4C::EditTargetType_ParticleModel ) return; \
    nw::gfx::res::ResParticleModel res_particle_model; \
    if ( !GetResParticleModel( select->Id, &res_particle_model ) ) return; \
    if ( !GetParticleModel( res_particle_model, &editObject ) ) return;

#define GET_PARTICLE_EMITTER( select, editObject ) \
    nw::gfx::ParticleEmitter* editObject; \
    GET_RES_PARTICLE_EMITTER( select, res_emitter ) \
    if ( !GetParticleEmitter( res_emitter, &editObject ) ) return;

#define GET_PARTICLE_SET( select, editObject ) \
    nw::gfx::ParticleSet* editObject = NULL; \
    GET_RES_PARTICLE_SET( select, res_set ) \
    nw::gfx::ParticleModel* pModel; \
    if ( !GetParticleSet( res_set, &pModel, &editObject ) ) return;

#define GET_MATERIAL( select, editObject ) \
    nw::gfx::Material* editObject = NULL; \
    if ( m_SelectedType != ProtocolEM4C::EditTargetType_Material ) return; \
    nw::gfx::ParticleModel* pModel; \
    nw::gfx::res::ResMaterial res_mat; \
    if ( !GetResMaterial( select->Id, select->Index, &res_mat ) ) return; \
    if ( !GetMaterial( res_mat, &pModel, &editObject ) ) return;

#define COPY_FLAG_INITIALIZER( src, dst ) \
    dst.SetIsResourceCopyEnabled( ( src->Flag & ProtocolEM4C::FlagBitType_IsResourceCopyEnabled ) ? true : false ); \
    dst.SetInitializerEnabled( ( src->Flag & ProtocolEM4C::FlagBitType_IsEnabled ) ? true : false );

#define COPY_FLAG_UPDATER( src, dst ) \
    dst.SetIsResourceCopyEnabled( ( src->Flag & ProtocolEM4C::FlagBitType_IsResourceCopyEnabled ) ? true : false ); \
    dst.SetUpdaterEnabled( ( src->Flag & ProtocolEM4C::FlagBitType_IsEnabled ) ? true : false );

#define SET_FLAGS( obj, flag, bitFlag ) \
    u32 nFlags = obj.GetFlags(); \
    if ( flag ) { nFlags |= bitFlag; } \
    else { nFlags &= ~(bitFlag); } \
    obj.SetFlags( nFlags );

#define COPY_VEC1( src, dst, param ) dst.Set##param( src->param );
#define COPY_VEC2( src, dst, param ) dst.Set##param( src->param.X, src->param.Y );
#define COPY_VEC3( src, dst, param ) dst.Set##param( src->param.X, src->param.Y, src->param.Z );
#define COPY_RGB( src, dst, param ) dst.Set##param( src->param.R, src->param.G, src->param.B );
#define COPY_RGBA( src, dst, param ) dst.Set##param( src->param.R, src->param.G, src->param.B, src->param.A );

class ParamSetter
{
public:
    static ParamSetter& GetInstance();
    static void Initialize();
    static void Finalize();

    void
    SetSelectObject(
        ProtocolEM4C::EditTargetType selectedType,
        nw::ut::MoveArray<ProtocolEM4C::EditSelectedObject>& selectedObject)
    {
        m_SelectedType = selectedType;
        m_SelectedObject = selectedObject;
    }

    void
    ClearSelectObject( void )
    {
        m_SelectedType = ProtocolEM4C::EditTargetType_Node;
        m_SelectedObject.clear();
    }

    bool
    GetResParticleModel( const unsigned int hashID, nw::gfx::res::ResParticleModel* particle_model )
    {
        // Get resource set from object ID
        vw::g3d::ResourceManager::ResourceList::iterator it = m_ResMgr->Find( vw::g3d::ResourceSet::Type_Particle, hashID );
        if (it == m_ResMgr->GetResourcesEnd()) return false;

        // Get resource body from resource instance
        nw::gfx::res::ResGraphicsFile resource( it->GetResource() );
        if (!resource.IsValid()){
            NW_VIEWER_LOG("GetResParticleModel | null object\n");
            return false;
        }

        // Confirmation of number of models (one ParticleModel in the file)
        if (resource.GetModelsCount() != 1){
            NW_VIEWER_LOG("GetResParticleModel | #particle model must be 1\n");
            return false;
        }

        // Gets the model.
        *particle_model = nw::ut::ResDynamicCast<nw::gfx::res::ResParticleModel>( resource.GetModels()[ 0 ] );

        // Check model type
        if ( particle_model->ptr() == NULL ){
            NW_VIEWER_LOG("GetResParticleModel | model type must be [ResParticleModel_TYPE_INFO]\n");
            return false;
        }

        return true;
    }

    int
    GetResParticleEmitterIndex( const unsigned int hashID, const wchar_t* name )
    {
        // Get resource set from object ID
        vw::g3d::ResourceManager::ResourceList::iterator it = m_ResMgr->Find( vw::g3d::ResourceSet::Type_Particle, hashID );
        if (it == m_ResMgr->GetResourcesEnd()) return false;

        // Get resource body from resource instance
        nw::gfx::res::ResGraphicsFile resource( it->GetResource() );
        if (!resource.IsValid()){
            NW_VIEWER_LOG("GetResParticleEmitterIndex | null object\n");
            return -1;
        }

        // Converts character string to char.
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(name, utf8Str, MAX_PATH_SIZE);

        // Go around the array while comparing the name
        for (int i = 0; i < resource.GetEmittersCount(); ++i)
        {
            if ( std::strcmp(resource.GetEmitters()[ i ].GetName(), utf8Str) == 0 ) {
                return i;
            }
        }

        // Error messages
        NW_VIEWER_LOG("GetResParticleEmitterIndex | cannot find [%s]\n", utf8Str);

        return -1;
    }

    bool
    GetResParticleEmitter(
        const unsigned int hashID,
        const int index,
        nw::gfx::res::ResParticleEmitter* emitter
    ) {
        // Get resource set from object ID
        vw::g3d::ResourceManager::ResourceList::iterator it = m_ResMgr->Find( vw::g3d::ResourceSet::Type_Particle, hashID );
        if (it == m_ResMgr->GetResourcesEnd()) return false;

        // Get resource body from resource instance
        nw::gfx::res::ResGraphicsFile resource( it->GetResource() );
        if (!resource.IsValid()){
            NW_VIEWER_LOG("GetResParticleEmitter | null object\n");
            return false;
        }

        // Check if index is in the array
        if (index < 0 || index > resource.GetEmittersCount() - 1) {
            NW_VIEWER_LOG("GetResParticleEmitter | index is out of array\n");
            return false;
        }

        // Copy value
        *emitter = resource.GetEmitters()[ index ];

        return true;
    }

    int
    GetResParticleSetIndex( const unsigned int hashID, const wchar_t* name )
    {
        // Get ResParticleModel
        nw::gfx::res::ResParticleModel model;
        if (!GetResParticleModel(hashID, &model)) {
            NW_VIEWER_LOG("GetResParticleSetIndex | null object\n");
            return -1;
        }

        // Converts character string to char.
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(name, utf8Str, MAX_PATH_SIZE);

        // Go around the array while comparing the name
        for (int i = 0; i < model.GetParticleSetsCount(); ++i)
        {
            if ( std::strcmp(model.GetParticleSets()[ i ].GetName(), utf8Str) == 0 ) {
                return i;
            }
        }

        // Error messages
        NW_VIEWER_LOG("GetResParticleSetIndex | cannot find [%s]\n", utf8Str);

        return -1;
    }

    bool
    GetResParticleSet(
        const unsigned int hashID,
        const int index,
        nw::gfx::res::ResParticleSet* particle_set
    ) {
        // Get ResParticleModel
        nw::gfx::res::ResParticleModel model;
        if (!GetResParticleModel(hashID, &model)) {
            NW_VIEWER_LOG("GetResParticleSet | null object\n");
            return false;
        }

        // Check if index is in the array
        if (index < 0 || index > model.GetParticleSetsCount() - 1) {
            NW_VIEWER_LOG("GetResParticleSet | index is out of array\n");
            return false;
        }

        // Copy value
        *particle_set = model.GetParticleSets()[ index ];

        return true;
    }

    bool
    GetResParticleInitializer(
        const nw::gfx::res::ResParticleSet particle_set,
        const int index,
        const nw::gfx::res::ResTypeInfoDefinition type,
        const nw::gfx::res::ParticleUsage usage,
        nw::gfx::res::ResParticleInitializer* initializer
    ) {
        // Same type, number of times Usage is registered
        int checknum = 0;

        // Go around resource array
        for ( int i = 0; i < particle_set.GetParticleInitializersCount(); ++ i )
        {
            // Get value
            const nw::gfx::res::ResParticleInitializer init = particle_set.GetParticleInitializers()[ i ];

            // Check Usage
            if (init.GetTargetStream() != usage) continue;

            // Check class type
            if (init.GetTypeInfo() != type) continue;

            // Check index
            if (checknum == index) {
                // Copy value
                *initializer = init;
                return true;
            } else {
                ++checknum;
            }
        }

        return false;
    }

    bool
    GetResParticleUpdater(
        const nw::gfx::res::ResParticleSet particle_set,
        const int index,
        const nw::gfx::res::ResTypeInfoDefinition type,
        const nw::gfx::res::ParticleUsage usage,
        nw::gfx::res::ResParticleUpdater* updater
    ) {
        // Same type, number of times Usage is registered
        int checknum = 0;

        // Go around resource array
        for ( int i = 0; i < particle_set.GetParticleUpdatersCount(); ++ i )
        {
            // Get value
            const nw::gfx::res::ResParticleUpdater update = particle_set.GetParticleUpdaters()[ i ];

            // Check Usage
            if (update.GetTargetStream() != usage) continue;

            // Check class type
            if (update.GetTypeInfo() != type) continue;

            // Check index
            if (checknum == index) {
                // Copy value
                *updater = update;
                return true;
            } else {
                ++checknum;
            }
        }

        return false;
    }

    int
    GetResMaterialIndex( const unsigned int hashID, const wchar_t* name )
    {
        // Get ResParticleModel
        nw::gfx::res::ResParticleModel model;
        if (!GetResParticleModel(hashID, &model)) {
            NW_VIEWER_LOG("GetResMaterialIndex | null object\n");
            return -1;
        }

        // Converts character string to char.
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(name, utf8Str, MAX_PATH_SIZE);

        // Search for index in names
        int index = model.GetMaterialsIndex( utf8Str );

        // Error messages
        if ( index == -1 ) {
            NW_VIEWER_LOG("GetResMaterialIndex | cannot find [%s]\n", utf8Str);
        }

        return index;
    }

    bool
    GetResMaterial(
        const unsigned int hashID,
        const int index,
        nw::gfx::res::ResMaterial* material
    ) {
        // Get ResParticleModel
        nw::gfx::res::ResParticleModel model;
        if (!GetResParticleModel(hashID, &model)) {
            NW_VIEWER_LOG("GetResMaterial | null object\n");
            return false;
        }

        // Check if index is in the array
        if (index < 0 || index > model.GetMaterialsCount() - 1) {
            NW_VIEWER_LOG("GetResMaterial | index is out of array\n");
            return false;
        }

        // Copy value
        *material = model.GetMaterials( index );

        return true;
    }

    bool
    GetParticleModel(
        const nw::gfx::res::ResParticleModel res_particle_model,
        nw::gfx::ParticleModel** partilce_model
    ) {
        // Gets the scene context.
        nw::gfx::SceneContext* content = m_ScnMgr->GetSceneContext();

        // Get Emitter node array
        nw::gfx::ParticleModelArray::iterator ite = content->GetParticleModelBegin();
        nw::gfx::ParticleModelArray::iterator end = content->GetParticleModelEnd();

        // Go around node
        for ( ; ite != end ; ++ite )
        {
            nw::gfx::ParticleModel* pm = *ite;
            if (!pm) continue;

            // Compare actual pointer and check if it is references same resource
            if (pm->GetResModel().ptr() == res_particle_model.ptr()){
                *partilce_model = pm;
                return true;
            }
        }

        return false;
    }

    bool
    GetParticleEmitter(
        const nw::gfx::res::ResParticleEmitter res_emitter,
        nw::gfx::ParticleEmitter** emitter
    ) {
        // Gets the scene context.
        nw::gfx::SceneContext* content = m_ScnMgr->GetSceneContext();

        // Get Emitter node array
        nw::gfx::ParticleEmitterArray::iterator ite = content->GetParticleEmitterBegin();
        nw::gfx::ParticleEmitterArray::iterator end = content->GetParticleEmitterEnd();

        // Go around node
        for ( ; ite != end ; ++ite )
        {
            nw::gfx::ParticleEmitter* em = *ite;
            if (!em) continue;

            // Compare actual pointer and check if it is references same resource
            if (em->GetResParticleEmitter().ptr() == res_emitter.ptr()){
                *emitter = em;
                return true;
            }
        }

        return false;
    }

    bool
    GetParticleSet(
        const nw::gfx::res::ResParticleSet res_set,
        nw::gfx::ParticleModel** model,
        nw::gfx::ParticleSet** set
    ) {
        // Gets the scene context.
        nw::gfx::SceneContext* content = m_ScnMgr->GetSceneContext();

        // Get ParticleModel node array
        nw::gfx::ParticleModelArray::iterator ite = content->GetParticleModelBegin();
        nw::gfx::ParticleModelArray::iterator end = content->GetParticleModelEnd();

        // Go around node
        for ( ; ite != end ; ++ite )
        {
            nw::gfx::ParticleModel* pm = *ite;
            if (!pm) continue;

            // Go around material
            for ( int i = 0; i < pm->GetParticleSetsCount(); ++i )
            {
                // Compare actual pointer and check if it is references same resource
                if (pm->GetParticleSets( i )->GetResParticleSet().ptr() == res_set.ptr()) {
                    *model = pm;
                    *set = pm->GetParticleSets( i );
                    return true;
                }
            }
        }

        return false;
    }

    bool
    GetMaterial(
        const nw::gfx::res::ResMaterial res_material,
        nw::gfx::ParticleModel** model,
        nw::gfx::Material** material
    ) {
        // Gets the scene context.
        nw::gfx::SceneContext* content = m_ScnMgr->GetSceneContext();

        // Get ParticleModel node array
        nw::gfx::ParticleModelArray::iterator ite = content->GetParticleModelBegin();
        nw::gfx::ParticleModelArray::iterator end = content->GetParticleModelEnd();

        // Go around node
        for ( ; ite != end ; ++ite )
        {
            nw::gfx::ParticleModel* pm = *ite;
            if (!pm) continue;

            // Go around material
            for ( int i = 0; i < pm->GetMaterialCount(); ++i )
            {
                // Compare actual pointer and check if it is references same resource
                if (pm->GetMaterial( i )->GetOriginal().ptr() == res_material.ptr()) {
                    *model = pm;
                    *material = pm->GetMaterial( i );
                    return true;
                }
            }
        }

        return false;
    }

    template<class T, class U>
    inline bool CheckResourceType(
        const T& res_partice_resource,
        U& res_edit_object
    ) {
        // Check if there is any data
        res_edit_object = nw::ut::ResDynamicCast<U>( res_partice_resource );
        if ( res_edit_object.ptr() == NULL ) {
            NW_VIEWER_LOG( "resource type [%x] != request type [%x]\n", res_partice_resource.GetTypeInfo(), U::TYPE_INFO );
            return false;
        }

        return true;
    }

    template<class T, class U>
    inline bool CheckResourceTypeAndUsage(
        const T& res_partice_resource,
        const nw::gfx::res::ParticleUsage usage,
        U& res_edit_object
    ) {
        // Check if there is any data
        res_edit_object = nw::ut::ResDynamicCast<U>( res_partice_resource );
        if ( res_edit_object.ptr() == NULL ) {
            NW_VIEWER_LOG( "resource type [%x] != request type [%x]\n", res_partice_resource.GetTypeInfo(), U::TYPE_INFO );
            return false;
        }

        // Check Usage type
        nw::gfx::res::ParticleUsage res_usage = res_edit_object.GetTargetStream();
        if ( res_usage != usage ) {
            NW_VIEWER_LOG( "resouce usage [%d] != request usage [%d]\n", res_usage, usage );
            return false;
        }

        return true;
    }

    /*
        Particle model
    */
    void
    SetParticleModelTransform(ProtocolEM4C::MESSAGEPACKET_ParticleModel_SetTransform* transform)
    {
        // Transform that should be set
        nw::math::Transform3 transForm(
            nw::math::VEC3( transform->Scale.X, transform->Scale.Y, transform->Scale.Z ),
            nw::math::VEC3( transform->Rotate.X, transform->Rotate.Y, transform->Rotate.Z ),
            nw::math::VEC3( transform->Translate.X, transform->Translate.Y, transform->Translate.Z )
        );

        FOREACH_SELECTED_EM( selected )
        {
            GET_PARTICLE_MODEL( selected, model )
            model->Transform().SetTransform( transForm );
        }
    }

    /*
        Emitter
    */
    void
    SetEmitterTransform(ProtocolEM4C::MESSAGEPACKET_Emitter_SetTransform* transform)
    {
        // Transform that should be set
        nw::math::Transform3 transForm(
            nw::math::VEC3( transform->Scale.X, transform->Scale.Y, transform->Scale.Z ),
            nw::math::VEC3( transform->Rotate.X, transform->Rotate.Y, transform->Rotate.Z ),
            nw::math::VEC3( transform->Translate.X, transform->Translate.Y, transform->Translate.Z )
        );

        FOREACH_SELECTED_EM( selected )
        {
            GET_PARTICLE_EMITTER( selected, emitter )
            emitter->Transform().SetTransform( transForm );
        }
    }

    void
    SetEmitterEmission(ProtocolEM4C::MESSAGEPACKET_Emitter_SetEmission* emission)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_EMITTER( selected, emitter )
            COPY_VEC1( emission, emitter, EmissionRatio )
            COPY_VEC1( emission, emitter, EmissionRatioRandom )
            COPY_VEC1( emission, emitter, EmissionInterval )
            COPY_VEC1( emission, emitter, EmissionIntervalRandom )
            COPY_VEC1( emission, emitter, EmissionSpanInfinity )
            COPY_VEC1( emission, emitter, EmissionSpan )
            COPY_VEC1( emission, emitter, EmissionStart )
        }
    }

    /*
        Emitter shape
    */
    void
    SetCylinderForm(ProtocolEM4C::MESSAGEPACKET_ParticleForm_CylinderForm* form)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_FORM( selected, ResParticleCylinderForm, res_form )
            COPY_VEC3( form, res_form, Scale )
            COPY_VEC1( form, res_form, Inner )
            COPY_VEC1( form, res_form, AngleOffset )
            COPY_VEC1( form, res_form, AngleWidth )
            COPY_VEC1( form, res_form, AngleSwing )
            COPY_VEC1( form, res_form, FixedOffset )
            COPY_VEC1( form, res_form, EvenInterval )
        }
    }

    void
    SetSphereForm(ProtocolEM4C::MESSAGEPACKET_ParticleForm_SphereForm* form)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_FORM( selected, ResParticleSphereForm, res_form )
            COPY_VEC3( form, res_form, Scale )
            COPY_VEC1( form, res_form, Inner )
            COPY_VEC1( form, res_form, AngleOffset )
            COPY_VEC1( form, res_form, AngleWidth )
            COPY_VEC1( form, res_form, AngleSwing )
            COPY_VEC1( form, res_form, FixedOffset )
            COPY_VEC1( form, res_form, EvenInterval )
        }
    }

    void
    SetDiscForm(ProtocolEM4C::MESSAGEPACKET_ParticleForm_DiscForm* form)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_FORM( selected, ResParticleDiscForm, res_form )
            COPY_VEC2( form, res_form, Scale )
            COPY_VEC1( form, res_form, Inner )
            COPY_VEC1( form, res_form, AngleOffset )
            COPY_VEC1( form, res_form, AngleWidth )
            COPY_VEC1( form, res_form, AngleSwing )
            COPY_VEC1( form, res_form, FixedOffset )
            COPY_VEC1( form, res_form, EvenInterval )
        }
    }

    void
    SetRectangleForm(ProtocolEM4C::MESSAGEPACKET_ParticleForm_RectangleForm* form)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_FORM( selected, ResParticleRectangleForm, res_form )
            COPY_VEC2( form, res_form, Scale )
            COPY_VEC1( form, res_form, Inner )
            COPY_VEC1( form, res_form, EvenInterval )
        }
    }

    void
    SetCubeForm(ProtocolEM4C::MESSAGEPACKET_ParticleForm_CubeForm* form)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_FORM( selected, ResParticleCubeForm, res_form )
            COPY_VEC3( form, res_form, Scale )
            COPY_VEC1( form, res_form, Inner )
            COPY_VEC1( form, res_form, EvenInterval )
        }
    }

    /*
        Particle initializer
    */
    void
    SetYAxisVelocityInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_YAxisVelocityInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleYAxisVelocityInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Power )
        }
    }

    void
    SetOriginVelocityInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_OriginVelocityInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleOriginVelocityInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Power )
        }
    }

    void
    SetRandomVelocityInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_RandomVelocityInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleRandomVelocityInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Power )
        }
    }

    void
    SetDirectionalVelocityInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_DirectionalVelocityInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleDirectionalVelocityInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Power )
            COPY_VEC3( initializer, res_initializer, Direction )
        }
    }

    void
    SetRandomDirectionalVelocityInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_RandomDirectionalVelocityInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleRandomDirectionalVelocityInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Power )
            COPY_VEC1( initializer, res_initializer, Angle )
            COPY_VEC3( initializer, res_initializer, Direction )
        }
    }

    void
    SetVector3MultRandomInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_Vector3MultRandomInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleVector3MultRandomInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, Random )
        }
    }

    void
    SetFloatRangeRandomInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_FloatRangeRandomInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleFloatRangeRandomInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, MaxValue )
            COPY_VEC1( initializer, res_initializer, MinValue )
        }
    }

    void
    SetFloatImmediateInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_FloatImmediateInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleFloatImmediateInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, ImmediateValue )
        }
    }

    void
    SetFloatRandomInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_FloatRandomInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleFloatRandomInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC1( initializer, res_initializer, BaseValue )
            COPY_VEC1( initializer, res_initializer, Random )
        }
    }

    void
    SetVector3Random1Initializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_Vector3Random1Initializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleVector3Random1Initializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC3( initializer, res_initializer, BaseValue )
            COPY_VEC1( initializer, res_initializer, Random )
        }
    }

    void
    SetVector3Random3Initializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_Vector3Random3Initializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleVector3Random3Initializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC3( initializer, res_initializer, BaseValue )
            COPY_VEC3( initializer, res_initializer, Random )
        }
    }

    void
    SetVector3ImmediateInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_Vector3ImmediateInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleVector3ImmediateInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC3( initializer, res_initializer, ImmediateValue )
        }
    }

    void
    SetVector2ImmediateInitializer(ProtocolEM4C::MESSAGEPACKET_ParticleInitializer_Vector2ImmediateInitializer* initializer)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_INITIALIZER( selected, initializer->Index, ResParticleVector2ImmediateInitializer, initializer->Usage, res_initializer )
            COPY_FLAG_INITIALIZER( initializer, res_initializer )
            COPY_VEC2( initializer, res_initializer, ImmediateValue )
        }
    }

    /*
        Particle updater
    */
    void
    SetGravityUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_GravityUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleGravityUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Power )
            COPY_VEC3( updater, res_updater, Direction )
        }
    }

    void
    SetSpinUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_SpinUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleSpinUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Power )
            COPY_VEC3( updater, res_updater, Axis )
        }
    }

    void
    SetRandomUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_RandomUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleRandomUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Interval )
            COPY_VEC3( updater, res_updater, Power )
        }
    }

    void
    SetAccelarationUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_AccelarationUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleAccelarationUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Factor )
        }
    }

    void
    SetRotateUpVectorUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_RotateUpVectorUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleRotateUpVectorUpdater, updater->Usage, res_updater )
            res_updater.SetUpdaterEnabled( updater->IsUpdaterEnabled & ProtocolEM4C::FlagBitType_IsEnabled );
            res_updater.SetSource( (nw::gfx::res::ResParticleRotateUpVectorUpdater::ParticleRotateUpVectorSource)updater->Source );
        }
    }

    void
    SetFloatImmediateUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_FloatImmediateUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleFloatImmediateUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, DefaultValue )
            //COPY_VEC1( updater, res_updater, LoopDuration )
            //COPY_VEC1( updater, res_updater, RandomOffset )
        }
    }

    void
    SetVector3AdditiveUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_Vector3AdditiveUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleVector3AdditiveUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC3( updater, res_updater, DefaultValue )
            //COPY_VEC1( updater, res_updater, LoopDuration )
            //COPY_VEC1( updater, res_updater, RandomOffset )
        }
    }

    void
    SetVector3ImmediateUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_Vector3ImmediateUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleVector3ImmediateUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC3( updater, res_updater, DefaultValue )
            //COPY_VEC1( updater, res_updater, LoopDuration )
            //COPY_VEC1( updater, res_updater, RandomOffset )
        }
    }

    void
    SetVector2ImmediateUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_Vector2ImmediateUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleVector2ImmediateUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC2( updater, res_updater, DefaultValue )
            //COPY_VEC1( updater, res_updater, LoopDuration )
            //COPY_VEC1( updater, res_updater, RandomOffset )
        }
    }

    void
    SetTexturePatternUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_TexturePatternUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleTexturePatternUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            res_updater.SetDivisionX( (nw::gfx::res::ParticleTexturePatternUpdaterDivisionType)updater->DivisionX );
            res_updater.SetDivisionY( (nw::gfx::res::ParticleTexturePatternUpdaterDivisionType)updater->DivisionY );
            //COPY_VEC1( updater, res_updater, TextureIndex )
            //COPY_VEC1( updater, res_updater, LoopDuration )
            //COPY_VEC1( updater, res_updater, RandomOffset )
        }
    }

    void
    SetChildUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_ChildUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleChildUpdater, -1, res_updater )
            //COPY_FLAG_UPDATER( updater, res_updater )
            nw::gfx::res::ResParticleForm form = res_updater.GetParticleForm();
            if ( form.GetTypeInfo() == nw::gfx::res::ResParticleCylinderForm_TYPE_INFO ) {
                nw::gfx::res::ResParticleCylinderForm res_form;
                CheckResourceType( form, res_form );
                COPY_VEC3( updater, res_form, Scale )
                COPY_VEC1( updater, res_form, Inner )
                COPY_VEC1( updater, res_form, AngleOffset )
                COPY_VEC1( updater, res_form, AngleWidth )
                COPY_VEC1( updater, res_form, AngleSwing )
                COPY_VEC1( updater, res_form, FixedOffset )
                COPY_VEC1( updater, res_form, EvenInterval )
            } else if ( form.GetTypeInfo() == nw::gfx::res::ResParticleSphereForm_TYPE_INFO ) {
                nw::gfx::res::ResParticleSphereForm res_form;
                CheckResourceType( form, res_form );
                COPY_VEC3( updater, res_form, Scale )
                COPY_VEC1( updater, res_form, Inner )
                COPY_VEC1( updater, res_form, AngleOffset )
                COPY_VEC1( updater, res_form, AngleWidth )
                COPY_VEC1( updater, res_form, AngleSwing )
                COPY_VEC1( updater, res_form, FixedOffset )
                COPY_VEC1( updater, res_form, EvenInterval )
            } else if ( form.GetTypeInfo() == nw::gfx::res::ResParticleDiscForm_TYPE_INFO ) {
                nw::gfx::res::ResParticleDiscForm res_form;
                CheckResourceType( form, res_form );
                COPY_VEC2( updater, res_form, Scale )
                COPY_VEC1( updater, res_form, Inner )
                COPY_VEC1( updater, res_form, AngleOffset )
                COPY_VEC1( updater, res_form, AngleWidth )
                COPY_VEC1( updater, res_form, AngleSwing )
                COPY_VEC1( updater, res_form, FixedOffset )
                COPY_VEC1( updater, res_form, EvenInterval )
            } else if ( form.GetTypeInfo() == nw::gfx::res::ResParticleRectangleForm_TYPE_INFO ) {
                nw::gfx::res::ResParticleRectangleForm res_form;
                CheckResourceType( form, res_form );
                COPY_VEC2( updater, res_form, Scale )
                COPY_VEC1( updater, res_form, Inner )
                COPY_VEC1( updater, res_form, EvenInterval )
            } else if ( form.GetTypeInfo() == nw::gfx::res::ResParticleCubeForm_TYPE_INFO ) {
                nw::gfx::res::ResParticleCubeForm res_form;
                CheckResourceType( form, res_form );
                COPY_VEC3( updater, res_form, Scale )
                COPY_VEC1( updater, res_form, Inner )
                COPY_VEC1( updater, res_form, EvenInterval )
            }
        }
    }

    void
    SetFloatImmediate4KeyUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_FloatImmediate4KeyUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleFloatImmediate4KeyUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, InTime )
            COPY_VEC1( updater, res_updater, OutTime )
            COPY_VEC1( updater, res_updater, Value0 )
            COPY_VEC1( updater, res_updater, Value1 )
            COPY_VEC1( updater, res_updater, Value2 )
            COPY_VEC1( updater, res_updater, Value3 )
            COPY_VEC1( updater, res_updater, Value4 )
        }
    }

    void
    SetVector3Immediate4KeyUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_Vector3Immediate4KeyUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleVector3Immediate4KeyUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, InTime )
            COPY_VEC1( updater, res_updater, OutTime )
            COPY_VEC3( updater, res_updater, Value0 )
            COPY_VEC3( updater, res_updater, Value1 )
            COPY_VEC3( updater, res_updater, Value2 )
            COPY_VEC3( updater, res_updater, Value3 )
            COPY_VEC3( updater, res_updater, Value4 )
        }
    }

    void
    SetMagnetUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_MagnetUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleMagnetUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Power )
            COPY_VEC3( updater, res_updater, Position )

            u32 flag = res_updater.GetFlags();
            flag &= ~(
                nw::gfx::res::ResParticleMagnetUpdater::MAGNET_X_ENABLED |
                nw::gfx::res::ResParticleMagnetUpdater::MAGNET_Y_ENABLED |
                nw::gfx::res::ResParticleMagnetUpdater::MAGNET_Z_ENABLED
            );
            if ( updater->AxisType & ProtocolEM4C::MagnetAxisType_MagnetXEnable ) flag |= nw::gfx::res::ResParticleMagnetUpdater::MAGNET_X_ENABLED;
            if ( updater->AxisType & ProtocolEM4C::MagnetAxisType_MagnetYEnable ) flag |= nw::gfx::res::ResParticleMagnetUpdater::MAGNET_Y_ENABLED;
            if ( updater->AxisType & ProtocolEM4C::MagnetAxisType_MagnetZEnable ) flag |= nw::gfx::res::ResParticleMagnetUpdater::MAGNET_Z_ENABLED;
            res_updater.SetFlags( flag );
        }
    }

    void
    SetCollisionUpdater(ProtocolEM4C::MESSAGEPACKET_ParticleUpdater_CollisionUpdater* updater)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_UPDATER( selected, updater->Index, ResParticleCollisionUpdater, updater->Usage, res_updater )
            COPY_FLAG_UPDATER( updater, res_updater )
            COPY_VEC1( updater, res_updater, Power )
            COPY_VEC1( updater, res_updater, Position )

            u32 flag = 0;
            if ( updater->IsWorldCollision ) flag |= nw::gfx::res::ResParticleCollisionUpdater::COLLISION_IS_WORLD;
            switch ( updater->ReactionType )
            {
            case ProtocolEM4C::CollisionReactionType_Reflection:
                flag |= nw::gfx::res::ResParticleCollisionUpdater::COLLISION_REFLECTION;
                break;
            case ProtocolEM4C::CollisionReactionType_Cesser:
                flag |= nw::gfx::res::ResParticleCollisionUpdater::COLLISION_CESSER;
                break;
            default:
                break;
            }
            res_updater.SetFlags( flag );
        }
    }

    /*
        Shape builder
    */
    void
    SetShapeBuilder(ProtocolEM4C::MESSAGEPACKET_ParticleSet_ShapeBuilder* builder)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_RES_PARTICLE_SHAPE_BUILDER( selected, res_shape_builder )
            COPY_VEC1( builder, res_shape_builder, AscendingOrder )
            COPY_VEC3( builder, res_shape_builder, DrawOffset )
        }
    }

    /*
        Material
    */
    void
    SetMaterialBlender(ProtocolEM4C::MESSAGEPACKET_Material_MaterialBlender* material)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_MATERIAL( selected, obj_material )

            // MaterialOriginal settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetOriginal();
                res_material.SetTranslucencyKind( (nw::gfx::res::ResMaterial::TranslucencyKind)material->TranslucencyKind );
                SET_FLAGS( res_material, material->IsFogEnabled, nw::gfx::res::ResMaterialData::FLAG_FOG_ENABLED )
            }

            // MaterialBuffer settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetBuffer();
                res_material.SetTranslucencyKind( (nw::gfx::res::ResMaterial::TranslucencyKind)material->TranslucencyKind );
                SET_FLAGS( res_material, material->IsFogEnabled, nw::gfx::res::ResMaterialData::FLAG_FOG_ENABLED )
            }
        }
    }

    void
    SetMaterialColor(ProtocolEM4C::MESSAGEPACKET_Material_MaterialColor* material)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_MATERIAL( selected, obj_material )

            // MaterialOriginal settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetOriginal();
                GET_RES_MATERIAL_COLOR( res_material_color )
                COPY_RGB( material, res_material_color, Emission )
                COPY_RGB( material, res_material_color, Ambient )
                COPY_RGBA( material, res_material_color, Diffuse )
            }

            // MaterialBuffer settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetBuffer();
                GET_RES_MATERIAL_COLOR( res_material_color )
                COPY_RGB( material, res_material_color, Emission )
                COPY_RGB( material, res_material_color, Ambient )
                COPY_RGBA( material, res_material_color, Diffuse )
            }
        }
    }

    void
    SetStandardSampler(
        nw::gfx::res::ResMaterial res_material,
        ProtocolEM4C::MESSAGEPACKET_Material_MaterialMapper* material
    )
    {
        const nw::gfx::res::ResStandardTextureSampler::Wrap wrapList[] = {
            nw::gfx::res::ResStandardTextureSampler::WRAP_REPEAT,            // TextureSamplerWrap_Repeat = 0,           // Repeat
            nw::gfx::res::ResStandardTextureSampler::WRAP_MIRRORED_REPEAT,   // TextureSamplerWrap_MirroredRepeat = 1,           // MirroredRepeat
            nw::gfx::res::ResStandardTextureSampler::WRAP_CLAMP_TO_EDGE,     // TextureSamplerWrap_ClampToEdge = 2,           // ClampToEdge
            nw::gfx::res::ResStandardTextureSampler::WRAP_CLAMP_TO_BORDER,   // TextureSamplerWrap_ClampToBorder = 3,           // ClampToBorder
        };

        GET_RES_STANDARD_TEXTURE_SAMPLER( material->Index, res_standard_sampler )
        nw::gfx::res::ResTextureSampler::MinFilter filter = nw::gfx::res::ResTextureSampler::MINFILTER_LINEAR;
        if ( material->IsMipMapUsed ) {
            if ( material->IsMipMapInterpolated ) {
                filter = nw::gfx::res::ResTextureSampler::MINFILTER_LINEAR_MIPMAP_LINEAR;
            } else {
                filter = nw::gfx::res::ResTextureSampler::MINFILTER_LINEAR_MIPMAP_NEAREST;
            }
        }
        res_standard_sampler.SetMinFilter( filter );
        //res_standard_sampler.SetMagFilter( (nw::gfx::res::ResTextureSampler::MagFilter)material->magFilter );
        res_standard_sampler.SetWrapS( wrapList[ material->WrapS ] );
        res_standard_sampler.SetWrapT( wrapList[ material->WrapT ] );
        res_standard_sampler.SetMinLod( material->MinLod );
        res_standard_sampler.SetLodBias( material->LodBias );
    }

    void
    SetTextureCoordinator(
        nw::gfx::res::ResMaterial res_material,
        ProtocolEM4C::MESSAGEPACKET_Material_MaterialMapper* material
    )
    {
        const nw::gfx::res::ResTextureCoordinator::MappingMatrixMode matrixList[] = {
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_MAYA,       // TextureMatrixModeCtr_DccMaya = 0x0,           // Perform same texture coordinate calculations as for Maya.
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_SOFTIMAGE,  // TextureMatrixModeCtr_DccSoftimage = 0x1,           // Perform same texture coordinate calculations as for Softimage.
            nw::gfx::res::ResTextureCoordinator::MAPPINGMATRIXMODE_3DSMAX      // TextureMatrixModeCtr_Dcc3dsMax = 0x2,           // Perform same texture coordinate calculations as for 3dsmax.
        };

        GET_RES_TEXTURE_COORDINATOR( 0, res_coordinator )
        res_coordinator.SetScale(
            nw::math::VEC2( material->TexCoordScale.X,  material->TexCoordScale.Y )
        );
        res_coordinator.SetTranslate(
            nw::math::VEC2( material->TexCoordTranslate.X,  material->TexCoordTranslate.Y )
        );
        res_coordinator.SetMatrixMode( matrixList[ material->MatrixMode ] );
    }

    void
    SetMaterialMapper(ProtocolEM4C::MESSAGEPACKET_Material_MaterialMapper* material)
    {
        FOREACH_SELECTED_EM( selected )
        {
            GET_MATERIAL( selected, obj_material )

            // MaterialOriginal settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetOriginal();
                SetStandardSampler( res_material, material );
                SetTextureCoordinator( res_material, material );
            }

            // MaterialBuffer settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetBuffer();
                SetStandardSampler( res_material, material );
                SetTextureCoordinator( res_material, material );
            }
        }
    }

    void
    SetMaterialCombiner(ProtocolEM4C::MESSAGEPACKET_Material_MaterialCombiner* material)
    {
        const nw::gfx::res::ResTextureCombiner::Source sourceList[] = {
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE0,                   // TextureCombinerSourceKind_Texture0 = 0,           // Texture 0.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE1,                   // TextureCombinerSourceKind_Texture1 = 1,           // Texture 1.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE2,                   // TextureCombinerSourceKind_Texture2 = 2,           // Texture 2.
            nw::gfx::res::ResTextureCombiner::SOURCE_TEXTURE3,                   // TextureCombinerSourceKind_Texture3 = 3,           // Texture ３.
            nw::gfx::res::ResTextureCombiner::SOURCE_CONSTANT,                   // TextureCombinerSourceKind_Constant = 4,           // Constant color.
            nw::gfx::res::ResTextureCombiner::SOURCE_PRIMARY_COLOR,              // TextureCombinerSourceKind_PrimaryColor = 5,           // This is the fragment color that enters texture combiner 0 before texture processing.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS,                   // TextureCombinerSourceKind_Previous = 6,           // This is the fragment color that enters from the previous texture combiner.
            nw::gfx::res::ResTextureCombiner::SOURCE_PREVIOUS_BUFFER,            // TextureCombinerSourceKind_PreviousBuffer = 7,           // This is the color that enters from the previous texture combiner buffer.
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_PRIMARY_COLOR,     // TextureCombinerSourceKind_FragmentPrimaryColor = 8,           // Fragment light primary color (representing diffused, ambient, and emissive).
            nw::gfx::res::ResTextureCombiner::SOURCE_FRAGMENT_SECONDARY_COLOR,   //TextureCombinerSourceKind_FragmentSecondaryColor = 9,           // Fragment light secondary color (representing specular).
        };
        const nw::gfx::res::ResTextureCombiner::OperandRgb operandRGBList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_COLOR,          // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_COLOR,// TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_SRC_ALPHA,          // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_SRC_ALPHA,// TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_RED,                // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_GREEN,              // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_BLUE,               // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_RED,      // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_GREEN,    // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDRGB_ONE_MINUS_BLUE,     // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::OperandAlpha operandAlphaList[] = {
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Color = 0,           // Source color.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusColor = 1,           // 1 color
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ALPHA,            // TextureCombinerOperandKind_Alpha = 2,           // Source alpha value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_SRC_ONE_MINUS_ALPHA,  // TextureCombinerOperandKind_OneMinusAlpha = 3,           // 1 alpha.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_RED,                  // TextureCombinerOperandKind_Red = 4,           // Source red value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_GREEN,                // TextureCombinerOperandKind_Green = 5,           // Source green value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_BLUE,                 // TextureCombinerOperandKind_Blue = 6,           // Source blue value.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_RED,        // TextureCombinerOperandKind_OneMinusRed = 7,           // 1 red.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_GREEN,      // TextureCombinerOperandKind_OneMinusGreen = 8,           // 1 green.
            nw::gfx::res::ResTextureCombiner::OPERANDALPHA_ONE_MINUS_BLUE,       // TextureCombinerOperandKind_OneMinusBlue = 9,           // 1 blue.
        };
        const nw::gfx::res::ResTextureCombiner::Combine combineList[] = {
            nw::gfx::res::ResTextureCombiner::COMBINE_REPLACE,       // TextureCombineMode_Replace = 0,           // Uses operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_MODULATE,      // TextureCombineMode_Modulate = 1,           // Multiplies operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD,           // TextureCombineMode_Add = 2,           // Adds operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADDSIGNED,     // TextureCombineMode_AddSigned = 3,           // Subtracts 0.5 from the addition of operation 0 and operation1.
            nw::gfx::res::ResTextureCombiner::COMBINE_INTERPOLATE,   // TextureCombineMode_Interpolate = 4,           // Interpolates by operation 0 and operation 1 for operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_SUBTRACT,      // TextureCombineMode_Subtract = 5,           // Subtracts operation 1 from operation 0.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGB,      // TextureCombineMode_DotProduct3Rgb = 6,           // Replaces each of R, G, and B with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_DOT3_RGBA,     // TextureCombineMode_DotProduct3Rgba = 7,           // Replaces each of R, G, B, and A with the inner product of operation 0 and operation 1.
            nw::gfx::res::ResTextureCombiner::COMBINE_ADD_MULT,      // TextureCombineMode_AddMult = 8,           // Multiplies the addition of operation 0 and operation 1 with operation 2.
            nw::gfx::res::ResTextureCombiner::COMBINE_MULT_ADD,      // TextureCombineMode_MultAdd = 9,           // Adds the multiplication of operation 0 and operation 1 with operation 2.
        };

        FOREACH_SELECTED_EM( selected )
        {
            GET_MATERIAL( selected, obj_material )

            // MaterialOriginal settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetOriginal();

                GET_RES_MATERIAL_COLOR( res_material_color )
                COPY_RGBA( material, res_material_color, Constant0 )

                GET_RES_TEXTURE_COMBINER( res_combiner )
                res_combiner.SetCombineRgb( combineList[material->CombineRgb] );
                res_combiner.SetCombineAlpha( combineList[material->CombineAlpha] );
                res_combiner.SetScaleRgb( nw::gfx::res::ResTextureCombiner::SCALE_ONE );
                res_combiner.SetScaleAlpha( nw::gfx::res::ResTextureCombiner::SCALE_ONE );
                res_combiner.SetConstant( nw::gfx::res::ResTextureCombiner::CONSTANT0 );
                res_combiner.SetSourceRgb( 0, sourceList[material->SourceRgb0] );
                res_combiner.SetSourceRgb( 1, sourceList[material->SourceRgb1] );
                res_combiner.SetSourceRgb( 2, sourceList[material->SourceRgb2] );
                res_combiner.SetOperandRgb( 0, operandRGBList[material->OperandRgb0] );
                res_combiner.SetOperandRgb( 1, operandRGBList[material->OperandRgb1] );
                res_combiner.SetOperandRgb( 2, operandRGBList[material->OperandRgb2] );
                res_combiner.SetSourceAlpha( 0, sourceList[material->SourceAlpha0] );
                res_combiner.SetSourceAlpha( 1, sourceList[material->SourceAlpha1] );
                //res_combiner.SetSourceAlpha( 2, sourceList[material->SourceAlpha2] );
                res_combiner.SetOperandAlpha( 0, operandAlphaList[material->OperandAlpha0] );
                res_combiner.SetOperandAlpha( 1, operandAlphaList[material->OperandAlpha1] );
                //res_combiner.SetOperandAlpha( 2, operandAlphaList[material->OperandAlpha2] );
            }

            // MaterialBuffer settings
            {
                nw::gfx::res::ResMaterial res_material = obj_material->GetBuffer();

                GET_RES_MATERIAL_COLOR( res_material_color )
                COPY_RGBA( material, res_material_color, Constant0 )

                GET_RES_TEXTURE_COMBINER( res_combiner )
                res_combiner.SetCombineRgb( combineList[material->CombineRgb] );
                res_combiner.SetCombineAlpha( combineList[material->CombineAlpha] );
                res_combiner.SetScaleRgb( nw::gfx::res::ResTextureCombiner::SCALE_ONE );
                res_combiner.SetScaleAlpha( nw::gfx::res::ResTextureCombiner::SCALE_ONE );
                res_combiner.SetConstant( nw::gfx::res::ResTextureCombiner::CONSTANT0 );
                res_combiner.SetSourceRgb( 0, sourceList[material->SourceRgb0] );
                res_combiner.SetSourceRgb( 1, sourceList[material->SourceRgb1] );
                res_combiner.SetSourceRgb( 2, sourceList[material->SourceRgb2] );
                res_combiner.SetOperandRgb( 0, operandRGBList[material->OperandRgb0] );
                res_combiner.SetOperandRgb( 1, operandRGBList[material->OperandRgb1] );
                res_combiner.SetOperandRgb( 2, operandRGBList[material->OperandRgb2] );
                res_combiner.SetSourceAlpha( 0, sourceList[material->SourceAlpha0] );
                res_combiner.SetSourceAlpha( 1, sourceList[material->SourceAlpha1] );
                //res_combiner.SetSourceAlpha( 2, sourceList[material->SourceAlpha2] );
                res_combiner.SetOperandAlpha( 0, operandAlphaList[material->OperandAlpha0] );
                res_combiner.SetOperandAlpha( 1, operandAlphaList[material->OperandAlpha1] );
                //res_combiner.SetOperandAlpha( 2, operandAlphaList[material->OperandAlpha2] );
            }
        }
    }

    /*
        3D Link
    */
    void
    GatherSkeletalModelList(nw::ut::MoveArray<unsigned int>& hashBuffer, nw::ut::MoveArray<char>& strBuffer)
    {
        hashBuffer.clear();
        strBuffer.clear();

        vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->GetModelsBegin();
        vw::g3d::SceneManager::ModelList::iterator end = m_ScnMgr->GetModelsEnd();
        for ( ; it != end; ++it ) {
            if ( it->GetResObject().GetTypeInfo() == nw::gfx::res::ResSkeletalModel::TYPE_INFO )
            {
                hashBuffer.push_back(it->GetResSource()->GetHashID());
                char* name = (char*)it->GetResObject().GetName();
                while( *name != '\0' ) { strBuffer.push_back( *name++ ); };
                strBuffer.push_back( '\0' );
            }
        }
    }

    void
    GatherBoneList(const int hashID, nw::ut::MoveArray<char>& strBuffer, int& boneCount)
    {
        strBuffer.clear();
        boneCount = 0;

        vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->FindModel( hashID );
        if ( it != m_ScnMgr->GetModelsEnd() ) {
            if ( it->GetResObject().GetTypeInfo() == nw::gfx::res::ResSkeletalModel::TYPE_INFO )
            {
                nw::gfx::res::ResSkeletalModel res_model =
                    nw::ut::ResDynamicCast<nw::gfx::res::ResSkeletalModel>( it->GetResObject() );
                for ( int i = 0; i < res_model.GetSkeleton().GetBonesCount(); ++i )
                {
                    char* name = (char*)res_model.GetSkeleton().GetBones( i ).GetName();
                    while( *name != '\0' ) { strBuffer.push_back( *name++ ); };
                    strBuffer.push_back( '\0' );
                }

                boneCount = res_model.GetSkeleton().GetBonesCount();
            }
        }
    }

    void
    SetEmissionStartFrame(ProtocolEM4C::MESSAGEPACKET_BoneLinkage_SetBoneLink* link)
    {
        s32 startFrame = link->StartFrame;
        vw::g3d::ParticleModelElem *elem = &*m_ScnMgr->FindParticleModel( link->BcptlNameHashID );
        elem->SetEmissionStartFrame( link->StartFrame );
        elem->SetEmissionLength( link->EmissionLength );
    }

    void
    SetBoneLink(ProtocolEM4C::MESSAGEPACKET_BoneLinkage_SetBoneLink* link)
    {
        int selectedIndex = -1;
        if ( link->TargetType == ProtocolEM4C::EditTargetType_ParticleModel ) {
            selectedIndex = 0;
        } else if ( link->TargetType == ProtocolEM4C::EditTargetType_Emitter ) {
            selectedIndex = GetResParticleEmitterIndex(
                link->BcptlNameHashID,
                link->EmitterName.ToString()
            );
        }
        if ( selectedIndex == -1 ) return;

        nw::ut::MoveArray<nw::gfx::TransformNode*> nodes( Memory_GetAllocator() );
        if ( link->TargetType == ProtocolEM4C::EditTargetType_ParticleModel ) {
            nw::gfx::ParticleModel* particle_model;
            nw::gfx::res::ResParticleModel res_particle_model;
            if ( !GetResParticleModel( link->BcptlNameHashID, &res_particle_model ) ) return;
            if ( !GetParticleModel( res_particle_model, &particle_model ) ) return;
            nodes.push_back( particle_model );
        } else if ( link->TargetType == ProtocolEM4C::EditTargetType_Emitter ) {
            nw::gfx::ParticleEmitter* emitter;
            nw::gfx::res::ResParticleEmitter res_emitter;
            if ( !GetResParticleEmitter( link->BcptlNameHashID, selectedIndex, &res_emitter ) ) return;
            if ( !GetParticleEmitter( res_emitter, &emitter ) ) return;
            nodes.push_back( emitter );
        }

        // For ParticleModel, also add sibling Emitter to constraint target
        if ( link->TargetType == ProtocolEM4C::EditTargetType_ParticleModel )
        {
            nw::gfx::ParticleModel* model = nw::ut::DynamicCast<nw::gfx::ParticleModel*>( nodes[ 0 ] );
            vw::g3d::SceneManager::ParticleModelList::iterator elem = m_ScnMgr->FindParticleModel( model );
            if ( elem == m_ScnMgr->GetParticleModelsEnd() ) return;

            nw::gfx::ParticleEmitterArray::iterator it = elem->GetParticleEmittersBegin();
            nw::gfx::ParticleEmitterArray::iterator end = elem->GetParticleEmittersEnd();
            for ( ; it != end; ++it ) {
                if ( (*it)->GetParent() != model ) nodes.push_back( *it );
            }
        }

        // Cancels the target object link.
        {
            vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->GetModelsBegin();
            vw::g3d::SceneManager::ModelList::iterator end = m_ScnMgr->GetModelsEnd();
            for ( ; it != end; ++it ) {
                nw::ut::MoveArray<nw::gfx::TransformNode*>::iterator it_ = nodes.begin();
                nw::ut::MoveArray<nw::gfx::TransformNode*>::iterator end_ = nodes.end();
                for ( ; it_ != end_; ++it_ ) {
                    it->GetNodeConstraint().DeleteConstraint(*it_);
                }
            }
        }

        // Converts character string to char.
        const int MAX_PATH_SIZE = 260;
        char utf8Str[MAX_PATH_SIZE];
        vw::system::ConvertStrWtoUTF8(link->BoneName.ToString(), utf8Str, MAX_PATH_SIZE);

        if ( utf8Str[0] != 0 )
        {
            vw::g3d::SceneManager::ModelList::iterator it = m_ScnMgr->FindModel( link->ModelNameHashID );
            if ( it != m_ScnMgr->GetModelsEnd() ) {
                if ( it->GetResObject().GetTypeInfo() == nw::gfx::res::ResSkeletalModel::TYPE_INFO )
                {
                    // Constraint object declaration
                    vw::g3d::Constraint constraint;
                    memcpy( constraint.m_OffsetScale, &( link->Scale.X ), sizeof( float ) * 3 );
                    memcpy( constraint.m_OffsetRotate, &( link->Rotate.X ), sizeof( float ) * 3 );
                    memcpy( constraint.m_OffsetTranslate, &( link->Translate.X ), sizeof( float ) * 3 );
                    constraint.m_UseOffset[0] = true;
                    constraint.m_UseOffset[1] = true;
                    constraint.m_UseOffset[2] = true;

                    // Process when the target is the ParticleModel and a sibling Emitter exists
                    if ( nodes.size() >= 2 ) {

                        nw::ut::MoveArray<nw::gfx::TransformNode*>::iterator it_ = nodes.begin();
                        nw::ut::MoveArray<nw::gfx::TransformNode*>::iterator end_ = nodes.end();

                        // ParticleModel settings
                        {
                            constraint.m_ConstrainedObject = *it_;
                            // Offset enables only scale
                            constraint.m_UseOffset[0] = true;
                            constraint.m_UseOffset[1] = true;
                            constraint.m_UseOffset[2] = true;
                            // The constraint target position matrix is invalid
                            constraint.m_InheritTransform[0] = false;
                            constraint.m_InheritTransform[1] = false;
                            constraint.m_InheritTransform[2] = false;
                            it->GetNodeConstraint().AddConstraint( constraint, utf8Str );
                            ++it_;
                        }

                        // Sibling ParticleEmitter settings
                        {
                            constraint.m_UseOffset[0] = true;
                            constraint.m_UseOffset[1] = true;
                            constraint.m_UseOffset[2] = true;
                            constraint.m_InheritTransform[0] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowScale;
                            constraint.m_InheritTransform[1] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowRotate;
                            constraint.m_InheritTransform[2] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowTranslate;
                            for ( ; it_ != end_; ++it_ ) {
                                constraint.m_ConstrainedObject = *it_;
                                it->GetNodeConstraint().AddConstraint( constraint, utf8Str );
                            }
                        }

                    // Process for all other cases
                    } else {
                        constraint.m_ConstrainedObject = nodes[ 0 ];
                        constraint.m_InheritTransform[0] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowScale;
                        constraint.m_InheritTransform[1] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowRotate;
                        constraint.m_InheritTransform[2] = link->FollowFlag & ProtocolEM4C::FollowBitType_IsFollowTranslate;
                        it->GetNodeConstraint().AddConstraint( constraint, utf8Str );
                    }

                    // Move camera focal point
                    const nw::gfx::CalculatedTransform* trans = it->GetNodeConstraint().GetBoneTransform( nodes[ 0 ] );
                    if ( trans ) {
                        nw::math::VEC3 translate;
                        trans->GetTranslate( &translate );
                        vw::g3d::CameraMgr::GetInstance()->Focus( translate );
                    }
                }
            }

            system::App::GetInstance().SetFocusID( system::EFFECT_ID );
        }
        SetEmissionStartFrame(link);
    }

    /*
        Other
    */
    void
    StartUpdateAll()
    {
        vw::effect::AnimManager::GetInstance()->PlayAll();
        vw::g3d::AnimManager::GetInstance()->PlayAll();
    }

    void
    StepUpdateAll()
    {
        vw::effect::AnimManager::GetInstance()->StepForwardAll();
        vw::g3d::AnimManager::GetInstance()->StepForwardAll();
    }

    void
    StopUpdateAll()
    {
        vw::effect::AnimManager::GetInstance()->PauseAll();
        vw::g3d::AnimManager::GetInstance()->PauseAll();
    }

    void
    MoveFirstFrame()
    {
        vw::effect::AnimManager::GetInstance()->StopAll();
        vw::g3d::AnimManager::GetInstance()->StopAll();
    }

    void
    SetFrameRange(ProtocolEM4C::MESSAGEPACKET_Object_SetFrameRange *body)
    {
        system::App::GetInstance().SetFocusID( system::EFFECT_ID );
        vw::effect::AnimManager::GetInstance()->SetStartFrame( body->StartFrame );
        vw::effect::AnimManager::GetInstance()->SetMaxFrameCount( body->EndFrame );
    }

    /*
        callback call
    */
    void
    CallParticleModelCallBack()
    {
        // Gets the callback class.
        nw::viewer::ef::IResourceListener* efListener =
            nw::viewer::sys::App::GetInstance().GetModuleEF().GetResourceListener();

        if (efListener)
        {
            FOREACH_SELECTED_EM( selected )
            {
                GET_PARTICLE_MODEL( selected, model )
                if (model) {
                    efListener->OnUpdateParticleModel( model );
                }
            }
        }
    }

    void
    CallParticleEmitterCallBack()
    {
        // Gets the callback class.
        nw::viewer::ef::IResourceListener* efListener =
            nw::viewer::sys::App::GetInstance().GetModuleEF().GetResourceListener();

        if (efListener)
        {
            FOREACH_SELECTED_EM( selected )
            {
                GET_PARTICLE_EMITTER( selected, emitter )
                if (emitter) {
                    efListener->OnUpdateParticleEmitter( emitter );
                }
            }
        }
    }

    void
    CallParticleSetCallBack()
    {
        // Gets the callback class.
        nw::viewer::ef::IResourceListener* efListener =
            nw::viewer::sys::App::GetInstance().GetModuleEF().GetResourceListener();

        if (efListener)
        {
            FOREACH_SELECTED_EM( selected )
            {
                GET_PARTICLE_SET( selected, set )
                if (pModel && set) {
                    efListener->OnUpdateParticleSet( pModel, set );
                }
            }
        }
    }

    void
    CallMaterialCallBack()
    {
        // Gets the callback class.
        nw::viewer::ef::IResourceListener* efListener =
            nw::viewer::sys::App::GetInstance().GetModuleEF().GetResourceListener();

        if (efListener)
        {
            FOREACH_SELECTED_EM( selected )
            {
                GET_MATERIAL( selected, objMaterial )
                if (pModel && objMaterial) {
                    efListener->OnUpdateMaterial( pModel, objMaterial );
                }
            }
        }
    }

private:
    vw::g3d::ResourceManager *m_ResMgr;
    vw::g3d::SceneManager *m_ScnMgr;
    ProtocolEM4C::EditTargetType  m_SelectedType;
    nw::ut::MoveArray<ProtocolEM4C::EditSelectedObject> m_SelectedObject;
    static ParamSetter* m_pInstance;

    ParamSetter() : m_SelectedObject( Memory_GetAllocator() )
    {
        m_ResMgr = vw::g3d::ResourceManager::GetInstance();
        m_ScnMgr = vw::g3d::SceneManager::GetInstance();
    }

    ~ParamSetter() {}
};

}
}
#endif
