﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4R_VIEWER_EFFECT_CHECKSUM_H_
#define NW4R_VIEWER_EFFECT_CHECKSUM_H_

#include <dolphin.h>
#include <nw4r/ut.h>

namespace   vw
{
namespace   effect
{


inline u32
_CheckSum(const nw4r::ut::List* list)
{
    u32 sum = 0;
    const nw4r::ef::LinkedObject* obj;
    for (obj = reinterpret_cast<const nw4r::ef::LinkedObject*>(nw4r::ut::List_GetFirstConst(list));
       obj != NULL;
       obj = reinterpret_cast<const nw4r::ef::LinkedObject*>(nw4r::ut::List_GetNextConst(list, obj)))
    {
      sum += obj->GetObjectID();
    }
    return sum;
}

inline u32
_CheckSum(const nw4r::ef::ActivityList* act)
{
    u32 sum = 0;
    sum += act->mNumActive;
    sum += _CheckSum(&act->mActiveList);
    sum += _CheckSum(&act->mClosingList);
    return sum;
}


inline u32
_CheckSum(const nw4r::ef::LinkedObject* obj)
{
    u32 sum = 0;
    sum += obj->mObjectID;
    return sum;
}

inline u32
_CheckSum(const nw4r::ef::ReferencedObject* obj)
{
    u32 sum = _CheckSum(static_cast<const nw4r::ef::LinkedObject*>(obj));
    sum += static_cast<u32>(obj->GetLifeStatus());
    sum += obj->GetRefCount();
    return sum;
}

inline u32
_CheckSum(const nw4r::ef::Random* obj)
{
    return obj->GetCurrentSeed();
}

inline u32
_CheckSum(const nw4r::math::MTX34* mtx)
{
    u32 sum = 0;
    int i;
    for (i = 0; i < 12; i++)
    {
        sum += *reinterpret_cast<const u32*>(&mtx->a[i]);
    }
    return sum;
}

inline u32
_CheckSum(const nw4r::math::VEC2* vec)
{
    u32 sum = 0;
    sum += *reinterpret_cast<const u32*>(&vec->x);
    sum += *reinterpret_cast<const u32*>(&vec->y);
    return sum;
}

inline u32
_CheckSum(const nw4r::math::VEC3* vec)
{
    u32 sum = 0;
    sum += *reinterpret_cast<const u32*>(&vec->x);
    sum += *reinterpret_cast<const u32*>(&vec->y);
    sum += *reinterpret_cast<const u32*>(&vec->z);
    return sum;
}

inline u32
_CheckSum(const GXColor* color)
{
    u32 sum = 0;
    sum += static_cast<u32>(color->r) << 24;
    sum += static_cast<u32>(color->g) << 16;
    sum += static_cast<u32>(color->b) << 8;
    sum += static_cast<u32>(color->a);
    return sum;
}



}
}
#endif
