/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4R_VIEWER_EFFECT_DRAWUTIL_H_
#define NW4R_VIEWER_EFFECT_DRAWUTIL_H_

#include <dolphin.h>
#include <viewer/system.h>
#include <viewer/effect.h>

namespace   vw
{
namespace   effect
{

/*
Utility to display add-on information specific to Effects on the Viewer
*/
namespace DrawUtil
{

void Setup();
void DrawTetra(const Mtx mtx);
void DrawArrow(const nw4r::math::MTX34* pnmtx, const GXColor color, bool drawHead = true);
void DrawTargetMark(const nw4r::math::MTX34* pnmtx);


inline void
Setup()
{
    GXSetNumChans(1);
    GXColor color = {128, 128, 128, 128};
    GXSetChanMatColor(GX_COLOR0A0, color);
    GXSetChanAmbColor(GX_COLOR0A0, color);
    GXSetChanCtrl(GX_COLOR0A0, GX_FALSE, GX_SRC_REG, GX_SRC_REG, GX_LIGHT_NULL, GX_DF_NONE, GX_AF_NONE);
    GXSetNumTexGens(0);
    GXSetNumIndStages(0);
    GXSetNumTevStages(1);
    GXSetTevOrder(GX_TEVSTAGE0, GX_TEXCOORD0, GX_TEXMAP_NULL, GX_COLOR0A0);
    GXSetTevOp(GX_TEVSTAGE0, GX_PASSCLR);
    GXSetFog(GX_FOG_NONE, 0, 1, 0, 1, color);
    GXSetZMode(GX_TRUE, GX_LESS, GX_FALSE);
    GXSetZCompLoc(GX_TRUE);
    GXSetCullMode(GX_CULL_NONE);
    GXSetBlendMode(GX_BM_BLEND, GX_BL_SRCALPHA, GX_BL_INVSRCALPHA, GX_LO_CLEAR);
    GXSetAlphaCompare(GX_ALWAYS, 0, GX_AOP_OR, GX_ALWAYS, 0);
    for (int i = 0; i < GX_MAX_TEVSTAGE; i++)
    {
        GXSetTevDirect(static_cast<GXTevStageID>(GX_TEVSTAGE0 + i));
    }
}

/*---------------------------------------------------------------------------*
  Name:         DrawTetra

  Description:  Renders tetrahedron

  Arguments:    vec[]
                mtx

  Returns:      None.
 *---------------------------------------------------------------------------*/
inline void
DrawTetra(const Mtx aMtx )
{
    static const f32 ROOT3 = 1.73205080756f;
    const Vec vtx[4] =
    {
        { 0.0f,  0.0f, 0.0f                   },
        { -0.2f, -0.1f, -0.1f * (ROOT3 / 3)    },
        { -0.2f,  0.1f, -0.1f * (ROOT3 / 3)    },
        { -0.2f,  0.0f, 0.1f * (ROOT3 * 2 / 3) },
    };

    Mtx mtx;
#if 0
    int i;
    for (i = 0; i < 3; i++)
    {
        nw4r::math::VEC3 v;
        v.x = aMtx[0][i];
        v.y = aMtx[1][i];
        v.z = aMtx[2][i];

        nw4r::math::VEC3Normalize(&v, &v);

        mtx[0][i] = v.x;
        mtx[1][i] = v.y;
        mtx[2][i] = v.z;
    }
    for (i = 0; i < 3; i++)
    {
        mtx[i][3] = aMtx[i][3];
    }
#endif
    MTXCopy(aMtx, mtx);
    {
        Mtx t;
        MTXTrans(t, 1, 0, 0);
        MTXConcat(mtx, t, mtx);

        Mtx s;
        f32 scale = 0.3F / ModuleEffect::GetEffectModule()->GetGuideScale();
        MTXScale(s, scale, scale, scale);
        MTXConcat(mtx, s, mtx);
    }

    Vec tVtx[4];

    MTXMultVec( mtx, &vtx[0], &tVtx[0] );
    MTXMultVec( mtx, &vtx[1], &tVtx[1] );
    MTXMultVec( mtx, &vtx[2], &tVtx[2] );
    MTXMultVec( mtx, &vtx[3], &tVtx[3] );

    GXBegin( GX_TRIANGLESTRIP, GX_VTXFMT0, 6 );
        GXPosition3f32( tVtx[0].x,  tVtx[0].y,  tVtx[0].z );
        GXPosition3f32( tVtx[1].x,  tVtx[1].y,  tVtx[1].z );
        GXPosition3f32( tVtx[2].x,  tVtx[2].y,  tVtx[2].z );
        GXPosition3f32( tVtx[3].x,  tVtx[3].y,  tVtx[3].z );
        GXPosition3f32( tVtx[0].x,  tVtx[0].y,  tVtx[0].z );
        GXPosition3f32( tVtx[1].x,  tVtx[1].y,  tVtx[1].z );
    GXEnd();
}


/*---------------------------------------------------------------------------*
  Name:         Stage::DrawTargetMark
  Description:  Renders focal point mark
  Arguments:    None.
  Returns:      None.
*---------------------------------------------------------------------------*/
inline void
DrawTargetMark(const nw4r::math::MTX34* pnmtx)
{
    // GX setup
    GXClearVtxDesc();
    GXSetVtxAttrFmt( GX_VTXFMT0, GX_VA_POS, GX_POS_XYZ, GX_F32, 0 );
    GXSetVtxDesc   ( GX_VA_POS, GX_DIRECT );

    GXSetLineWidth( 6, GX_TO_ZERO );

    nw4r::math::MTX34 v;

    //const Vec* vec = GetViewCamera()->GetTargetPos();
    //MTXTrans( v, vec->x, vec->y, vec->z );
    //nw4r::math::MTX34Mult(&v, pnmtx, &v);
    //MTXScale( v, 0.2f, 0.2f, 0.2f );
    MTXScale(v, 1, 1, 1);
    nw4r::math::MTX34Mult(&v, pnmtx, &v);

    GXLoadPosMtxImm(v, GX_PNMTX0);
    GXSetCurrentMtx(GX_PNMTX0);

    Mtx mtx;

    GXColor colorX = {255, 128, 128, 255};
    GXColor colorY = {128, 255, 128, 255};
    GXColor colorZ = {128, 128, 255, 255};

    // X-axis
    GXSetChanMatColor(GX_COLOR0A0, colorX);
    GXBegin( GX_LINES, GX_VTXFMT0, 2 );
        GXPosition3f32( -1.0f,  0.0f,  0.0f );
        GXPosition3f32(  1.0f,  0.0f,  0.0f );
    GXEnd();
    MTXIdentity( mtx );
    DrawUtil::DrawTetra(mtx );

    // Y axis
    GXSetChanMatColor( GX_COLOR0A0, colorY);
    GXBegin( GX_LINES, GX_VTXFMT0, 2 );
        GXPosition3f32(  0.0f,  1.0f,  0.0f );
        GXPosition3f32(  0.0f, -1.0f,  0.0f );
    GXEnd();
    MTXRotRad( mtx, 'z', MTXDegToRad(90.0f) );
    DrawUtil::DrawTetra(mtx );

    // Z-axis
    GXSetChanMatColor( GX_COLOR0A0, colorZ);
    GXBegin( GX_LINES, GX_VTXFMT0, 2 );
        GXPosition3f32(  0.0f,  0.0f,  1.0f );
        GXPosition3f32(  0.0f,  0.0f, -1.0f );
    GXEnd();
    MTXRotRad( mtx, 'y', MTXDegToRad(-90.0f) );
    DrawUtil::DrawTetra(mtx );
}


inline void
DrawArrow(const nw4r::math::MTX34* pnmtx, const GXColor color, bool drawHead)
{
    // GX setup
    GXClearVtxDesc();
    GXSetVtxAttrFmt( GX_VTXFMT0, GX_VA_POS, GX_POS_XYZ, GX_F32, 0 );
    GXSetVtxDesc   ( GX_VA_POS, GX_DIRECT );

    GXSetLineWidth( 6, GX_TO_ZERO );

    nw4r::math::MTX34 v;

    MTXScale(v, 1, 1, 1);
    nw4r::math::MTX34Mult(&v, pnmtx, &v);

    GXLoadPosMtxImm(v, GX_PNMTX0);
    GXSetCurrentMtx(GX_PNMTX0);

    Mtx mtx;

    // Y axis
    GXSetChanMatColor( GX_COLOR0A0, color);
    GXBegin( GX_LINES, GX_VTXFMT0, 2 );
        GXPosition3f32(  0.0f,  1.0f,  0.0f );
        GXPosition3f32(  0.0f,  0.0f,  0.0f );
    GXEnd();

    if (drawHead)
    {
        MTXRotRad( mtx, 'z', MTXDegToRad(90.0f) );
        DrawUtil::DrawTetra(mtx );
    }
}


}

void SpaceNormalize(nw4r::math::MTX34* pnmtx);

inline void
SpaceNormalize(nw4r::math::MTX34* pnmtx)
{
    {
        nw4r::math::VEC3 v;

        v.x = pnmtx->_01;
        v.y = pnmtx->_11;
        v.z = pnmtx->_21;
        nw4r::math::VEC3Normalize(&v, &v);
        pnmtx->_01 = v.x;
        pnmtx->_11 = v.y;
        pnmtx->_21 = v.z;

        v.x = pnmtx->_00;
        v.y = pnmtx->_10;
        v.z = pnmtx->_20;
        nw4r::math::VEC3Normalize(&v, &v);
        pnmtx->_00 = v.x;
        pnmtx->_10 = v.y;
        pnmtx->_20 = v.z;

        v.x = pnmtx->_02;
        v.y = pnmtx->_12;
        v.z = pnmtx->_22;
        nw4r::math::VEC3Normalize(&v, &v);
        pnmtx->_02 = v.x;
        pnmtx->_12 = v.y;
        pnmtx->_22 = v.z;
    }
    {
        f32 k = -pnmtx->_23;

        if (k != 0)
        {
            nw4r::math::VEC3 vec(k, k, k);
            nw4r::math::MTX34Scale(pnmtx, pnmtx, &vec);
        }
    }
    {
        f32 scale = 0.05F;
        nw4r::math::VEC3 vec(scale, scale, scale);
        nw4r::math::MTX34Scale(pnmtx, pnmtx, &vec);
    }
}


}
}

#endif

