﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/
#ifndef NW4R_VIEWER_EFFECT_MemoryManager_ext_H_
#define NW4R_VIEWER_EFFECT_MemoryManager_ext_H_

#include <new>
#include <dolphin.h>
#include <nw4r/ut/inlines.h>
#include <nw4r/ut/list.h>
#include "nw4r/ef.h"

namespace   vw
{
namespace   effect
{


class EffectSystemExt : public nw4r::ef::EffectSystem
{
public:
    u32 _CheckSum() const;
};

class EffectExt : public nw4r::ef::Effect
{
friend class MemoryManagerExt;
public:
    virtual bool Initialize(nw4r::ef::EffectSystem* manager, nw4r::ef::EmitterResource* eh, u16 calcRemain);
    u32 _CheckSum() const;
    static void PrevEmission(
        nw4r::ef::Emitter* em,
        nw4r::ef::ParticleManager* pm,
        int* count,
        u32* flags,
        f32 (*params)[6],
        u16* aPtclLife,
        f32* aPtclLifeRnd,
        nw4r::math::MTX34* space
    );
};

class EmitterExt : public nw4r::ef::Emitter
{
friend class MemoryManagerExt;
private:
    u32 mEmissionRandom;
public:
    u32 _CheckSum() const;
    void DrawFormInfo(const nw4r::math::MTX34* vmtx);
    void DrawFieldInfo(const nw4r::math::MTX34* vmtx, u32 frame, u32 mask);
    void DrawPostFieldInfo(const nw4r::math::MTX34* vmtx, u32 mask);
    void DrawSpecDir(const nw4r::math::MTX34* vmtx);
    void DrawIcon(const nw4r::math::MTX34* vmtx);
    void DrawPointLightIcon(const nw4r::math::MTX34* vmtx);
    void DrawAxis(const nw4r::math::MTX34* vmtx);
    void SetEmissionRandom(u32 val)
    {
        mEmissionRandom = val;
    }
    bool                UpdateDatas(nw4r::ef::EmitterResource* eh);
};

class ParticleExt : public nw4r::ef::Particle
{
friend class MemoryManagerExt;
public:
    struct ReferenceInfo
    {
        enum ReferenceInfoFlag
        {
            REFERENCEINFOFLAG_SCALE = 1,
            REFERENCEINFOFLAG_ALPHA_PRI = 2,
            REFERENCEINFOFLAG_ALPHA_SEC = 4,
            REFERENCEINFOFLAG_COLOR = 8,
            REFERENCEINFOFLAG_ROTATE = 16
        };

        u8 mFlag;
        u8 mRotateOffset[3];
        nw4r::math::VEC2 mSize;
        GXColor mColor[2][2];
    } mRefInfo;

    u32 _CheckSum() const;
    virtual bool    Initialize
                    (
                        u16 life,
                        nw4r::math::VEC3 position,
                        nw4r::math::VEC3 velocity,
                        nw4r::ef::ParticleManager* pm,
                        const nw4r::math::MTX34* space,
                        f32 momentum,
                        const nw4r::ef::EmitterInheritSetting* setting,
                        nw4r::ef::Particle* referencePP
                    );
};

class ParticleManagerExt : public nw4r::ef::ParticleManager
{
friend class MemoryManagerExt;
public:
    u32 _CheckSum() const;
    void DrawInfo(const nw4r::math::MTX34* vmtx, u32 frame, u32 mask);
    void DrawPostFieldInfo(const nw4r::math::MTX34* vmtx, u32 mask);
    void DrawPointLightIcon(const nw4r::math::MTX34* vmtx);

    bool        UpdateDatas(nw4r::ef::EmitterResource* resource);
    bool        UpdatePtclDatas(nw4r::ef::EmitterResource* resource);
};

#define NW4R_EF_MEMCLASS_MEMMANAGER MemoryManagerExt
#define NW4R_EF_MEMCLASS_EFFECT vw::effect::EffectExt
#define NW4R_EF_MEMCLASS_EMITTER vw::effect::EmitterExt
#define NW4R_EF_MEMCLASS_PARTICLEMANAGER vw::effect::ParticleManagerExt
#define NW4R_EF_MEMCLASS_PARTICLE vw::effect::ParticleExt

#include "nw4r/ef/memorymanagertmp.h"
}
}
#endif
