/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/


#include <viewer/effect/ProtocolEM4C.h>
#include "../include/SurvivalStreamTask.h"



namespace vw     {
namespace effect {

static void SurvivalStream_Main_( void* arg );


/* NOTE: The same ping process for confirmation of existence is performed by each module.
 * Because disconnect detection is not performed on the server with MCS currently, this process is necessary for disconnect detection.
 * When it becomes possible to detect disconnection for the communication circuit on the MCS server in the future, this process should not need to be performed by each module independently.
 * 
 */

// Communication task for existence confirmation
static system::VTaskObj sSurvivalStreamTask;

/*---------------------------------------------------------------------------*
  Name:         SurvivalStream_Main_
  Description:  Communication process for existence confirmation
  Arguments:    retraceCount    VIRetrace count value
                arg     Parameter arguments
  Returns:      None.
 *---------------------------------------------------------------------------


*/
static void
SurvivalStream_Main_(void* arg )
{
    u32 size;

    while ( ( size = system::Stream_GetReadableBytes( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL ) ) >= sizeof(ProtocolEM4C::Header) )
    {
        ProtocolEM4C::Header header;
        system::Stream_Peek( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL, &header, sizeof(ProtocolEM4C::Header) );
        if ( size < (header.size + sizeof(ProtocolEM4C::Header)) )
        {
            break;
        }
        system::Stream_Skip( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL, sizeof(ProtocolEM4C::Header) );
        system::Stream_Skip( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL, header.size );
        header.size = 0;
        system::Stream_Write( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL, &header, sizeof(ProtocolEM4C::Header) );
    }
}


/*---------------------------------------------------------------------------*
  Name:         SurvivalStream_Init
  Description:  Registers the existence confirmation process to VTask
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
SurvivalStream_Init( void )
{
    static const u32 SUVIVAL_STREAM_BUFFER_SIZE = 128;

    // Create MCS channel for existence confirmation communication
    system::Stream_CreateChannel( ProtocolEM4C::Constants::CHANNEL_ID_EFFECTMAKER_SURVIVAL, SUVIVAL_STREAM_BUFFER_SIZE );

    // Add SurvivalStream_Main_() to VTask that is started in each VTrace
    // The existence confirmation result will be returned as long as the program is alive even if the main thread is blocked.
    VTask_InitObj( &sSurvivalStreamTask, SurvivalStream_Main_, NULL );
    VTask_AddObj( &sSurvivalStreamTask );
}


}   /* namespace effect */
}   /* namespace vw  */

