/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G2D_MENU_SUB_MODE_H_
#define NW4C_VIEWER_G2D_MENU_SUB_MODE_H_

#include "Module.h"
#include "SubMode.h"

namespace vw {
namespace g2d {

#define DECL_MENU( type, base ) \
class type##Menu : public base \
{ \
public: \
    type##Menu(); \
    virtual ~type##Menu(); \
    virtual void Prev(); \
    virtual void Next(); \
    virtual void Apply(); \
    virtual bool Draw(); \
    virtual const char* GetMenuName() const; \
    virtual MenuType GetMenuType() const; \
};

// Menu type
enum MenuType {
    Menu_Reset,
    Menu_ScreenCapture,
#ifdef NW_TEST_LYT_VRAM
    Menu_TexMemArea,
#endif
    Menu_ZCompare,
    Menu_Fovy,
    Menu_Near,
    Menu_Far,
    Menu_MinNear,
    Menu_Stereo_BasePlane,
    Menu_Stereo_Factor,
    Menu_Stereo_RealSwitch,
    Menu_Max
};

// Menu interface declaration
class IMenu
{
public:
    IMenu() {};
    virtual ~IMenu() {};
    virtual void Prev() {};
    virtual void Next() {};
    virtual void Apply() {};
    virtual bool Draw() { return false; }
    virtual const char* GetMenuName() const = 0;
    virtual MenuType GetMenuType() const = 0;
};

// Menu sub-mode
class MenuSubModeBase : public ISubMode
{
public:
    MenuSubModeBase();
    virtual ~MenuSubModeBase();
    virtual bool Main(vw::system::HidStatus& hidStatus);
    virtual void Draw();
    virtual const char* GetModeName() const;
    virtual SubModeType GetModeType() const;

protected:
    nw::ut::MoveArray<IMenu*> mMenu;
    int mCurrentMenuIndex;
};

// Menu sub-mode
class MenuSubMode : public MenuSubModeBase
{
public:
    MenuSubMode();
    virtual ~MenuSubMode();
};


}   /* namespace g2d */
}   /* namespace vw  */

#endif
