﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/SubMode.h"

namespace vw  {
namespace g2d {


#define DECL_SUBMODE_FUNC( type ) \
type##SubMode::type##SubMode() {} \
type##SubMode::~type##SubMode() {} \
void type##SubMode::Draw() { SubModeBase::Draw(); } \
const char* type##SubMode::GetModeName() const { return #type; } \
SubModeType type##SubMode::GetModeType() const { return SubMode_##type; }

DECL_SUBMODE_FUNC( Main )


/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeBase::SubModeBase()
 : mTouchPanelStartX(-1), mTouchPanelStartY(-1),
   mTouchPanelEndX(-1), mTouchPanelEndY(-1)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeBase::~SubModeBase()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SubModeBase::Main(vw::system::HidStatus& hidStatus)
{
    if(hidStatus.touchPanel.touch) {
        u16 x = hidStatus.touchPanel.x;
        u16 y = hidStatus.touchPanel.y;

        if(-1 == mTouchPanelStartX && -1 == mTouchPanelStartY)
        {
            mTouchPanelStartX = x;
            mTouchPanelStartY = y;
        }

        mTouchPanelEndX = x;
        mTouchPanelEndY = y;
    } else {
        mTouchPanelStartX = -1;
        mTouchPanelStartY = -1;
        mTouchPanelEndX = -1;
        mTouchPanelEndY = -1;
    }

    return false;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SubModeBase::Draw()
{
    vw::system::Draw2DHelper& draw2D = vw::system::Draw2DHelper::GetInstance();

    const nw::ut::Color8 white = nw::ut::Color8(~0);
    int w = 5;
    int sx = mTouchPanelStartX;
    int sy = mTouchPanelStartY;
    int ex = mTouchPanelEndX;
    int ey = mTouchPanelEndY;
    if((-1 != sx) && (-1 != ey) && sx!=ex)
    {
        draw2D.BeginDraw();
        draw2D.SetLineWidth(3);

        draw2D.DrawLine(sx,sy,ex,ey,white);

        nw::math::VEC2 dir  = nw::math::VEC2(sx,sy) - nw::math::VEC2(ex,ey);
        VEC2Normalize(&dir,&dir);
        nw::math::VEC2 right = nw::math::VEC2(dir.y,-dir.x);

        nw::math::VEC2 p = nw::math::VEC2(ex,ey);
        nw::math::VEC2 r = right*8;

        nw::math::VEC2 pt1 = p - dir*20;
        nw::math::VEC2 pt2 = p + r;
        nw::math::VEC2 pt3 = p - r;

        draw2D.FillTriangle(pt1,pt2,pt3,white);
        draw2D.EndDraw();
    }
}

bool
MainSubMode::Main(vw::system::HidStatus& hidStatus)
{
    return false;
}

}   /* namespace g2d */
}   /* namespace vw  */
