﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/SubMode.h"
#include "../include/InfoSubMode.h"
#include "../include/MenuSubMode.h"
#include "../../system/include/HidManager.h"

namespace vw  {
namespace g2d {


#define DELETE_SUBMODE( type ) \
{ \
    type##SubMode* mode = (type##SubMode*)mMode[i]; \
    mode->~type##SubMode(); \
    pAllocator->Free( mode ); \
} \

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
bool
SubModeManagerBase::Main(vw::system::HidStatus& hidStatus)
{
    s32 modeSelect = 0;
    switch (system::GetPadEvent(hidStatus, system::GetSubModePadBindings()))
    {
    case system::SUB_SWITCH_PREV_SUBMODE:
        modeSelect = -1;
        break;
    case system::SUB_SWITCH_NEXT_SUBMODE:
        modeSelect = +1;
        break;
    default:
        break;
    }

    if( modeSelect != 0 )
    {
        mCurrentModeIndex += modeSelect;
        if(mCurrentModeIndex < 0)
        {
            mCurrentModeIndex = mMode.size()-1;
        }
        if(mCurrentModeIndex >= mMode.size())
        {
            mCurrentModeIndex = 0;
        }

        return true;
    }
    else
    {
        if(mCurrentModeIndex < mMode.size())
        {
            return mMode[mCurrentModeIndex]->Main(hidStatus);
        }
    }

    return false;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SubModeManagerBase::Draw()
{
    if (mCurrentModeIndex < mMode.size()) {
        mMode[mCurrentModeIndex]->Draw();
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SubModeManagerBase::SetMode(SubModeType mode)
{
    for (int i = 0; i < mMode.size(); ++i )
    {
        if (mMode[i]->GetModeType() == mode)
        {
            mCurrentModeIndex = i;
        }
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeType
SubModeManagerBase::GetCurrentModeType() const
{
    NW_ASSERT(mCurrentModeIndex < mMode.size());
    return mMode[mCurrentModeIndex]->GetModeType();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const char*
SubModeManagerBase::GetCurrentModeName() const
{
    NW_ASSERT(mCurrentModeIndex < mMode.size());
    return mMode[mCurrentModeIndex]->GetModeName();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManagerBase::SubModeManagerBase()
: mMode(Memory_GetAllocator())
, mCurrentModeIndex(0)
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManagerBase::~SubModeManagerBase()
{
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager&
SubModeManager::GetInstance()
{
    static SubModeManager instance;
    return instance;
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager::SubModeManager() : SubModeManagerBase()
{
     nw::os::IAllocator* pAllocator = Memory_GetAllocator();

#define ADD_SUBMODE( type ) \
{ \
    void* buf = pAllocator->Alloc(sizeof(type##SubMode)); \
    type##SubMode* mode = new(buf) type##SubMode(); \
    mMode.push_back( mode ); \
} \

    ADD_SUBMODE( Main )
    ADD_SUBMODE( Menu )
    ADD_SUBMODE( Info )
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
SubModeManager::~SubModeManager()
{
    nw::os::IAllocator* pAllocator = Memory_GetAllocator();

    NW_FOREACH(ISubMode* mode, mMode)
    {
        mode->~ISubMode();
        pAllocator->Free(mode);
    }

    mMode.clear();
}


}   /* namespace g2d */
}   /* namespace vw  */
