/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_GRID_H_
#define NW4C_VIEWER_G3D_GRID_H_

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw  {
namespace g3d {


//--------------------------------
// Grid class
//--------------------------------
class Grid
{
//------- Public members ------------
public:
    /* ctor */  Grid();

    void               CreateBuffer( f32 scale, s32 division );
    void               Draw( nw::gfx::Camera* pCamera, nw::gfx::Camera* pViewCamera );

    f32                GetScale( void ) const                          { return mScale; }
    u32                GetDivision( void ) const                       { return mDivision; }

    void               SetMeshColor( const nw::ut::FloatColor& color ) { mMeshColor = color; }
    nw::ut::FloatColor GetMeshColor( void ) const                      { return mMeshColor; }

    void               SetLineWidth( f32 width )                       { mLineWidth = width; }
    f32                GetLineWidth( void ) const                      { return mLineWidth; }

private:
    void               CreateGrid( f32 scale, s32 division, f32 lineWidth );

//------- Private members ------------
private:
    f32                 mScale;     // Gradated scale
    u32                 mDivision;  // Number of gradation divisions
    nw::ut::FloatColor  mMeshColor; // Grid color

    vw::system::XYZTC   mGridModel;
    bool                mInit;

    f32                 mLineWidth;
    f32                 mDistanceFromCamera;

};



}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif


/* NW4C_VIEWER_G3D_GRID_H_ */
#endif
