/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_MENU_SUB_MODE_H_
#define NW4C_VIEWER_G3D_MENU_SUB_MODE_H_

#include "SubMode.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw {
namespace g3d {

#define DECL_MENU( type, base ) \
class type##Menu : public base \
{ \
public: \
    type##Menu(); \
    virtual ~type##Menu(); \
    virtual void Prev(); \
    virtual void Next(); \
    virtual void Apply(); \
    virtual const char* GetMenuName() const; \
    virtual MenuType GetMenuType() const; \
};

// Menu type
enum MenuType {
    Menu_ScreenCapture,
    Menu_LoadBinary,
    Menu_Sort,
    Menu_FrameRate,
    Menu_DepthLevel,
    Menu_Factor,
    Menu_RealSwitch,
    Menu_GridScale,
    Menu_GridDivision,
    Menu_ScreenPreview,
    Menu_TargetCameraAxis,
    Menu_ParticleEmitterAxisForm,
    Menu_ConstraintTargetBoneAxis,
    Menu_BoundingBox,
    Menu_ForceUseCameraWScale,
    Menu_CameraNear,
    Menu_CameraFar,
    Menu_FrameSkip,
    Menu_FocusScale,
    Menu_SettingReload,
    Menu_GridLineWidth,
    Menu_ManipulatorScale,
    Menu_DepthLevelBias,
    Menu_DollyThreshold,
    Menu_CameraManipulatorScale,
    Menu_DepthLevelSurface,
    Menu_DepthLevelSurfaceAlpha,
    Menu_CategorySystem,
    Menu_Category3D_General,
    Menu_Category3D_Information,
    Menu_Category3D_Camera,
    Menu_Category3D_Stereovision,
    Menu_CategoryEffect
};

// Menu interface declaration
class IMenu
{
public:
    IMenu() {};
    virtual ~IMenu() {};
    virtual void Prev() = 0;
    virtual void Next() = 0;
    virtual void Apply() = 0;
    virtual const char* GetMenuName() const = 0;
    virtual MenuType GetMenuType() const = 0;
};

// Menu instance declaration
DECL_MENU( ScreenCapture, IMenu )
DECL_MENU( LoadBinary, IMenu )
DECL_MENU( FrameRate, IMenu )
DECL_MENU( DepthLevel, IMenu )
DECL_MENU( Factor, IMenu )
DECL_MENU( RealSwitch, IMenu )
DECL_MENU( GridScale, IMenu )
DECL_MENU( GridDivision, IMenu )
DECL_MENU( ScreenPreview, IMenu )
DECL_MENU( TargetCameraAxis, IMenu )
DECL_MENU( ParticleEmitterAxisForm, IMenu )
DECL_MENU( ConstraintTargetBoneAxis, IMenu )
DECL_MENU( BoundingBox, IMenu )
DECL_MENU( ForceUseCameraWScale, IMenu )
DECL_MENU( CameraNear, IMenu )
DECL_MENU( CameraFar, IMenu )
DECL_MENU( FrameSkip, IMenu )
DECL_MENU( FocusScale, IMenu )
DECL_MENU( SettingReload, IMenu )
DECL_MENU( GridLineWidth, IMenu )
DECL_MENU( ManipulatorScale, IMenu )
DECL_MENU( DepthLevelBias, IMenu )
DECL_MENU( DollyThreshold, IMenu )
DECL_MENU( CameraManipulatorScale, IMenu )
DECL_MENU( DepthLevelSurface, IMenu )
DECL_MENU( DepthLevelSurfaceAlpha, IMenu )
DECL_MENU( CategorySystem, IMenu )
DECL_MENU( Category3D_General, IMenu )
DECL_MENU( Category3D_Information, IMenu )
DECL_MENU( Category3D_Camera, IMenu )
DECL_MENU( Category3D_Stereovision, IMenu )
DECL_MENU( CategoryEffect, IMenu )

// Sort menu
class SortMenu : public IMenu
{
public:
    SortMenu();
    virtual ~SortMenu();
    virtual void Prev();
    virtual void Next();
    virtual void Apply();
    virtual const char* GetMenuName() const;
    virtual MenuType GetMenuType() const;
protected:
    nw::gfx::ISceneUpdater::RenderSortMode mSortMode;
};

// Menu sub-mode
class MenuSubModeBase : public ISubMode
{
public:
    MenuSubModeBase();
    virtual ~MenuSubModeBase();
    virtual void Main(vw::system::HidStatus& hidStatus);
    virtual void Draw();
    virtual const char* GetModeName() const;
    virtual SubModeType GetModeType() const;

protected:
    nw::ut::MoveArray<IMenu*> mMenu;
    int mCurrentMenuIndex;
};

// Menu sub-mode
class MenuSubMode : public MenuSubModeBase
{
public:
    MenuSubMode();
    virtual ~MenuSubMode();
};

}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif

#endif
