/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_MODULE_G3D_IMPL_H_
#define NW4C_VIEWER_G3D_MODULE_G3D_IMPL_H_

#include <viewer/system/Hid.h>
#include <viewer/g3d/ModuleG3d.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw {
namespace g3d {

class ModuleG3dImpl : public ModuleG3d
{
public:
    // Ctor
    ModuleG3dImpl();
    // Dtor
    ~ModuleG3dImpl();

    // Init
    virtual void    Init( void );

    virtual void    InitMenu( void );

    // Init Graphics
    virtual void    InitGraphics( void );

    // PollStream
    virtual void    PollStream( void );

    // Main
    virtual void    Main(vw::system::HidStatus& hidStatus);

    // Calc
    virtual void    Calc( void );

    // Draw
    virtual void    Draw( void );

    // DrawInfo
    virtual void    DrawInfo( void );

    // Destroy
    virtual void    Destroy( void );

    // OnModuleFocused
    virtual void    OnModuleFocused( void );


    // Command for playback all is executed
    virtual void        OnAnmCmd( u32   /* command    */,     // Command (RESET/PLAY/REV_PLAY/STOP/STEP/REV_STEP/LOOP/SET_FRAME)
                                  u32   /* target     */,     // Target (ALL/3D_MDL/2D_LYT)
                                  u32   /* targetID   */,     // Target ID (do not use when target is ANM_TGT_ALL)
                                  void* /* arg        */,     // Option parameters
                                  u32   /* trigModule */ );   // Module ID that is the trigger

    // <CODE>Available</CODE>
    virtual bool    Available( void ) { return true; }

    // SetCanDraw
    void            SetCanDraw( bool draw ) { m_CanDraw = draw; }

    // SetHidden
    static void     SetHidden( bool hide ) { sHiddenFlag = hide; }

    // Get3DEditorActiveChannel
    int             Get3DEditorActiveChannel( void ) { return m_3DEditorActiveChannel; }

    // Set3DEditorActiveChannel
    void            Set3DEditorActiveChannel( int channel ) { m_3DEditorActiveChannel = channel; }

    // Get3DEditorPassiveChannel
    int             Get3DEditorPassiveChannel( void ) { return m_3DEditorPassiveChannel; }

    // Set3DEditorPassiveChannel
    void            Set3DEditorPassiveChannel( int channel ) { m_3DEditorPassiveChannel = channel; }

    // Get3DEditorSurvialChannel
    int             Get3DEditorSurvivalChannel( void ) { return m_3DEditorSurvivalChannel; }

    // Set3DEditorSurvialChannel
    void            Set3DEditorSurvivalChannel( int channel ) { m_3DEditorSurvivalChannel = channel; }

    // GetCSconnect
    bool            GetCSconnect( void ) { return m_CSconnect; }

    // SetCSconnect
    void            SetCSconnect( bool connect ) { m_CSconnect = connect; }

    // GetModuleG3dImpl
    static ModuleG3dImpl* GetInstance();

protected:

    // Open connection to CreativeStudio
    void            OpenCSconnection( void );

    // instance
    static ModuleG3dImpl* m_pIntance;

    // Hidden Flag
    static bool sHiddenFlag /* = true */;

    // Serial Channel
    nn::hio::CTR::SerialChannel m_Serial;

    // Serial Work Memory
    void* m_pWorkMemory;

    // CanDraw
    bool m_CanDraw;

    // 3DEditorActiveChannel
    int m_3DEditorActiveChannel;

    // 3DEditorPassiveChannel
    int m_3DEditorPassiveChannel;

    // 3DEditorSurvialChannel
    int m_3DEditorSurvivalChannel;

    // CSconnect
    bool m_CSconnect;

    // is connection to CreativeStudio open ?
    bool m_CSconnectionOpen;

    friend class ModuleG3d;
};


}   /* namespace g3d */
}   /* namespace vw  */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif
