/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_G3D_PARTICLE_ANIM_CONTROLLER_H_
#define NW4C_VIEWER_G3D_PARTICLE_ANIM_CONTROLLER_H_

#include "AnimController.h"
#include "ParticleSceneElem.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw {
namespace g3d {

class ParticleModelElem;

class ParticleAnimController : public IAnimController
{
public:
    // Frame advance.
    virtual void StepForward();
    // Frame reverse
    virtual void StepBackward();
    // Play/Pause
    virtual void PauseAndPlay();
    virtual void Pause();
    virtual void Play();
    // Update
    virtual void Update();
    // Increase/decrease playback speed
    virtual void SpeedUp(f32 step);
    // Stop (pause playback, move frame to 0)
    virtual void Stop();
    // Resets.
    virtual void Reset();
    // Set/get frame
    virtual void SetFrame(f32 frame);
    virtual f32 GetFrame();
    // Set/get playback rate
    virtual void SetStepFrame(f32 step);
    virtual f32 GetStepFrame(){ return m_StepFrame; }
    // Animation enable/disable switch
    virtual bool IsActive(){ return m_IsActive; }

    class Builder
    {
    public:
        Builder() : m_pParticleModelElem(NULL) {}

        ParticleAnimController*
        Create(nw::os::IAllocator *pAllocator)
        {
            NW_NULL_ASSERT( m_pParticleModelElem );
            void *buf = pAllocator->Alloc(sizeof(ParticleAnimController));
            if (buf == NULL) return NULL;
            return new(buf) ParticleAnimController( m_pParticleModelElem, pAllocator );
        }

        Builder&
        ParticleElem(ParticleModelElem *pParticleModelElem)
        {
            m_pParticleModelElem = pParticleModelElem;
            return *this;
        }

    private:
        ParticleModelElem *m_pParticleModelElem;
    };

    void Destroy()
    {
        this->~ParticleAnimController();
        m_pAllocator->Free( this );
    }

    void SetEmissionStartFrame(s32 frame){ m_EmissionStartFrame = frame; }
    s32  GetEmissionStartFrame(){ return m_EmissionStartFrame; }
    void SetEmissionLength(s32 length);

    virtual void OnTargetModuleChanged(u16 id);

protected:
    ParticleModelElem *m_pParticleModelElem;
    //
    bool m_IsActive;
    //
    f32 m_StepFrame;
    //
    nw::gfx::ParticleContext *m_pParticleContext;
    //
    bool m_EmissionControlActive;

    nw::os::IAllocator *m_pAllocator;

    nw::gfx::ParticleModel *m_pParticleModel;

    s32 m_EmissionStartFrame;
    s32 m_EmissionLength;

    ParticleAnimController( ParticleModelElem *pParticleModelElem, nw::os::IAllocator *pAllocator );

    ~ParticleAnimController();
};

}
}
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif

#endif
