/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/AnimSndUtils.h"
#include "../include/SceneManager.h"
#include "../include/ResourceManager.h"

namespace vw {
namespace g3d {


/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
LoadFile( const void *filename, u32 hashID, bool isWChar )
{
    ResourceManager::GetInstance()->LoadBinaryData(
        ResourceSet::Type_AnimSnd,
        hashID,
        0xFFFF,
        filename,
        isWChar
    );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
RemoveFile( u32 hashID )
{
    ResourceManager::GetInstance()->DeleteResource(
        ResourceSet::Type_AnimSnd,
        hashID
    );
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
u32
GetModelCount()
{
    return SceneManager::GetInstance()->GetModelsCount();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
u32
GetAnimCount()
{
    return SceneManager::GetInstance()->GetAnimsCount();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
nw::gfx::Model*
GetModel( u32 index )
{
    // Index check
    if ( index >= GetModelCount() ) return NULL;

    // Gets the start iterator
    SceneManager::ModelList::iterator it =
        SceneManager::GetInstance()->GetModelsBegin();

    // Iterator progresses for index-portion only
    for ( u32 i = 0; i < index; ++i ) { ++it; }

    return it->GetSceneNode();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
nw::anim::res::ResAnim
GetAnim( u32 index )
{
    // Index check
    if ( index >= GetAnimCount() ) return nw::anim::res::ResAnim();

    // Gets the start iterator
    SceneManager::AnimList::iterator it =
        SceneManager::GetInstance()->GetAnimsBegin();

    // Iterator progresses for index-portion only
    for ( u32 i = 0; i < index; ++i ) { ++it; }

    return it->GetResObject();
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void BindCurrentAnim(
    nw::gfx::Model* model,
    nw::ut::MoveArray<nw::anim::res::ResAnim>& anims,
    const char* animSetName
) {
    // Gets iterator.
    SceneManager::ModelList::iterator it =
        SceneManager::GetInstance()->FindModel( model );

    // Returns immediately if it is the final iterator
    if ( it == SceneManager::GetInstance()->GetModelsEnd() ) return;

    // Releases current animation
    it->GetAnimLinker()->Clear();

    // Creates animation set array
    nw::ut::MoveArray<AnimElem*> animSet( Memory_GetAllocator() );

    // Go around input animation
    for ( u32 i = 0; i < anims.size(); ++i )
    {
        // Searches for corresponding animation element
        SceneManager::AnimList::iterator anim_it =
            SceneManager::GetInstance()->FindAnim( anims[ i ] );

        // Checks if final iterator
        if ( anim_it != SceneManager::GetInstance()->GetAnimsEnd() )
        {
            animSet.push_back( &(*anim_it) );
        }
    }

    // Checks if empty array
    if ( animSet.size() != 0 )
    {
        // Registers animation set
        it->GetAnimLinker()->Group( animSetName, animSet );

        // Switches to the registered animation set
        it->GetAnimLinker()->ChangeTo( animSetName );
    }
}

/*---------------------------------------------------------------------------*
  Name:

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
SetFrame( nw::gfx::Model* model, f32 frame )
{
    // Gets iterator.
    SceneManager::ModelList::iterator it =
        SceneManager::GetInstance()->FindModel( model );

    // Sets frame if not final iterator
    if ( it != SceneManager::GetInstance()->GetModelsEnd() )
    {
        it->GetAnimController()->SetFrame( frame );
    }
}


}
}
