/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <cstring>
#include <nn.h>
#include <nw/dev.h>
#include <nn/hio/CTR/hio_Api.h>
#include <nn/hio/CTR/hio_HostFile.h>
#include <nn/hio/CTR/hio_SerialChannel.h>
#include <viewer/system/Capture.h>
#include "../include/ResourceManager.h"
#include "../../system/include/ConvertCharEncoding.h"
#include "../../system/include/LogUtil.h"
#include "../../effect/include/ModuleEffectImpl.h"

namespace vw {
namespace g3d {

    const int   MAX_COMMAND_STRING = 8;
    const char* COMMAND_RELOAD = "reload";
    const char* COMMAND_CAPTURE = "capture";

    //----------------------------------------
    //
    struct Chunk
    {
        char8    tag[4];                                //
        u16 totalSize;                    //
        u8 headSize;                    //
        u8 pad;
    };

    //----------------------------------------
    //
    struct PacketHeader : public Chunk
    {
        u32    command[ MAX_COMMAND_STRING / sizeof(u32) ];        //
    };

    void
    ProtocolExecCS_Main( nn::hio::CTR::SerialChannel &serial )
    {
        nn::Result result;

        result = serial.WaitHost(nn::hio::CTR::SerialChannel::ATTRIBUTE_NO_WAIT);
        if ( result == nn::hio::CTR::ResultConnected() )
        {
            char buffer[1024];
            memset(buffer, 0, sizeof(buffer));

            s32 readsize = 0;
            do {
                result = serial.Read((size_t*)&readsize, (void*)buffer, (size_t)1024, nn::hio::CTR::SerialChannel::ATTRIBUTE_NO_WAIT );
                if ( result.IsFailure() )
                {
                    NW_VIEWER_DEV_LOG("SIO Read Failed.[%d bytes]\n", readsize);
                    serial.Disconnect();
                    return;
                }
            } while ( readsize == 0 );

            s32 sioReadResult = 1;
            serial.Write(&sioReadResult, sizeof(s32), nn::hio::CTR::SerialChannel::ATTRIBUTE_NONE);

            result = serial.Flush();
            if ( result.IsFailure() )
            {
                NW_VIEWER_DEV_LOG("SIO Flush Failed.\n");
            }

            result = serial.Disconnect();
            if ( result.IsFailure() )
            {
                NW_VIEWER_DEV_LOG("SIO Disconnect Failed.\n");
            }

            PacketHeader *packet = (PacketHeader*)buffer;

            if (strcmp(packet->tag, "CMD") == 0)
            {
                vw::effect::ModuleEffectImpl::GetInstance()->ResetMaxParticle();
                if (strcmp((const char*)packet->command, COMMAND_RELOAD) == 0)
                {
                    ResourceManager::GetInstance()->LoadBinaryData(
                        ResourceSet::Type_Packed,
                        0,
                        0,
                        (wchar_t*)(buffer+packet->headSize)
                    );
                }
                else if (strcmp((const char*)packet->command, COMMAND_CAPTURE) == 0)
                {
                    char szName[260];
                    vw::system::ConvertStrWtoUTF8(
                        (wchar_t*)(buffer+packet->headSize),
                        szName,
                        260
                    );
                    vw::system::GetCapture().Capture(szName);
                }
            }
            else
            {
                NW_VIEWER_DEV_LOG("Unknown command.\n");
            }
        }
    }
}   /* namespace g3d */
}   /* namespace vw  */

