﻿/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw4r/viewer/g3d/Module3D.h>
#include <viewer/system.h>
#include "g3d_private.h"
#include "if_Listener.h"

namespace nw     {
namespace viewer {
namespace g3d    {


/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyReloadMdl

  Description:

  Arguments:    modelID

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyReloadMdl( u32 modelID )
{
    ModelIter mdl;

    mdl.mResMdl = vw::g3d::GetResource3D().GetModelById( modelID );
    mListener->OnUpdateModel( mdl );
}

/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyReloadScn

  Description:

  Arguments:    sceneID

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyReloadScn( u32 sceneID )
{
    SceneIter scn;

    scn.mResScn = vw::g3d::GetResource3D().GetScnById( sceneID );
    mListener->OnUpdateScene( scn );
}

/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyReloadAnm

  Description:

  Arguments:    anmID
                type

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyReloadAnm( u32 modelID, u32 actID, u8 type )
{
    ActionIter act;
    vw::g3d::VwResMdl* resMdl = vw::g3d::GetResource3D().GetModelById( modelID );

    IResourceListener::AnmType anmType;
    switch ( type )
    {
    case vw::g3d::VwResAct::ANIM_VIA: { anmType = IResourceListener::ANMTYPE_VIS;    } break;
    case vw::g3d::VwResAct::ANIM_CLA: { anmType = IResourceListener::ANMTYPE_CLR;    } break;
    case vw::g3d::VwResAct::ANIM_TPA: { anmType = IResourceListener::ANMTYPE_TEXPAT; } break;
    case vw::g3d::VwResAct::ANIM_TSA: { anmType = IResourceListener::ANMTYPE_TEXSRT; } break;
    case vw::g3d::VwResAct::ANIM_SHA: { anmType = IResourceListener::ANMTYPE_SHP;    } break;
    case vw::g3d::VwResAct::ANIM_CHA: { anmType = IResourceListener::ANMTYPE_CHR;    } break;
    default:
        NW4R_FATAL_ERROR("Unknown animtype\n");
    }

    act.mResMdl = resMdl;
    act.mResAct = resMdl->GetActionById( actID );
    mListener->OnUpdateAction( act, anmType );
}


/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyRemoveMdl

  Description:

  Arguments:    modelID

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyRemoveMdl( u32 modelID )
{
    ModelIter mdl;

    mdl.mResMdl = vw::g3d::GetResource3D().GetModelById( modelID );
    mListener->OnRemoveModel( mdl );
}

/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyRemoveScn

  Description:

  Arguments:    sceneID

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyRemoveScn( u32 sceneID )
{
    SceneIter scn;

    scn.mResScn = vw::g3d::GetResource3D().GetScnById( sceneID );
    mListener->OnRemoveScene( scn );
}

/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyRemoveAnm

  Description:

  Arguments:    anmID
                type

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyRemoveAnm( u32 modelID, u32 actID, u8 type )
{
    ActionIter act;
    vw::g3d::VwResMdl* resMdl = vw::g3d::GetResource3D().GetModelById( modelID );
    IResourceListener::AnmType anmType;
    switch ( type )
    {
    case vw::g3d::VwResAct::ANIM_VIA: { anmType = IResourceListener::ANMTYPE_VIS;    } break;
    case vw::g3d::VwResAct::ANIM_CLA: { anmType = IResourceListener::ANMTYPE_CLR;    } break;
    case vw::g3d::VwResAct::ANIM_TPA: { anmType = IResourceListener::ANMTYPE_TEXPAT; } break;
    case vw::g3d::VwResAct::ANIM_TSA: { anmType = IResourceListener::ANMTYPE_TEXSRT; } break;
    case vw::g3d::VwResAct::ANIM_SHA: { anmType = IResourceListener::ANMTYPE_SHP;    } break;
    case vw::g3d::VwResAct::ANIM_CHA: { anmType = IResourceListener::ANMTYPE_CHR;    } break;
    default:
        NW4R_FATAL_ERROR("Unknown animtype\n");
    }

    act.mResMdl = resMdl;
    act.mResAct = resMdl->GetActionById( actID );
    mListener->OnRemoveAction( act, anmType );
}


/*---------------------------------------------------------------------------*
  Name:         ResManager::AppendUpdateMat

  Description:

  Arguments:    modelID
                matID

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::AppendUpdateMat( u32 modelID, u32 matID )
{
    for ( MatIDVec::iterator it = mMatIDs.begin(); it != mMatIDs.end(); ++ it )
    {
        if ( it->mdlID == modelID && it->matID == matID ) { return; }
    }
    mMatIDs.push_back( MatID( modelID, matID ) );
}


/*---------------------------------------------------------------------------*
  Name:         ResManager::NotifyUpdateMat

  Description:

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
void
ResManager::NotifyUpdateMat()
{
    for ( MatIDVec::iterator it = mMatIDs.begin(); it != mMatIDs.end(); ++it )
    {
        ModelIter    mdl;
        MaterialIter mat;

        mdl.mResMdl = vw::g3d::GetResource3D().GetModelById( it->mdlID );

        mat.mResMdl = mdl.mResMdl;
        mat.mIdx    = s32(it->matID);

        mListener->OnUpdateMaterial( mdl, mat );
    }

    mMatIDs.clear();
}

} /* namespace g3d    */
} /* namespace viewer */
} /* namespace nw4r   */





