/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_BITMAP_DATA_H_
#define NW4C_VIEWER_SYSTEM_BITMAP_DATA_H_

#include "ImageFmt.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 2530 // padding added to end of struct.
#endif
namespace vw     {
namespace system {

class BitmapData : public ImageFmt
{
//--- Public members --------------
public:
    typedef struct
    {
        u8  bfType[2];      // File type "BM"
        u16 bfSize[2];      // File size.
        u16 bfReserved1;    // Reserved region always 0
        u16 bfReserved2;    // Reserved region always 0
        u16 bfOffBits[2];   // Offset from start of file to image data
        u16 bcSize[2];      // Information header size
        union
        {
            struct
            {
                u16 bcWidth[2];     // Image width (pixels)
                u16 bcHeight[2];    // Image height (pixels)
                u16 bcPlanes;       // Number of planes, always 1
                u16 bcBitCount;     // data size (bit) of one pixel
                u16 compMode[2];    // Compression format (0 - no compression, 1 - Runlength 8 bits/pixel, 2 - Runlength 4 bits/pixel, 3 - Bitfields)
                u16 dataSize[2];    // Size of image data portion (bytes)
                u16 hPpm[2];        // Horizontal resolution (number of pixels per 1m), use 0
                u16 vPpm[2];        // Vertical resolution (number of pixels per 1m), use 0
                u16 pltNum[2];      // Number of stored palettes; 0 if no palettes used
                u16 impPltNum[2];   // Index of important palette, use 0
            };
            struct
            {
                u16 bcWidth;
                u16 bcHeight;
                u16 bcPlanes;
                u16 bcBitCount;
            } os2;
        };
    } Header;

    /* ctor */          BitmapData( u32 width, u32 height, u8 bpp = 3 );
    /* dtor */ virtual  ~BitmapData( void );

    virtual u32         GetBodySize() const   { return mWidth * mHeight * mBpp; }

    virtual u8*         GetHeader()           { return reinterpret_cast<u8*>( &mHeader ); }
    virtual u32         GetHeaderSize() const { return sizeof(Header); }

    virtual void        ConvertData( void );    // Convert RGBA data in buffer to match format.
    virtual void        ConvertData( void* src_image, u32 width, u32 height, u32 offset_x, u32 offset_y );
    virtual void        ConvertData( void* src_image,
                                     u32 src_width, u32 src_height,
                                     u32 dst_ofs_x, u32 dst_ofs_y,
                                     u32 src_ofs_x, u32 src_ofs_y,
                                     u32 width,     u32 height );

//--- Private members--------------
private:

    Header mHeader;
};






}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 2530 // padding added to end of struct.
#endif



/* NW4C_VIEWER_SYSTEM_BITMAP_DATA_H_ */
#endif
