/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_BMP_READER_IMPL_H_
#define NW4C_VIEWER_SYSTEM_BMP_READER_IMPL_H_

#include "ImageReader.h"

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw     {
namespace system {

class BmpReaderImpl : public ImageReaderImpl
{
//--- Public members ------
public:
    /* ctor */          BmpReaderImpl( void* data );
    /* dtor */ virtual  ~BmpReaderImpl();

    virtual bool        IsValid();
    virtual u32         GetWidth();
    virtual u32         GetHeight();
    virtual bool        Expand( void* expBuf, u32 bufWidth, u32 bufHeight );
    virtual u32         GetType();
    virtual void        Dispose();

//--- Private members ------
private:
    u8*   mBuf;
    u8*   mBody;
    u8*   mPltt;
    u32   mWidth;
    u32   mHeight;
    bool  mIsValid;
    u32   mBpp;
    u32   mCompressType;
    u32   mPlttBpp;

    enum
    {
        COMP_TYPE_NONE,
        COMP_TYPE_RL8,
        COMP_TYPE_RL4,
        COMP_TYPE_BITFIELDS
    };

    bool  CheckValid_();
    void  Uncomp_( u8* pOut );
    void  UncompRL8_( u8* pOut );
    void  UncompRL4_( u8* pOut );
    void  UncompBF_ ( u8* pOut );
};



}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif



/* NW4C_VIEWER_SYSTEM_BMP_READER_IMPL_H_ */
#endif
