/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4R_VIEWER_SYSTEM_CONVERT_CHAR_ENCODING_H_
#define NW4R_VIEWER_SYSTEM_CONVERT_CHAR_ENCODING_H_

namespace vw
{
namespace system
{

/*
  

  
  
  

  
 */
inline const char*
ConvertStrWtoUTF8(
    const wchar_t* szSrc,
    char *szTempBuf,
    int tempBufMaxSize )
{
    if ( szSrc == NULL )
    {
        szTempBuf[0] = 0;
        return szTempBuf;
    }

    int i = 0;
    int index = 0;
    wchar_t ch = szSrc[i];


    while ( ch != L'\0' )
    {
        NW_ASSERT( ch != 0xFFFF );

        int length = 0;
        if ( ch < (wchar_t)0x0080 )
        {
            length = 1;
        }
        else if ( ch < (wchar_t)0x0800 )
        {
            length = 2;
        }
        else if ( ch < (wchar_t)0xFFFF )
        {
            length = 3;
        }

        if ( index + length - 1 >= tempBufMaxSize )
        {
            break;
        }

        switch (length)
        {
        case 1:
            szTempBuf[index] = (char)ch;
            break;
        case 2:
            szTempBuf[index ] = (char) ((ch >> 6) | 0xC0);
            szTempBuf[index + 1] = (char) ((ch & 0x3F) | 0x80);
            break;
        case 3:
            szTempBuf[index ] = (char) ((ch >> 12) | 0xE0);
            szTempBuf[index + 1] = (char) (((ch >> 6) & 0x3F ) | 0x80);
            szTempBuf[index + 2] = (char) ((ch & 0x3F) | 0x80);
            break;
        default:
            break;
        }

        index += length;

        ++i;
        ch = szSrc[i];
    }

    szTempBuf[index] = 0;

    return szTempBuf;
}

}
}
#endif
