/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_IMAGE_FMT_H_
#define NW4C_VIEWER_SYSTEM_IMAGE_FMT_H_

#include <viewer.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw     {
namespace system {

class ImageFmt
{
public:

    // Operation method when setting color or alpha information in an image
    typedef enum
    {
        OPE_SETVALUE = 0,   // Substitute
        OPE_ADD,            // Addition
        OPE_DEC,            // Subtraction
        OPE_NOTDEC          // Subtract and then invert
    } Operation;

    /* ctor */          ImageFmt( u32 width, u32 height, u8 bpp ) : mWidth( width ), mHeight( height ), mBpp( bpp ), mBody( NULL ) {}

    /* dtor */  virtual ~ImageFmt( void )     { if ( mBody ) { Memory_Free( mBody ); } }

    u8*                 GetBody() { return reinterpret_cast<u8*>( mBody ); }
    virtual u32         GetBodySize() const   = 0;

    virtual u8*         GetHeader()           = 0;
    virtual u32         GetHeaderSize() const = 0;

    u32                 GetWidth()  const     { return mWidth;  }
    u32                 GetHeight() const     { return mHeight; }

    virtual void        ConvertData( void )   = 0;    // Convert RGBA data in buffer to match format.
    virtual void        ConvertData( void* src_image, u32 src_width, u32 src_height, u32 offset_x, u32 offset_y ) = 0;
    virtual void        ConvertData( void* src_image,
                                     u32 src_width, u32 src_height,
                                     u32 dst_ofs_x, u32 dst_ofs_y,
                                     u32 src_ofs_x, u32 src_ofs_y,
                                     u32 width,     u32 height ) = 0;

    virtual void        ConvertAlphaData( void ) {}    // Convert RGBA data in buffer to match format.
    virtual void        ConvertAlphaData( void* /* src_image */,
                                          u32 /* width     */, u32 /* height    */,
                                          u32 /* dst_ofs_x */, u32 /* dst_ofs_y */,
                                          Operation /* op */ ) {}
    virtual void        ConvertAlphaData( void* /* src_image */, // Converts RGBA data in buffer to match format, and applies alpha only.
                                          u32 /* src_width */, u32 /* src_height */,
                                          u32 /* dst_ofs_x */, u32 /* dst_ofs_y  */,
                                          u32 /* src_ofs_x */, u32 /* src_ofs_y  */,
                                          u32 /* width     */, u32 /* height     */,
                                          Operation /* op */ ) {}

    virtual void        Finalize( void ) {}

    bool                IsValid() const { return (mBody != NULL)? true : false; }

protected:
    u32         mWidth;     // Image width
    u32         mHeight;    // Image height
    u8          mBpp;       // Data size surrounding a bit
    void        *mBody;     // Buffer
};






}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif



/* NW4C_VIEWER_SYSTEM_IMAGE_FMT_H_ */
#endif
