/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#ifndef NW4C_VIEWER_SYSTEM_LOG_SYSTEM_H_
#define NW4C_VIEWER_SYSTEM_LOG_SYSTEM_H_

#include <viewer.h>

#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_suppress 1301 // padding inserted in struct.
#endif
namespace vw     {
namespace system {

/*===========================================================================*
  Name:         LogSystem

  Description:  Log management class
 *===========================================================================*/
class LogSystem
{
public:
    static LogSystem* GetInstance();
    static void Initialize();
    static void Finalize();

    enum TYPE
    {
        ERROR = 0,
        WARNING,
        SYSTEM_LOG,
        HISTORY
    };

    struct Log
    {
        TYPE m_Type;
        CharArray m_Message;
        u32 m_HashID;
        void* m_MainKey;
        u32 m_SubKey;
        u32 m_Index;
        Log();
        ~Log();
    };

    void AddError(const char* message, u32 hashID, void* mainKey, u32 subKey = 0);
    void AddWarning(const char* message, u32 hashID, void* mainKey, u32 subKey = 0);
    void AddSystemLog(const char* message);
    u32  GetNumLog() const;
    u32  GetNumError() const;
    u32  GetNumWarning() const;
    u32  GetNumSystemLog() const;
    Log* GetLog(u32 index);
    Log* GetError(u32 index);
    Log* GetWarning(u32 index);
    Log* GetSystemLog(u32 index);

    bool IsExist(TYPE type, void* mainKey, u32 subKey = 0);

    void Clear();
    void Clear(TYPE type);
    void Clear(u32 hashID);
    void Clear(void* mainKey);
    void Clear(void* mainKey, u32 subKey);
    void OutputLog(const char* filename, TYPE type);

private:
    /* ctor */              LogSystem();
    /* dtor */              ~LogSystem();
    void                    ResetIndex();

private:
    static LogSystem* m_pInstance;
    nw::ut::MoveArray<Log*> m_Log;
    nw::ut::MoveArray<u32>  m_Error;
    nw::ut::MoveArray<u32>  m_Warning;
    nw::ut::MoveArray<u32>  m_SystemLog;
};

static inline LogSystem*
GetLogSystem()
{
    return LogSystem::GetInstance();
}

}   /* namespace system */
}   /* namespace vw     */
#if NN_CURRENT_VERSION_NUMBER >= NN_VERSION_NUMBER(4,0,0,0)
#pragma diag_default 1301 // padding inserted in struct.
#endif



/* NW4C_VIEWER_SYSTEM_LOG_SYSTEM_H_ */
#endif
