/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <viewer/system.h>
#include "../include/HidManager.h"

namespace vw {
namespace system {


/*---------------------------------------------------------------------------*
  Name:         GetSubModePadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetSystemPadBindings()
{
    static PadBinding bindingArray[] =
    {
        { SYS_SWITCH_MODE_PREV, HLD_Start | TRG_Left },
        { SYS_SWITCH_MODE_NEXT, HLD_Start | TRG_Right },
        { SYS_SWITCH_MODE_PREV, HLD_Start | TRG_Left  | HLD_R },
        { SYS_SWITCH_MODE_NEXT, HLD_Start | TRG_Right | HLD_R },
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetG3dPadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetG3dPadBindings()
{
    static PadBinding bindingArray[] =
    {
        { G3D_START_ANIMATION,                  HLD_LR | TRG_A },
        { G3D_STOP_ANIMATION,                   HLD_LR | TRG_B },
        { G3D_SWITCH_NEXT_SCENE,                HLD_LR | TRG_X },
        { G3D_SWITCH_NEXT_ANIMATION,            HLD_LR | TRG_Y },
        //{ G3D_DECREMENT_ANIMATION_FRAME,        HLD_LR | TRG_Left },
        //{ G3D_INCREMENT_ANIMATION_FRAME,        HLD_LR | TRG_Right},
        //{ G3D_SPEED_UP_ANIMATION,               HLD_LR | TRG_Up},
        //{ G3D_SLOW_DOWN_ANIMATION,              HLD_LR | TRG_Down},
        //{ G3D_DECREMENT_ANIMATION_FRAME,        HLD_LR | HLD_Left   | HLD_AWHILE},
        //{ G3D_INCREMENT_ANIMATION_FRAME,        HLD_LR | HLD_Right  | HLD_AWHILE},
        //{ G3D_SPEED_UP_ANIMATION,               HLD_LR | HLD_Up     | HLD_AWHILE},
        //{ G3D_SLOW_DOWN_ANIMATION,              HLD_LR | HLD_Down   | HLD_AWHILE},
        { G3D_SWITCH_PREV_SELECTION_ITEM_SUB,   HLD_LR  | TRG_Left},
        { G3D_SWITCH_NEXT_SELECTION_ITEM_SUB,   HLD_LR  | TRG_Right},
        //{ G3D_SWITCH_PREV_SELECTION,            HLD_LR  | TRG_Up},
        //{ G3D_SWITCH_NEXT_SELECTION,            HLD_LR  | TRG_Down},
        { G3D_SWITCH_PREV_SELECTION_ITEM_SUB,   HLD_LR  | HLD_Left   | HLD_AWHILE},
        { G3D_SWITCH_NEXT_SELECTION_ITEM_SUB,   HLD_LR  | HLD_Right  | HLD_AWHILE},
        { G3D_SWITCH_NEXT_VIEW_CAMERA,          HLD_R  | TRG_A },
        { G3D_CHANGE_INFO_RENDER_STATUS,        HLD_R  | TRG_B },
        { G3D_SWITCH_NEXT_CONTROL_TARGET_SEQUENTIALLY,   HLD_R  | TRG_X },
        { G3D_SWITCH_NEXT_RESOURCE_SEQUENTIALLY,  HLD_R  | TRG_Y },
        //{ G3D_DECREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | TRG_Down},
        //{ G3D_INCREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | TRG_Up},
        //{ G3D_SPEED_UP_ALL_ANIMATIONS,          HLD_R  | TRG_Up},
        //{ G3D_SLOW_DOWN_ALL_ANIMATIONS,         HLD_R  | TRG_Down},
        //{ G3D_DECREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | HLD_Down   | HLD_AWHILE},
        //{ G3D_INCREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | HLD_Up  | HLD_AWHILE},
        //{ G3D_SPEED_UP_ALL_ANIMATIONS,          HLD_R  | HLD_Up     | HLD_AWHILE},
        //{ G3D_SLOW_DOWN_ALL_ANIMATIONS,         HLD_R  | HLD_Down   | HLD_AWHILE},
        { G3D_SWITCH_PREV_SELECTION_ITEM_MAIN,  HLD_R  | TRG_Left},
        { G3D_SWITCH_NEXT_SELECTION_ITEM_MAIN,  HLD_R  | TRG_Right},
        { G3D_SWITCH_PREV_SELECTION,            HLD_R  | TRG_Up},
        { G3D_SWITCH_NEXT_SELECTION,            HLD_R  | TRG_Down},
        { G3D_SWITCH_PREV_SELECTION_ITEM_MAIN,  HLD_R  | HLD_Left   | HLD_AWHILE},
        { G3D_SWITCH_NEXT_SELECTION_ITEM_MAIN,  HLD_R  | HLD_Right  | HLD_AWHILE},
        { G3D_START_ALL_ANIMATIONS,                      TRG_A },
        { G3D_STOP_ALL_ANIMATIONS,                       TRG_B },
        { G3D_TOGGLE_ONETIME_LOOP,              HLD_R  | TRG_Start}
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetEffectPadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetEffectPadBindings()
{
    static PadBinding bindingArray[] =
    {
        { EF_SWITCH_VIEW_CAMERA,               HLD_R  | TRG_A },
        { EF_CHANGE_INFO_RENDER_STATUS,        HLD_R  | TRG_B },
        { EF_INCREMENT_MAX_FRAME,              HLD_R  | TRG_X },
        { EF_DECREMENT_MAX_FRAME,              HLD_R  | TRG_Y },
        { EF_CHANGE_ANIMATION_LOOP_STATUS,     HLD_R  | TRG_Left},
        { EF_INCREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | TRG_Right},
        { EF_SPEED_UP_ALL_ANIMATIONS,          HLD_R  | TRG_Up},
        { EF_SLOW_DOWN_ALL_ANIMATIONS,         HLD_R  | TRG_Down},
        { EF_INCREMENT_ALL_ANIMATION_FRAMES,   HLD_R  | HLD_Right  | HLD_AWHILE},
        { EF_SPEED_UP_ALL_ANIMATIONS,          HLD_R  | HLD_Up     | HLD_AWHILE},
        { EF_SLOW_DOWN_ALL_ANIMATIONS,         HLD_R  | HLD_Down   | HLD_AWHILE},
        { EF_START_ALL_ANIMATIONS,                      TRG_A },
        { EF_STOP_ALL_ANIMATIONS,                       TRG_B },
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetSubModePadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetSubModePadBindings()
{
    static PadBinding bindingArray[] =
    {
        { SUB_SWITCH_PREV_SUBMODE, HLD_Start | TRG_Up },
        { SUB_SWITCH_NEXT_SUBMODE, HLD_Start | TRG_Down },
        { SUB_SWITCH_PREV_SUBMODE, HLD_Start | TRG_Up   | HLD_R },
        { SUB_SWITCH_NEXT_SUBMODE, HLD_Start | TRG_Down | HLD_R },
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetMenuSubModePadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetMenuSubModePadBindings()
{
    static PadBinding bindingArray[] =
    {
        { MNU_MENU_APPLY,            TRG_A },
        { MNU_SWITCH_PREV_MENU_ITEM, TRG_Left },
        { MNU_SWITCH_NEXT_MENU_ITEM, TRG_Right },
        { MNU_SWITCH_PREV_MENU,      TRG_Up },
        { MNU_SWITCH_NEXT_MENU,      TRG_Down },
        { MNU_SWITCH_PREV_MENU_ITEM, HLD_Left  | HLD_AWHILE },
        { MNU_SWITCH_NEXT_MENU_ITEM, HLD_Right | HLD_AWHILE },
        { MNU_SWITCH_PREV_MENU,      HLD_Up    | HLD_AWHILE },
        { MNU_SWITCH_NEXT_MENU,      HLD_Down  | HLD_AWHILE },
        { MNU_SWITCH_PREV_MENU_ITEM, TRG_Left  | HLD_R },
        { MNU_SWITCH_NEXT_MENU_ITEM, TRG_Right | HLD_R },
        { MNU_SWITCH_PREV_MENU,      TRG_Up    | HLD_R },
        { MNU_SWITCH_NEXT_MENU,      TRG_Down  | HLD_R },
        { MNU_SWITCH_PREV_MENU_ITEM, HLD_Left  | HLD_AWHILE | HLD_R },
        { MNU_SWITCH_NEXT_MENU_ITEM, HLD_Right | HLD_AWHILE | HLD_R },
        { MNU_SWITCH_PREV_MENU,      HLD_Up    | HLD_AWHILE | HLD_R },
        { MNU_SWITCH_NEXT_MENU,      HLD_Down  | HLD_AWHILE | HLD_R },
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetLogSubModePadBindings

  Description:  Returns array with correspondence between pad input and processing.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
const PadBindings*
GetLogSubModePadBindings()
{
    static PadBinding bindingArray[] =
    {
        { LOG_EXPORT,                TRG_A },
        { LOG_CLEAR,                 TRG_B },
        { LOG_SWITCH_PREV_LOG,       TRG_Left },
        { LOG_SWITCH_NEXT_LOG,       TRG_Right },
        { LOG_SWITCH_PREV_LOG_ITEM,  TRG_Up },
        { LOG_SWITCH_NEXT_LOG_ITEM,  TRG_Down },
        { LOG_SWITCH_PREV_LOG,       HLD_Left  | HLD_AWHILE },
        { LOG_SWITCH_NEXT_LOG,       HLD_Right | HLD_AWHILE },
        { LOG_SWITCH_PREV_LOG_ITEM,  HLD_Up    | HLD_AWHILE },
        { LOG_SWITCH_NEXT_LOG_ITEM,  HLD_Down  | HLD_AWHILE },
        { LOG_SWITCH_PREV_LOG,       TRG_Left  | HLD_R },
        { LOG_SWITCH_NEXT_LOG,       TRG_Right | HLD_R },
        { LOG_SWITCH_PREV_LOG_ITEM,  TRG_Up    | HLD_R },
        { LOG_SWITCH_NEXT_LOG_ITEM,  TRG_Down  | HLD_R },
        { LOG_SWITCH_PREV_LOG,       HLD_Left  | HLD_AWHILE | HLD_R },
        { LOG_SWITCH_NEXT_LOG,       HLD_Right | HLD_AWHILE | HLD_R },
        { LOG_SWITCH_PREV_LOG_ITEM,  HLD_Up    | HLD_AWHILE | HLD_R },
        { LOG_SWITCH_NEXT_LOG_ITEM,  HLD_Down  | HLD_AWHILE | HLD_R },
    };

    static const PadBindings bindings
        = { bindingArray, sizeof(bindingArray) };

    return &bindings;
}

/*---------------------------------------------------------------------------*
  Name:         GetPadEvent

  Description:  Patrols the received PadBindings, and returns the one that matches the Pad input.

  Arguments:    None.

  Returns:      None.
 *---------------------------------------------------------------------------*/
s32
GetPadEvent( vw::system::HidStatus &hidStatus, const PadBindings *bindings )
{
    u32 bit = vw::system::GetHidBit( hidStatus );
    if (0 != bit)
    {
        const size_t n = bindings->size / sizeof( PadBinding );
        for (int i = 0; i < n; ++i)
        {
            // Compares the HLD_AWHILE bit only when the comparison target HLD_AWHILE bit is set.
            if (bindings->binding[i].bit & vw::system::HLD_AWHILE)
            {
                if (bit == bindings->binding[i].bit)
                {
                    return bindings->binding[i].enumurate;
                }
            }
            else
            {
                if ((bit & ~vw::system::HLD_AWHILE) == (bindings->binding[i].bit & ~vw::system::HLD_AWHILE))
                {
                    return bindings->binding[i].enumurate;
                }
            }
        }
    }
    return -1;
}

}
}
