/*---------------------------------------------------------------------------*
  Project:  NintendoWare

  Copyright (C)Nintendo/HAL Laboratory, Inc.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#include <nw/viewer/sys/Application.h>
#include <viewer/system.h>

namespace nw     {
namespace viewer {
namespace sys    {

/*---------------------------------------------------------------------------*
  Name:         App::GetInstance
  Description:  Get an instance
  Arguments:    None.
  Returns:
 *---------------------------------------------------------------------------


*/
/* static */ App&
App::GetInstance()
{
    static App sInstance;
    return sInstance;
}

/*---------------------------------------------------------------------------*
  Name:         App::App
  Description:  Constructor
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
/* ctor */
App::App()
 : mVideoSetting(),
   mHeap(),
   mModule3D(),
   mModuleEF()
{
}

/*---------------------------------------------------------------------------*
  Name:         App::SystemInit
  Description:  Initialization of the system.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::SystemInit( nw::os::IAllocator* pDeviceAllocator )
{
    vw::system::App::InitializeSystem( pDeviceAllocator );
}

/*---------------------------------------------------------------------------*
  Name:         App::Init
  Description:  Initialization
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Init()
{
    vw::system::Memory_Init();
    vw::system::App::Initialize( vw::system::Memory_GetSysAllocator() );
}

/*---------------------------------------------------------------------------*
  Name:         App::Init
  Description:  Initialization
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Init( nw::os::IAllocator* pAllocator, nw::os::IAllocator* pDeviceAllocator/*, bool useViewerVRAMManager*/ )
{
    vw::system::App::SetEmbeddedFlg( true );
    vw::system::App::SetUseMenuFlg( false );
    vw::system::App::SetUseGraphicsFlg( false );
    // If you want to use Viewer's VRAM manager, set to accept flags as follows.
    // Note that it is best not to use Viewer VRAM Manager together with ViewerLibrary because the order of initializing resources becomes complicated.
    //vw::system::App::SetUseGraphicsMemoryAllocator( useViewerVRAMManager );
    vw::system::App::SetUseGraphicsMemoryAllocator( false );

    vw::system::Memory_Init( pAllocator, pDeviceAllocator );
    vw::system::App::Initialize( pAllocator );
}

/*---------------------------------------------------------------------------*
  Name:         App::ViewerMenuInit
  Description:  Initializes the Viewer menu
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::ViewerMenuInit()
{
    vw::system::App::SetUseMenuFlg( true );
    vw::system::App::SetUseGraphicsFlg( false );
    vw::system::App::InitializeMenu();
}

/*---------------------------------------------------------------------------*
  Name:         App::ViewerGraphicsInit
  Description: Initializes Viewer graphics
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::ViewerGraphicsInit()
{
    vw::system::App::SetUseGraphicsFlg( true );
    vw::system::App::InitializeGraphics();
}

/*---------------------------------------------------------------------------*
  Name:         App::Finalize
  Description:  Finalize.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Finalize()
{
    vw::system::App::GetInstance().Finalize();
    vw::system::App::SetUseMenuFlg( false );
    vw::system::App::SetUseGraphicsFlg( false );
}

/*---------------------------------------------------------------------------*
  Name:         App::PollStream
  Description:  Polling processing of the MCS stream
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::PollStream()
{
    vw::system::App::GetInstance().PollStream();
}

/*---------------------------------------------------------------------------*
  Name:         App::Main
  Description:  Main processing
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Main()
{
    vw::system::App::GetInstance().PreMain();
    vw::system::App::GetInstance().Main();
}


/*---------------------------------------------------------------------------*
  Name:         App::UpdateFrame
  Description:  Update the animation frame
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::UpdateFrame()
{
    vw::system::App::GetInstance().UpdateFrame();
}


/*---------------------------------------------------------------------------*
  Name:         App::CalcAndDraw
  Description:  Update and render
  Arguments:    sync
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::CalcAndDraw( bool sync /* = true */ )
{
    vw::system::App::GetInstance().CalcAndDraw( sync );
}

/*---------------------------------------------------------------------------*
  Name:         App::Calc
  Description:  Frame update processing
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Calc()
{
    vw::system::App::GetInstance().Calc();
}

/*---------------------------------------------------------------------------*
  Name:         App::Draw
  Description:  Rendering
  Arguments:    sync
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::Draw( bool sync /* = true */ )
{
    vw::system::App::GetInstance().Draw( sync );
}

/*---------------------------------------------------------------------------*
  Name:         App::DrawMenu
  Description:  Menu
  Arguments:    sync
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::DrawMenu( bool sync /* = true */ )
{
    vw::system::App::GetInstance().DrawInfo();
}

/*---------------------------------------------------------------------------*
  Name:         App::WaitDrawDone
  Description:  Wait for the next frame
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::WaitNextFrame()
{
    vw::system::App::GetInstance().WaitNextFrame();
}

/*---------------------------------------------------------------------------*
  Name:         App::BeginDraw
  Description:  Makes preparations for rendering.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::BeginDraw()
{
    if( vw::system::App::GetUseGraphicsFlg() )
    {
        vw::system::Draw2DHelper::GetInstance().Begin();
    }
}

/*---------------------------------------------------------------------------*
  Name:         App::EndDraw
  Description:  Ends rendering.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::EndDraw()
{
    if( vw::system::App::GetUseGraphicsFlg() )
    {
        vw::system::Draw2DHelper::GetInstance().End();
    }
}

/*---------------------------------------------------------------------------*
  Name:         App::GetConfigString
  Description:
  Arguments:
  Returns:
 *---------------------------------------------------------------------------


*/
bool
App::GetConfigString( const char* param, const char** pValue )
{
    *pValue = vw::system::GetConfigFile()->GetString( param );
    return (*pValue != NULL);
}

/*---------------------------------------------------------------------------*
  Name:         App::GetConfigInt
  Description:
  Arguments:
  Returns:
 *---------------------------------------------------------------------------


*/
bool
App::GetConfigInt( const char* param, s32* pValue )
{
    return vw::system::GetConfigFile()->GetInt( param, pValue );
}

/*---------------------------------------------------------------------------*
  Name:         App::GetConfigUInt
  Description:
  Arguments:
  Returns:
 *---------------------------------------------------------------------------


*/
bool
App::GetConfigUInt( const char* param, u32* pValue )
{
    return vw::system::GetConfigFile()->GetUInt( param, pValue );
}

/*---------------------------------------------------------------------------*
  Name:         App::GetConfigFloat
  Description:
  Arguments:
  Returns:
 *---------------------------------------------------------------------------


*/
bool
App::GetConfigFloat( const char* param, float* pValue )
{
    return vw::system::GetConfigFile()->GetFloat( param, pValue );
}

/*---------------------------------------------------------------------------*
  Name:         App::SetConfigFilePath
  Description:  Sets the path for loading the settings file.
  Arguments:    The file path for loading the config file.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::SetConfigFilePath( const char* filePath )
{
    vw::system::ConfigFile::SetConfigFilePath( filePath );
}

/*---------------------------------------------------------------------------*
  Name:         App::GetConfigFilePath
  Description:  Gets the path for loading the settings file.
  Arguments:    None.
  Returns:      The file path for loading the config file.
 *---------------------------------------------------------------------------


*/
const char*
App::GetConfigFilePath()
{
    return vw::system::ConfigFile::GetConfigFilePath();
}

/*---------------------------------------------------------------------------*
  Name:         App::ReloadConfig
  Description:  Reloads the settings file.
  Arguments:    None.
  Returns:      None.
 *---------------------------------------------------------------------------


*/
void
App::ReloadConfig()
{
    const char* pConfigFilePath = vw::system::ConfigFile::GetConfigFilePath();

    vw::system::ConfigFile* pConfig = vw::system::GetConfigFile();
    if ( pConfig != NULL )
    {
        vw::system::GetConfigFile()->Reload( pConfigFilePath );
    }
}

} /* namespace sys    */
} /* namespace viewer */
} /* namespace nw     */
